/* $Id: samp2.cpp,v 1.3 1995/09/23 21:36:56 artc Exp $ */
// samp2.cpp
// This program demonstrates what it takes to create a custom object class.
// Essentially,  this means deriving a class from AcDbObject and providing
// your own implementations of dwgInFields, dwgOutFields, dxfInFields, and
// dxfOutFields.

#include <rxobject.h>
#include <rxregsvc.h>
#include <aced.h>
#include <dbsymtb.h>
#include <adslib.h>


// function prototypes

void create_dictionary();
void iterate_dictionary();
void initApp();
void unloadApp();
extern "C" AcRx::AppRetCode acrxEntryPoint(AcRx::AppMsgCode, void*);

                       
// class declarations

class MyClass : public AcDbObject
//
// This is a custom object class to demonstrate what's necessary to
// make it work.
//
// To keep it simple,  this class has a single integer data member.
// Get and set functions are provided for this data member.
//
{
public:
    ACRX_DECLARE_MEMBERS(MyClass);
    MyClass() : mIntval(0){};
    MyClass(const int& val): mIntval(val) {};
    Acad::ErrorStatus           getData(int&);
    Acad::ErrorStatus           setData(const int&);
    virtual Acad::ErrorStatus   dwgInFields(AcDbDwgFiler* filer);
    virtual Acad::ErrorStatus   dwgOutFields(AcDbDwgFiler* filer) const;
    virtual Acad::ErrorStatus   dxfInFields(AcDbDxfFiler* filer);
    virtual Acad::ErrorStatus   dxfOutFields(AcDbDxfFiler* filer) const;
private:
    int mIntval;
};

ACRX_DXF_DEFINE_MEMBERS(MyClass, AcDbObject, 0, MYCLASS, SAMP2);


// gets value of integer data member
//
Acad::ErrorStatus
MyClass::getData(int& val)
{
    // This is the "get" function.  "val" needs to return with the value
    // of the intval data member.  Don't forget to use assertReadEnabled(),
    // and to return a proper Acad::ErrorStatus value.
    //...

    return Acad::eOk;
}


// Sets value of integer data member
//
Acad::ErrorStatus
MyClass::setData(const int& val)
{
    // The intval data member needs to be set to the value in "val"
    // Don't forget to use assertWriteEnabled() to trigger openedForModify
    // notification,  and also to return a proper Acad::ErrorStatus value.
    //...

    return Acad::eOk;
}


// Files data in from a DWG file
//
Acad::ErrorStatus
MyClass::dwgInFields(AcDbDwgFiler* filer)
{
    // first you need to call assertWriteEnabled() to let AutoCAD know the
    // object is about to be written to and to trigger openedForModify()
    // notification.  Then you need to call the parent class's dwgInFields
    // member function then you need to read in the value for intval.
    // Finally you need to return the filer status.
    //...

    return filer->filerStatus();
}


// Files data out to a DWG file
//
Acad::ErrorStatus
MyClass::dwgOutFields(AcDbDwgFiler* filer) const
{
    // first you need to call assertReadEnabled() to let AutoCAD know the
    // object is about to be read.  Then you need to call the parent class's
    // dwgOutFields member function then you need to write out the intval
    // value.  Finally,  return the filer status
    //...

    return filer->filerStatus();
}


// Files data in from a DXF file
//
Acad::ErrorStatus
MyClass::dxfInFields(AcDbDxfFiler* filer)
{
    // call assertWriteEnabled() to let AutoCAD know the object is about to
    // be written to and to trigger openedForModify notification.
    //...

    // call the parent class's dxfInFields member function
    //...

    // Check to see if we're at the right subclass data marker
    // If not,  then return Acad::eBadDxfSequence
    //...

    // implement a loop to read in the data members
    //...

    // return the filer status
    //...

    return filer->filerStatus();
}


// Files data out to a DXF file
//
Acad::ErrorStatus
MyClass::dxfOutFields(AcDbDxfFiler* filer) const
{
    // first call assertReadEnabled() to let AutoCAD know a read on the object
    // is about to take place.  Then call the parent class's dxfOutFields member
    // function then write out the subclass data marker for "myclass"
    // then write out the intval value.  Finally,  return the filer status
    //...

    return filer->filerStatus();
}


// Creates two objects of class "myobject" fills them in with the integers
// 1 & 2 and then adds them to the dictionary associated with the key
// "TEST_DICT".  If this dictionary doesn't exist,  then it's created and
// added to the named objects dictionary.
//
void
create_dictionary()
{
  // Open the namedobjects dictionary for write
  //...

  // check to see if the dictionary we want to create is already present
  // if it's not,  then create it and add it to the namedobjects dictionary
  // don't forget to close the namedobjects dictionary when done with it.
  //...

  // if the new dictionary is successfully added to the namedobjects
  // dictionary,  create a couple of objects of the "myclass" class.
  // then add them to the newly created dictionary.  Don't forget to
  // close the new objects and the new dictionary when you're done
  // with them.  You'll need to close the myclass objects before closing
  // the new dictionary.
  //...

}


// Opens the dictionary associated with the key "TEST_DICT" and iterates
// through all it's entries printing out the integer data value in each
// entry.
//
void
iterate_dictionary()
{
  // Open the namedobjects dictionary for read
  //..

  // try to access and open the dictionary created in the create_dictionary
  // function above.  If it's not openable,  print an error message,
  // close namedobject dictionary,  and return.  If it does open then just
  // close the namedobject dictionary.
  //...
        
  // get an iterator for the TEST_DICT dictionary
  //..

  // Set up a loop that uses the iterator to walk down the entries in
  // the dictionary,  reading each "myclass" object's "intval" value
  // and printing it out.  Don't forget to close each myclass object
  // when you're done with it.  Also delete the iterator object and
  // close the dictionary when you're all done.
  //...

}


// Initialization function called from acrxEntryPoint during kInitAppMsg
// case.  This function is used to add commands to the command stack and
// to add classes to the ACRX class hierarchy
//
void
initApp()
{
    acedRegCmds->addCommand("DICTIONARY_COMMANDS", "CREATE", "CREATE",
        ACRX_CMD_MODAL, &create_dictionary);

    acedRegCmds->addCommand("DICTIONARY_COMMANDS", "ITERATE", "ITERATE",
        ACRX_CMD_MODAL, &iterate_dictionary);

    MyClass::rxInit();
    acrxBuildClassHierarchy();
}


// Clean up function called from acrxEntryPoint during the kUnloadAppMsg
// case.  This function removes this apps command set from the command
// stack.
//
void
unloadApp()
{
    acedRegCmds->removeGroup("DICTIONARY_COMMANDS");
    
    // Remove the myObject class from the ACRX runtime class hierarchy.
    // If this is done while database is still active, it should cause
    // all objects of class myObject to be zombified. 
    //
    deleteAcRxClass(MyClass::desc());
}


// ARX entry point
//
AcRx::AppRetCode
acrxEntryPoint(AcRx::AppMsgCode msg, void* pkt)
{
    switch (msg) {
    case AcRx::kInitAppMsg:
        acrxDynamicLinker->unlockApplication(pkt);
        initApp();
        break;
    case AcRx::kUnloadAppMsg:
        unloadApp();
        break;
    case AcRx::kLoadADSMsg:
    case AcRx::kUnloadADSMsg:
    default:
        break;
    }
    return AcRx::kRetOK;
}

