/* $Id: rxhello9.cpp,v 1.2 1995/09/23 21:39:19 artc Exp $ */
/* 
      (C) Copyright 1993-1995 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

//
// rxhello9.cpp - Rx app defining its own AutoCAD AcDb entity
//
// The minimal Rx program creating an own database
// entity.
//
// The entity is created by the AutoCAD command HELLO9.
// All other protocol is accessed by the polymorphism 
// of the AutoCAD editing commands.
//
// Our entity is a Jblob, a line with a filled circle at 
// its endpoint.
//
// Jeremy Tammik, Autodesk ESC, 94-08-11
//
// Based on bcmd.cpp by Peter Janson, 10. May 1994.
//
#include <string.h>
#include <rxobject.h>
#include <rxregsvc.h>
#include <rxdlinkr.h>
#include <aced.h>
#include <dbents.h>
#include <dbsymtb.h>
#include <adslib.h>
#include "jblob.h"
#include "rxhello9.h"

// hello9 - AutoCAD command to manipulate dictionary:
//
// We offer options to create a new dictionary, 
// and add, list and delete entries from it.
//
void
hello9()
{
    Jblob *pJ = new Jblob;

    if (!pJ)
        ads_printf("Error creating Jblob.\n");
    else
        acquireJblob(pJ);
}

// acquireJblob - acquire Jblob parameters from user:
//
void
acquireJblob(Jblob* pJ)
{
    AcGePoint3d p, q;
    double r;

    if (getPoint(0, "From point: ", p) == RTNORM
        && getPoint((double*)(&p), "To point: ", q) == RTNORM
        && ads_grdraw((double*)(&p), (double*)(&q), -1, 1) == RTNORM
        && ads_getdist((double*)(&q), "Blob radius: ", &r) == RTNORM)
    {
        pJ->setEp(p);
        pJ->setBp(q);
        pJ->setBr(r);
        append(pJ);
        pJ->close();
    }
}

// append - append an opened entity to paper space in the database:
//
Adesk::Boolean
append(AcDbEntity* pEntity)
{
    AcDbBlockTable *pBlockTable;
    Acad::ErrorStatus es = acdbCurDwg()->getBlockTable(pBlockTable,
        AcDb::kForRead);
    if (es != Acad::eOk) {
        ads_alert("Failed to get block table!");
        return Adesk::kFalse;
    }

    AcDbBlockTableRecord *pBlockRec;
    es = pBlockTable->getAt(ACDB_MODEL_SPACE, AcDb::kForWrite, pBlockRec);
    if (es != Acad::eOk) {
        ads_alert("Failed to get block table record!");
        pBlockTable->close();
        return Adesk::kFalse;
    }

    es = pBlockRec->appendAcDbEntity(pEntity);
    if (es != Acad::eOk) {
        ads_alert("Failed to append entity!");
        pBlockTable->close();
        pBlockRec->close();
        delete pEntity;
        return Adesk::kFalse;
    }
    pBlockRec->close();
    pBlockTable->close();
    return Adesk::kTrue;
}

// getPoint - acquire WCS point from user:
//
int
getPoint(double *pF, char* pMessage, AcGePoint3d& point)
{
    int stat;
    ads_point ptU, ptW;
    struct resbuf rbFrom, rbTo;

    if ((stat = ads_getpoint(pF, pMessage, ptU)) != RTNORM)
        return stat;

    rbFrom.restype = RTSHORT;
    rbFrom.resval.rint = 1;     // from UCS
    rbTo.restype = RTSHORT;
    rbTo.resval.rint = 0;       // to WCS

    if ((stat = ads_trans(ptU, &rbFrom, &rbTo, FALSE, ptW)) != RTNORM)
        return stat;

    point[X] = ptW[X];
    point[Y] = ptW[Y];
    point[Z] = ptW[Z];

    ads_printf("\n");
    return RTNORM;
}

void
initApp()
{
    ads_printf("Rx Hello 9 -- initApp.\n");
    acedRegCmds->addCommand("HELLO9", "HELLO9", "HELLO9",
        ACRX_CMD_MODAL, &hello9);
    Jblob::rxInit();
    acrxBuildClassHierarchy();
}

void
unloadApp()
{
    ads_printf("Rx Hello 9 -- unloadApp.\n");
    acedRegCmds->removeGroup("HELLO9");

    // remove the Jblob class from the AcRx runtime tree.  This will cause
    // any existing Joblob entities to become zombies if this app is unloaded
    // while AutoCAD is still active.
    //
    deleteAcRxClass(Jblob::desc());

}

extern "C" AcRx::AppRetCode
acrxEntryPoint(AcRx::AppMsgCode msg, void* pkt)
{
    switch (msg) {
    case AcRx::kInitAppMsg:
        acrxDynamicLinker->unlockApplication(pkt);
        initApp();
        break;
    case AcRx::kUnloadAppMsg:
        unloadApp();
        break;
    case AcRx::kLoadADSMsg:
        ads_printf("\nRx Hello 9 -- kLoadADSMsg\n");
        break;
    case AcRx::kUnloadADSMsg:
        ads_printf("Rx Hello 9 -- kUnloadADSMsg\n");
        break;
    default:
        ads_printf("Rx Hello 9 -- <unknown message %d>\n", msg);
        break;
    }
    return AcRx::kRetOK;
}
