/* $Id: rxhello8.cpp,v 1.2 1995/05/16 18:42:45 jamesr Exp $ */
/* 
      (C) Copyright 1993-1995 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

//
// rxhello8.cpp - Rx app to manipulate AutoCAD AcDb entities
//
// The minimal Rx program creating an own dictionary
// in the AutoCAD database and manipulating symbols
// in it (add, look up, delete).
//
// We define an AutoCAD command HELLO8 to do so.
//
// Jeremy Tammik, Autodesk ESC, 94-08-09
//
// Snarfed 100% from Bill Adkison's 
// $acad/rx/sample/acdbtest/acdbtest.cc
// with purely cosmetical changes.
//
#include <string.h>
#include <rxobject.h>
#include <rxregsvc.h>
#include <rxdlinkr.h>
#include <aced.h>
#include <dbents.h>
#include <dbsymtb.h>
#include <adslib.h>
#include "rxhello8.h"

// ObjectPointer8 class implementation:
//
// a custom object to play with ...
//
#include "obj8.h"

ACRX_DXF_CLASS_OBJ_BODY( ObjectPointer8, AcDbObject, 0, OBJPTR8, RxHello8 );

// abbreviations:
//
#include <adeskabb.h>
#include <acdbabb.h>

#define streq(s,t) (strcmp((s),(t)) == 0)

//
// hello8 - AutoCAD command to manipulate dictionary:
//
// We offer options to create a new dictionary, 
// and add, list and delete entries from it.
//
void hello8()
{
	int rc;
	char kw[20];
	AcDbObjectId dictId;

	ads_printf( "This is hello8() in RxHello8 ...\n" );
	ads_initget( 0, "Open Add List Delete" );
	rc = ads_getkword( "Open/List/Delete/<Add>: ", kw );

	if( rc == RTNONE ) {
		strcpy( kw, "Add" );
		rc = RTNORM;
	}
	if( rc != RTNORM ) {
		ads_printf( "Nothing selected.\n" );
		return;
	}
	if( !getDict( dictId ) ) {
		ads_printf( "Failed to open %s dictionary.\n", HELLO8_DICT );
		return;
	}
	if( streq( kw, "Add" ) ) {
		addDict( dictId ); // of course I'm addicted ... who isn't?
	}
	else if( streq( kw, "List" ) ) {
		listDict( dictId );
	}
	else if( streq( kw, "Delete" ) ) {
		delDict( dictId );
	}
	return;
}

// getDict
//
// Fetch our dictionary out of the current DWG, using our
// chosen object name.  If it isn't there, create it.
//
Boolean getDict( AcDbObjectId& dictId )
{
	AcDbDictionary *nameList;

	// Look for dictionary in the current drawing's named objects
	// dictionary.  If not present, create a dictionary, add it to
	// the database, then give it a name, by which it can be accessed
	// forever after:
	//
	if ( acdbCurDwg()->namedObjects( nameList, kForWrite ) != eOk ) {
		ads_printf( "Unable to open named objects dictionary.\n" );
		return kFalse;
	}
	if( nameList->getAt( HELLO8_DICT, dictId ) == eKeyNotFound ) {

		AcDbDictionary* dict = new AcDbDictionary;

		ads_printf( "Adding new dictionary %s to named objects dictionary.\n",
			HELLO8_DICT );

		if( nameList->setAt( HELLO8_DICT, dict, dictId ) != eOk ) {
			ads_printf( "Unable to add dictionary to named objects dictionary.\n" );
			return kFalse;
		}	
		if ( dict->close() != eOk ) {
			ads_printf( "Unable to close dictionary.\n" );
			return kFalse;
		}	
	}
	if( nameList->close() != eOk ) {
		ads_printf( "Unable to close named objects dictionary.\n" );
		return kFalse;
	}	
	return kTrue;
}

// addDict - add new entry to dictionary:
//
void addDict( AcDbObjectId& dictId )
{
	int rc;
	AcDbDictionary* dict;
	char objName[132];

	if ( acdbOpenObject( dict, dictId, kForWrite ) != eOk ) {
		ads_printf( "Unable to open dictionary for writing.\n" );		
		return;
	}
	rc = ads_getstring( kTrue, "Enter dictionary entry key: ", objName );

	if ( rc!=RTNORM || *objName == '\0' ) {
		ads_printf( "Invalid dictionary entry name '%s'.\n", objName );
		dict->close();
		return;
	}

	// create a new test entry ObjectPointer8
	// add it to the dictionary, demonstrating 
	// instantiation and addition of a
	// custom object class into the database.
	//
	AcDbEntity* pEnt;
	AcDbObjectId pEntId;

	pEnt = selectEntity( pEntId, kForRead );

	if ( pEnt == NULL ) {
		ads_printf( "Nothing selected.\n" );
		dict->close();
		return;
	}
	pEnt->close();  // didn't really want do anything, just get the Id.

	ObjectPointer8 * newObjPtr = new ObjectPointer8;

	newObjPtr->setId( pEntId );
	newObjPtr->print();

	AcDbObjectId retObjId;

	if ( dict->setAt( objName, newObjPtr, retObjId ) != eOk ) {
		ads_printf( "Failed to add new entry to dictionary.\n" );
		dict->close();
		return;
	}
	if( newObjPtr->close() != eOk ) {
		ads_printf( "Failed to close new object.\n" );
		dict->close();
		return;
	}
	if( dict->close() != eOk ) {
		ads_printf( "Failed to close dictionary.\n" );
		return;
	}
	return;
}

// delDict - delete entry from dictionary:
//
void delDict( AcDbObjectId& dictId )
{
	int rc;
	AcDbDictionary* dict;
	char objName[132];

	if ( acdbOpenObject( dict, dictId, kForWrite) != eOk ) {
		ads_printf( "Unable to open dictionary for writing.\n" );		
		return;
	}
	rc = ads_getstring( 1, "Name of dictionary entry to remove: ",
		objName );

	if ( rc!=RTNORM || *objName == '\0' ) {
		ads_printf( "Invalid entry name.\n" );
		return;
	}
	if ( getYorN( "By name" ) ) {
		// Tell the dictionary to delete its entry:
		//
		if( dict->remove( objName ) != eOk ) {
			ads_printf( "Error removing dictionary entry.\n" );
		}
	}
	else {
		// By Object, to test back reactor mechanism. and Erase.
		// Tell the entry to delete itself, and relay such 
		// to the dictionary via notification reactors.
		//
		AcDbObject* tObj;

		if ( dict->getAt( objName, kForWrite, tObj ) != eOk ) {
			ads_printf( "Error retrieving dictionary entry.\n" );
			return;
		}
		if ( tObj->erase( kTrue ) != eOk ) {
			ads_printf( "Error erasing entity.\n" );
		}
		if( tObj->close() != eOk ) {
			ads_printf( "Error closing object.\n" );
		}
	}
	if( dict->close() != eOk ) {
		ads_printf( "Error closing dictionary.\n" );
	}
}

// listDict - list entries in dictionary:
//
void listDict( const AcDbObjectId& dictId )
{
	AcDbDictionary * dict;

	if( acdbOpenObject( dict, dictId, kForRead ) != eOk ) {
		ads_printf( "Unable to open dictionary for reading.\n" );
		return;
	}
	ads_printf( "\nDictionary contents\n===\n" );

	AcDbDictionaryIterator * i;
	for ( i=dict->newIterator(); !i->done(); i->nextElement() ) {

		char * entryName = i->name();
		ads_printf( "Name: '%s',  ", entryName );
		delete entryName;
		printObjectInfo( i->objectId() );

		// now we have the object id for our PointerObject8,
		// but how do we extract any info from it ... this
		// works, but I don't know if there is a more elegant
		// method:
		//
        AcDbObject * obj;
		if( acdbOpenObject( obj, i->objectId(), kForRead ) != eOk ) {
			ads_printf( "Unable to open entry object for reading.\n" );
		}
		else {
			ads_printf( "      --> " ); // "points to"
			((ObjectPointer8 *)obj)->print(); // show internals
			obj->close();
		}
	}
	delete i;
	ads_printf( "===\nEnd of Dictionary Contents\n" );

	if( dict->close() != eOk ) {
		ads_printf( "Error closing dictionary.\n" );
	}
}

// selectEntity - utility function to select entity
//
// Select entity, retrieve object Id and open it,
// creating and returning a valid object pointer.
//
AcDbEntity* selectEntity( AcDbObjectId& eId, OpenMode openMode )
{
	int rc;
	ads_name en;
	ads_point pt;
    Acad::ErrorStatus es;

	rc = ads_entsel( "\nSelect an entity: ", en, pt );

	if (rc != RTNORM) {
		ads_printf( "Nothing selected.\n", rc );
		return NULL;
	}

	es = acdbGetObjectId( eId, en );
	if ( es != Acad::eOk ) {
		ads_printf( "Either acdbGetObjectId or ads_entsel failed: "
			"Entity name <%lx,%lx>, error %d.\n", en[0], en[1], es );
		return NULL;
	}
	AcDbEntity* entObj;

	es = acdbOpenEntity( entObj, eId, openMode );
	if ( es != eOk ) {
		ads_printf( "acdbOpenEntity failed with error %d.\n", es );
		return NULL;
	}
	return entObj;
}

// getYorN - utility function to get boolean from user:
//
Boolean getYorN( const char* pStr )
{
	char yorn_str[132];

	yorn_str[0] = 'Y';
	yorn_str[1] = '\0';
	ads_printf( "\n%s", pStr );
	ads_initget( 0, "No Yes" );
	ads_getkword( " -- No/<Yes>:  ", yorn_str );
	return( yorn_str[0] != 'N' );
}

// printObjectInfo - utility function to print object info:
//
void printObjectInfo( const AcDbObjectId& id )
{
	AcDbObject* obj;

	if ( acdbOpenObject( obj, id, kForRead ) != eOk )
		ads_printf( "printObjectInfo: open object failed.\n" );
	else {
		AcDbHandle objHandle;
		char handleStr[20];
		char *cname;

		obj->getAcDbHandle( objHandle );
		objHandle.getIntoAsciiBuffer( handleStr );
		cname = obj->objectClass()->getName()->cpyToAscii();

		ads_printf( "Object Id %lx, handle %s, class %s.\n",
			id, handleStr, cname );
		delete cname;
		obj->close();
	}
}

void initApp()
{
	ads_printf( "Rx Hello 8 -- initApp.\n" );
	acedRegCmds->addCommand( "HELLO8", "HELLO8",
		"HELLO8", ACRX_CMD_MODAL, &hello8);
    initObjectPointer8();
    acrxBuildClassHierarchy();
}

void unloadApp()
{
	ads_printf( "Rx Hello 8 -- unloadApp.\n" );
	acedRegCmds->removeGroup( "HELLO8" );
}

extern "C" AcRx::AppRetCode acrxEntryPoint( AcRx::AppMsgCode msg, void* )
{
	switch( msg ) {
	case AcRx::kInitAppMsg: initApp(); break;
	case AcRx::kUnloadAppMsg: unloadApp(); break;
	case AcRx::kLoadADSMsg:
		ads_printf( "\nRx Hello 8 -- kLoadADSMsg\n" );
		break;
	case AcRx::kUnloadADSMsg:
		ads_printf( "Rx Hello 8 -- kUnloadADSMsg\n" );
		break;
	default:
		ads_printf( "Rx Hello 8 -- <unknown message %d>\n", msg );
		break;
	}
	return AcRx::kRetOK;
}

