/* $Id: adsprint.cpp,v 1.1 1994/10/18 16:40:43 edwardb Exp $ */
/* 
      (C) Copyright 1993-1994 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

static char rcsid[] = "$Id: adsprint.cpp,v 1.1 1994/10/18 16:40:43 edwardb Exp $";
//////////////////////////////////////////////////////////////////////// INCA
//
//  ADSPRINT.CPP
//  written 1994 by Markus Kraus, Autodesk Germany.
//
//  Copyright(C) 1994 Autodesk, Inc.  All Rights Reserved.
//
//  Permission  to  use, copy, modify, and distribute this software for any
//  purpose  and  without  fee  is  hereby granted, provided that the above
//  copyright notice appears in all copies and that both that copyright no-
//  tice and this permission notice appear in all supporting documentation.
//
//  Autodesk makes no warrantees, express or implied, as to the correctness
//  of this code or any derivative works which incoporate it. Autodesk pro-
//  vides  the  code  on  an  ''as-is''  basis and explicitly disclaims any
//  liability,  express  or  implied, for errors, omissions, and other pro-
//  blems in the code, including conseqential and incidental damages.
//
//  Description:
//      Implementation of acout (object of class AdsOutput), which has a
//      behaviour similar to cout, but uses ads_printf for use in AutoCAD.
//
//  Used/tested Compiler & Linker:
//      MS Visual C++ 1.50
//
//  Functions:
//      printrb
//      printresvals
//      printdxf
//
//  Version: 1.0
//  last change:   27.07.94   10:52
//
//  History:
//  $Log: adsprint.cpp,v $
// Revision 1.1  1994/10/18  16:40:43  edwardb
// r085eab1  New ARX lab samples
//
//  Revision 1.1  1994/07/27 10:58:10  mkr
//  Initial Revision.
//
//  Notes and restrictions on use:
//
//////////////////////////////////////////////////////////////////////// INCA

/////////////////////////////////////////////////////////////////////////////
// Includes

#include <adslib.h>
#include "adsprint.h"

#ifndef NULL
#define NULL (void*)0
#endif

// Prototypes for local functions

static void printrb      (struct resbuf *eb);
static void printresvals (struct resbuf *eb);
static void printdxf     (struct resbuf *eb);

// The predefined stream
AdsOutput acout;


// insert the character
AdsOutput& AdsOutput::operator<< (char c)
{
    ads_printf( "%c", c );
    return *this;
}

// for the following, insert character representation of numeric value
AdsOutput& AdsOutput::operator<< (long l)
{
    ads_printf( "%ld", l );
    return *this;
}

AdsOutput& AdsOutput::operator<< (unsigned long ul)
{
    ads_printf( "%lu", ul );
    return *this;
}

AdsOutput& AdsOutput::operator<< (long double ld)
{
    ads_printf( "%Lf", ld );
    return *this;
}

// insert character representation of the value of the pointer
AdsOutput& AdsOutput::operator<< (void * vp)
{
    ads_printf( "0x%p", vp );
    return *this;
}

// insert the null-terminated string
void AdsOutput::outstr (const char * s)
{
    ads_printf( "%s", s );
}

// manipulators
AdsOutput& textpage( AdsOutput& ao2 )
{
    ads_textpage();
    return ao2;
}

AdsOutput& textscr( AdsOutput& ao2 )
{
    ads_textscr();
    return ao2;
}

AdsOutput& graphscr( AdsOutput& ao2 )
{
    ads_graphscr();
    return ao2;
}

AdsOutput& endl( AdsOutput& ao2 )
{
    return ao2 << "\n";
}

// print struct resbuf
AdsOutput& AdsOutput::operator<< (struct resbuf * rb)
{
    printrb( rb );
    return *this;
}


/////////////////////////////////////////////////////////////////////////////
// printrb
// print out contents of resbuf in a lisp-like style

static void
/*FCN*/printrb (struct resbuf *eb)
{
    while (eb != NULL) {
        if (eb->restype >= 5000)
            printresvals( eb );
        else
            printdxf( eb );
        eb = eb->rbnext;
    }
}


/////////////////////////////////////////////////////////////////////////////
// printresvals
// for lists returned by ads_getargs, for example.

static void
/*FCN*/printresvals (struct resbuf *eb)
{
    switch (eb->restype) {
    case RTNONE:    /* No result */
        ads_printf( "RTNONE " );
        break;

    case RTREAL:    /* Real number */
    case RTANG:     /* Angle */
    case RTORINT:   /* Orientation */
        ads_printf( "%0.3f ", eb->resval.rreal );
        break;

    case RTPOINT:   /* 2D point X and Y only */
        ads_printf( "(%0.3f %0.3f) ", eb->resval.rpoint[0],
                    eb->resval.rpoint[1] );
        break;

    case RTSHORT:   /* Short integer */
        ads_printf( "%d ", eb->resval.rint );
        break;

    case RTSTR:     /* String */
        ads_printf( "\"%s\" ", eb->resval.rstring );
        break;

    case RTENAME:   /* Entity name */
        ads_printf( "<Entity name: %8lx> ", eb->resval.rlname[0] );
        break;

    case RTPICKS:   /* Pick set */
        ads_printf( "<Selection set: %d> ", eb->resval.rlname[0] );
        break;

    case RT3DPOINT: /* 3D point - X, Y, and Z */
        ads_printf( "(%0.3f %0.3f %0.3f) ", eb->resval.rpoint[0],
                    eb->resval.rpoint[1], eb->resval.rpoint[2] );
        break;

    case RTLONG:    /* Long integer */
        ads_printf( "%ld ", eb->resval.rlong );
        break;

    case RTVOID:    /* Blank symbol */
        ads_printf( "RTVOID " );
        break;

    case RTLB:      /* list begin */
        ads_printf( "( " );
        break;

    case RTLE:      /* list end */
        ads_printf( ") " );
        break;

    case RTDOTE:    /* dotted pair */
        ads_printf( ". " );
        break;

    case RTNIL:     /* nil */
        ads_printf( "nil " );
        break;

    case RTDXF0:    /* DXF code 0 for ads_buildlist only */
        ads_printf( "RTDXF0 " );
        break;

    case RTT:       /* T atom */
        ads_printf( "T " );
        break;

    default:
        ads_printf( "Unknown type: %d", eb->restype );
        break;

    }
}


/////////////////////////////////////////////////////////////////////////////
// printdxf
// for entity-lists

static void
/*FCN*/printdxf (struct resbuf *eb)
{
    int type, rt;

    type = eb->restype;
    if (type >= 1000) /* extended entity data */
        type -= 1000;
    if ((type >= 0) && (type <= 9))
        rt = RTSTR;
    else if ((type >= 10) && (type <= 19))
        rt = RT3DPOINT;
    else if ((type >= 20) && (type <= 59))
        rt = RTREAL;
    else if ((type >= 60) && (type <= 79))
        rt = RTSHORT;
    else if ((type >= 210) && (type <= 239))
        rt = RT3DPOINT;
    else if (type == 999)
        rt = RTSTR;
    else if (type == -1)
        rt = -1; /* entity name */
    else if (type == -3)
        rt = RTSTR; /* appid - application name */
    else
        rt = RTNONE;    /* ???? */

    switch (rt) {

    case RTSHORT:
        ads_printf( "(%d . %d)", eb->restype, eb->resval.rint );
        break;

    case RTREAL:
        ads_printf( "(%d . %0.3f)", eb->restype, eb->resval.rreal );
        break;

    case RTSTR:
        ads_printf( "(%d . \"%s\")", eb->restype, eb->resval.rstring );
        break;

    case RT3DPOINT:
        ads_printf( "(%d %0.3f %0.3f %0.3f)", eb->restype,
                eb->resval.rpoint[0], eb->resval.rpoint[1],
                eb->resval.rpoint[2] );
        break;

    case RTNONE:
        ads_printf( "(%d . Unknown type)", eb->restype );
        break;

    case -1:
        ads_printf( "(%d . <Entity name: %8lx>)", eb->restype,
                eb->resval.rlname[0] );
    }
}
