/* $Id: adsprint.h,v 1.2 1995/05/16 18:47:21 jamesr Exp $ */
/* 
      (C) Copyright 1993-1995 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

#ifndef _adsprint_h
#define _adsprint_h
//////////////////////////////////////////////////////////////////////// INCA
//
//  ADSPRINT.H
//  written 1994 by Markus Kraus, Autodesk Germany.
//
//  Copyright(C) 1994 Autodesk, Inc.  All Rights Reserved.
//
//  Permission  to  use, copy, modify, and distribute this software for any
//  purpose  and  without  fee  is  hereby granted, provided that the above
//  copyright notice appears in all copies and that both that copyright no-
//  tice and this permission notice appear in all supporting documentation.
//
//  Autodesk makes no warrantees, express or implied, as to the correctness
//  of this code or any derivative works which incoporate it. Autodesk pro-
//  vides  the  code  on  an  ''as-is''  basis and explicitly disclaims any
//  liability,  express  or  implied, for errors, omissions, and other pro-
//  blems in the code, including conseqential and incidental damages.
//
//  Description:
//      Declaration of acout (object of class AdsOutput), which has a
//      behaviour similar to cout, but uses ads_printf for use in AutoCAD.
//
//  Used/tested Compiler & Linker:
//      MS Visual C++ 1.50
//
//  Functions:
//
//  Class Definitions:
//      AdsOutput
//
//  Version: 1.0
//  last change:   27.07.94   10:53
//
//  History:
/*  $Log: adsprint.h,v $
 * Revision 1.2  1995/05/16  18:47:21  jamesr
 * API> ----------------------------
 * API> revision 1.3
 * API> date: 1995/03/09 16:45:01;  author: edwardb;  state: Exp;  lines: +8 -6
 * API> rx04eab1  Corrected corrupted comments in header
 * API> ----------------------------
 * API> revision 1.2
 * API> date: 1995/02/15 10:26:35;  author: edwardb;  state: Exp;  lines: +4 -4
 * API> rx02eab1
 * API> ----------------------------
 * API> revision 1.1
 * API> Common base with R104
 * API> ----------------------------
 *
 * Revision 1.3  1995/03/09  16:45:01  edwardb
 * rx04eab1  Corrected corrupted comments in header
 *
 *  Revision 1.2  1995/02/15  10:26:35  edwardb
 *  rx02eab1
 *
 *  Revision 1.1  1994/07/27 10:58:11  mkr
 *  Initial Revision.
 */
//  Notes and restrictions on use:
//
//////////////////////////////////////////////////////////////////////// INCA

#if defined(_MSC_VER) && defined(_WIN32)
//adsprint.h(70) : warning C4069: long double is the same precision as double
#pragma warning( disable : 4069 )
#endif

/////////////////////////////////////////////////////////////////////////////
// class AdsOutput

class
/*CLASS*/AdsOutput
{
public:
    /*
     * Formatted insertion operations
     */
    // insert the character
    AdsOutput& operator<< (         char);
    AdsOutput& operator<< (  signed char);
    AdsOutput& operator<< (unsigned char);

    // for the following, insert character representation of numeric value
    AdsOutput& operator<< (short);
    AdsOutput& operator<< (unsigned short);
    AdsOutput& operator<< (int);
    AdsOutput& operator<< (unsigned int);
    AdsOutput& operator<< (long);
    AdsOutput& operator<< (unsigned long);
    AdsOutput& operator<< (float);
    AdsOutput& operator<< (double);
    AdsOutput& operator<< (long double);

    // insert the null-terminated string
    AdsOutput& operator<< (const          char *);
    AdsOutput& operator<< (const   signed char *);
    AdsOutput& operator<< (const unsigned char *);

    // insert character representation of the value of the pointer
    AdsOutput& operator<< (void *);

    // manipulators
    AdsOutput& operator<< (AdsOutput& (*f)(AdsOutput& ao2));

    // print struct resbuf
    AdsOutput& operator<< (struct resbuf * rb);

private:
    void outstr (const char *);
};


/*
 * Manipulators
 */
AdsOutput& endl     (AdsOutput& ao2);
AdsOutput& textpage (AdsOutput& ao2);
AdsOutput& textscr  (AdsOutput& ao2);
AdsOutput& graphscr (AdsOutput& ao2);

/*
 * The predefined stream
 */
extern AdsOutput acout;

/*
 * Inline operators
 */

inline AdsOutput& AdsOutput::operator<< (signed char _c)
{   return *this << (char)_c;   }

inline AdsOutput& AdsOutput::operator<< (unsigned char _c)
{   return *this << (char)_c;   }

inline AdsOutput& AdsOutput::operator<< (const char * _s)
{
    outstr( _s );
    return *this;
}

inline AdsOutput& AdsOutput::operator<< (const signed char * _s)
{
    outstr( (const char *)_s );
    return *this;
}

inline AdsOutput& AdsOutput::operator<< (const unsigned char * _s)
{
    outstr( (const char *)_s );
    return *this;
}

inline AdsOutput& AdsOutput::operator<< (short _i)
{   return *this << (long) _i;   }

inline AdsOutput& AdsOutput::operator<< (unsigned short _i)
{   return *this << (unsigned long) _i;   }

inline AdsOutput& AdsOutput::operator<< (int _i)
{   return *this << (long) _i;   }

inline AdsOutput& AdsOutput::operator<< (unsigned int _i)
{   return *this << (unsigned long) _i;   }

inline AdsOutput& AdsOutput::operator<< (float _f)
{   return *this << (long double) _f;   }

inline AdsOutput& AdsOutput::operator<< (double _d)
{   return *this << (long double) _d;   }

inline AdsOutput& AdsOutput::operator<< (AdsOutput & (*_f)(AdsOutput &))
{   return (*_f)(*this);   }

#endif /* _adsprint_h */
