/* Next available MSG number is   1 */
/* $Id: aced.h,v 1.25 1995/10/26 16:53:30 neb Exp $ */
#ifndef AC_ACED_H_
#define AC_ACED_H_  1

/*
      (C) Copyright 1982-1992,1994,1995 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

/*
   aced.h
  
   DESCRIPTION:  Header for Rx application definitions and C
  		 access to AutoCAD Editor-specific services.
*/

#include "rxnames.h"

/* Name of Registered Service Object for constructing AutoCAD-specific
   instances.
*/

#define ACED_SERVICES	        "AcEdServices"

#define ACED_EDITOR_OBJ	        "AcEditor"


enum xrefSubcommandActivities
{    
    kStart = 0,
    kStartItem = 2,
    kEndItem = 3,
    kEnd = 4,
    kAbort = 5
};

enum undoSubcommandActivities
{
    kNone = 0,
    kOne = 1,
    kAll = 2
};



/* AutoCAD Command Registration Facility */

#include "accmd.h"


#if defined(__cplusplus)

extern "C" {

#endif /* defined(__cplusplus) */

/* acedCmdLookup()

   Given a string representing a command name, looks through stack of
   registered command dictionaries and returns status indicating whether
   a command of that name is registered.  If a command is found, its
   elements are returned in retStruc.
   Returns TRUE if a command named <cmdStr> was found, FALSE if not.
*/

extern int acedCmdLookup(const char * cmdStr,
                         int globalLookup,
                         struct AcEdCommandStruc * retStruc);

/* acedPopCommandDirectory()

   Takes the input string and puts the command directory of that name
   first in the search path for command name resolution by AutoCAD.
   Returns TRUE if a dictionary named <cmdDirName> was found, FALSE if
   not.
*/

extern int acedPopCommandDirectory(const char * cmdDirName);
 
#if defined(__cplusplus)

} /* extern "C" */


#define acedServices  \
AcEdServices::cast(acrxServiceDictionary->at(ACED_SERVICES))

class AcEdServices: public AcRxService 
{ 
public:
    ACRX_DECLARE_MEMBERS(AcEdServices);

    virtual AcEdCommand * newAcEdCommand(AcRxFunctionPtr cmdFcnAddr,
			     Adesk::Int32 cmdFlags) const = 0;
};


class AcDbDatabase;

class AcEditorReactor: public AcRxObject 
{ 
public:
    ACRX_DECLARE_MEMBERS(AcEditorReactor);

    virtual void commandWillStart(const char * cmdStr);
    virtual void commandEnded(const char * cmdStr);
    virtual void commandCancelled(const char * cmdStr);
    virtual void commandFailed(const char * cmdStr);
 
    virtual void lispWillStart(const char * firstLine);
    virtual void lispEnded();
    virtual void lispCancelled();

    virtual void beginClose(AcDbDatabase* pDwg);

    virtual void beginDxfIn(AcDbDatabase* db);
    virtual void abortDxfIn(AcDbDatabase* db);
    virtual void dxfInComplete(AcDbDatabase* db);

    virtual void beginDxfOut(AcDbDatabase* pDwg);
    virtual void abortDxfOut(AcDbDatabase* pDwg);
    virtual void dxfOutComplete(AcDbDatabase* pDwg);

    virtual void dwgFileOpened(AcDbDatabase* pDwg, char* fileName);
    virtual void databaseToBeDestroyed(AcDbDatabase* pDwg);

    virtual void beginSave(AcDbDatabase* pDwg, const char* pIntendedName);
    virtual void saveComplete(AcDbDatabase* pDwg, const char* pActualName);

    virtual void beginInsert(AcDbDatabase* pTo,
			     const char* pBlockName,
			     AcDbDatabase* pFrom);
    virtual void beginInsert(AcDbDatabase* pTo,
			     const AcGeMatrix3d& xform,
			     AcDbDatabase* pFrom);
    virtual void otherInsert(AcDbDatabase* pTo,
			     AcDbIdMapping& idMap,
			     AcDbDatabase* pFrom);
    virtual void abortInsert(AcDbDatabase* pTo);
    virtual void endInsert(AcDbDatabase* pTo);

    virtual void beginWblock(AcDbDatabase* pTo,
                             AcDbDatabase* pFrom,
                             const AcGePoint3d*& insertionPoint);
    virtual void beginWblock(AcDbDatabase* pTo,
                             AcDbDatabase* pFrom,
                             AcDbObjectId blockId);
    virtual void beginWblock(AcDbDatabase* pTo, AcDbDatabase* pFrom); 
    virtual void otherWblock(AcDbDatabase* pTo,
                             AcDbIdMapping& pIdMap,
                             AcDbDatabase* pFrom);
    virtual void abortWblock(AcDbDatabase* pTo);
    virtual void endWblock(AcDbDatabase* pTo);

    virtual void beginDeepClone(AcDbDatabase* pTo,
				AcDbIdMapping& pIdMap);
    virtual void beginDeepCloneXlation(AcDbIdMapping& pIdMap,
				Acad::ErrorStatus* pRetStatus);
    virtual void abortDeepClone(AcDbIdMapping& pIdMap);
    virtual void endDeepClone(AcDbIdMapping& pIdMap);


    virtual void sysVarChanged(const char * varName,
			       Adesk::Boolean success);
    virtual void sysVarWillChange(const char * varName);
 
    // ----- XREF-related events -----
    virtual void beginAttach(AcDbDatabase* pTo,
			     const char*   pName,
			     AcDbDatabase* pFrom);
    virtual void otherAttach(AcDbDatabase* pTo,
			     AcDbDatabase* pFrom);
    virtual void abortAttach(AcDbDatabase* pTo);
    virtual void endAttach(AcDbDatabase*   pTo);
    virtual void redirected(AcDbObjectId   newId,
			    AcDbObjectId   oldId);
    virtual void comandeered(AcDbDatabase* pTo,
			     AcDbObjectId  id,
			     AcDbDatabase* pFrom);

    // ----- More XREF related events ----
    // Each notification may be sent to notify the reactor of:
    // 1) The start or end of the Subcommand.  (kStart, kEnd)
    // 2) Processing of each Item by the Subcommand.
    //    (kStartItem, kEndItem)
    //    Nesting and use of wildcards may lead to multiple Items processed.
    // 3) Some failure during the Subcommand (kAbort)
    //
    virtual void xrefSubcommandBindItem(int activity, AcDbObjectId blockId);
    virtual void xrefSubcommandAttachItem(int activity, const char* pPath);
    virtual void xrefSubcommandOverlayItem(int activity, const char* pPath);
    virtual void xrefSubcommandDetachItem(int activity, AcDbObjectId blockId);
    virtual void xrefSubcommandPathItem(int activity, AcDbObjectId blockId, 
                                        const char* pNewPath);
    virtual void xrefSubcommandReloadItem(int activity, AcDbObjectId blockId);

    //  -------------- UNDO related events ----------------
    //
    virtual void undoSubcommandAuto(int activity, Adesk::Boolean state);
    virtual void undoSubcommandControl(int activity, int option);
    virtual void undoSubcommandBegin(int activity);
    virtual void undoSubcommandEnd(int activity);
    virtual void undoSubcommandMark(int activity);
    virtual void undoSubcommandBack(int activity);
    virtual void undoSubcommandNumber(int activity, int num);

};


#define acedEditor AcEditor::cast(acrxSysRegistry()->at(ACED_EDITOR_OBJ))

class AcEditor: public AcRxObject 
{ 
public:
    ACRX_DECLARE_MEMBERS(AcEditor);
 
    virtual void addReactor(AcEditorReactor * newObj) = 0;
 
    virtual void removeReactor(AcEditorReactor * delObj) = 0;
};

#endif /* defined(__cplusplus) */

#include "dbjig.h"

#endif /* AC_ACED_H_ */
