#ifndef _ACGI_H
#define _ACGI_H 1
//
// $Id: acgi.h,v 1.34 1995/10/26 15:50:38 neb Exp $
//
// (C) Copyright 1994-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary
// to Autodesk,  Inc.,  and considered a trade secret as defined
// in section 499C of the penal code of the State of California.
// Use of  this information  by  anyone  other  than  authorized
// employees of Autodesk, Inc.  is granted  only under a written
// non-disclosure agreement,  expressly  prescribing  the  scope
// and manner of such use.
//
// This is the Graphics Interface for view independent and dependent
// elaboration of AcDb Entities.
//
// The AutoCAD Graphics Interface (AcGi) is a set of objects comprising 
// an API for elaborating the graphics representation of new AutoCAD
// entitites.  These objects support many geometric primitives, 
// transformation matrix querying, and graphical attributes. 
 
// Required for RX
//
#define ACGI_SERVICES  /*MSG0*/"AcGiServices"

// These are the current kinds of viewport regeneration modes.
// This mode is not settable by the user, but it can be queried 
// in case you need to take different actions for different
// regeneration modes.
//
typedef enum {
    eAcGiRegenTypeInvalid = 0,
    kAcGiStandardDisplay = 2,
    kAcGiHideOrShadeCommand,
    kAcGiRenderCommand,
    kAcGiSaveWorldDrawForR12
} AcGiRegenType;

// These are the current face filling modes
//
typedef enum {
     kAcGiFillAlways = 1,               // fill
     kAcGiFillNever                     // don't fill
} AcGiFillType;

// These are the edge visibility types
//
typedef enum {
    kAcGiInvisible = 0,
    kAcGiVisible,
    kAcGiSilhouette
} AcGiVisibility;

// These are the types of arcs
//
typedef enum {
    kAcGiArcSimple = 0,                 // just the arc (not fillable)
    kAcGiArcSector,                     // area bounded by the arc
					//     and its center of curvature
    kAcGiArcChord                       // area bounded by the arc
					//     and its end points
} AcGiArcType;

// These are the possible types of vertex orientation 
// 
typedef enum {
    kAcGiCounterClockwise = -1,
    kAcGiNoOrientation = 0,
    kAcGiClockwise = 1
} AcGiOrientationType;

// This signifies how to calculate maximum deviation for tesselation
//
typedef enum {
    kAcGiMaxDevForCircle = 0,
    kAcGiMaxDevForCurve,
    kAcGiMaxDevForBoundary,
    kAcGiMaxDevForIsoline,
    kAcGiMaxDevForFacet
} AcGiDeviationType;

#include "rxobject.h"
#include "gemat3d.h"
#include "gepnt2d.h"
#include "acdb.h"

class AcGiSubEntityTraits;
class AcGiSubEntityTraitsImp;
class AcGiWorldGeometry;
class AcGiViewport;
class AcGiViewportGeometry;
class AcGiImpEdgeData;
class AcGiImpFaceData;
class AcGiImpVertexData;
class AcGiImpTextStyle;
class AcGiTextStyle;
class AcGiEdgeData;
class AcGiFaceData;
class AcGiVertexData;


class AcGiWorldDraw: public AcRxObject
//
// This class provides an API for generating graphics
// in model coordinates.  These graphics cannot be
// controlled by any viewport, as can AcGiViewportDraw.
//
// regenType            current rendering mode
// regenAbort           if true, stop elaborating and
//                          return from your app.
// subEntityTraits      setting attributes
// geometry             drawing graphics and getting
//                          current graphics transforms
//                          and setting extents.
// isDragging           whether dragging is occurring
// deviation            suggested maximum tessellation
//                          deviation
// numberOfIsolines     number of isolines to be drawn
//                          per surface
//
{
public:
    ACRX_DECLARE_MEMBERS(AcGiWorldDraw);
    virtual AcGiRegenType           regenType() const = 0;
    virtual Adesk::Boolean          regenAbort() const = 0;
    virtual AcGiSubEntityTraits&    subEntityTraits() const = 0;
    virtual AcGiWorldGeometry&      geometry() const = 0;
    virtual Adesk::Boolean          isDragging() const = 0;
    virtual double                  deviation(const AcGiDeviationType, 
					      const AcGePoint3d&) const = 0;
    virtual Adesk::UInt32           numberOfIsolines() const = 0;
};


class AcGiViewportDraw: public AcRxObject 
//
// This class provides an API for generating graphics
// that can be sent to a particular viewport.
//
// regenType            current rendering mode
// regenAbort           if true, stop elaborating and
//                          return from your app.
// subEntityTraits      setting attributes
// geometry             drawing graphics and getting
//                          current graphics transforms
//                          and setting extents.
// isDragging           whether dragging is occurring
// deviation            suggested maximum tessellation
//                          deviation
// numberOfIsolines     number of isolines to be drawn
//                          per surface
//
{ 
public:
    ACRX_DECLARE_MEMBERS(AcGiViewportDraw);
    virtual AcGiRegenType        regenType() const = 0;
    virtual Adesk::Boolean       regenAbort() const = 0;
    virtual AcGiViewport&        viewport() const = 0;
    virtual AcGiSubEntityTraits& subEntityTraits() const = 0;
    virtual AcGiViewportGeometry& geometry() const = 0;
    virtual Adesk::Boolean       isDragging() const = 0;
    virtual double               deviation(const AcGiDeviationType, 
				           const AcGePoint3d&) const = 0;
    virtual Adesk::UInt32        numberOfIsolines() const = 0;
    virtual Adesk::UInt32        sequenceNumber() const = 0;
    virtual Adesk::Boolean       isValidId(const Adesk::UInt32 acgiId) const =0;
};

  
class AcGiViewport: public AcRxObject 
//
// This class returns viewport info.
//
// getModelToEyeTransform         model to pre-perspective transform
// getEyeToModelTransform         pre-perspective to model transform
// getWorldToEyeTransform         world to pre-perspective transform
// getEyeToWorldTransform         pre-perspective to world transform
// doPerspective                  do perpective transform
// doInversePerspective           do inverse perpective transform
// getNumPixelsInUnitSquare       get the x by y size in pixels of a
//                                unit square parallel to the viewport
//                                at a given world coordinate.
// getCameraLocation              world coord of the camera location
// getCameraTarget                world coord of the camera target
// getCameraUpVector              world coord of the camera up vector
// isPerspective                  whether in perspective mode
// getViewportDcCorners           get viewport corners in display coords
// getFrontAndBackClipValues      get front and back clipping values
//                                for pre-perspective view volumes.
//
{ 
public:
    ACRX_DECLARE_MEMBERS(AcGiViewport);
    virtual void getModelToEyeTransform(AcGeMatrix3d &pMatrix) const = 0;
    virtual void getEyeToModelTransform(AcGeMatrix3d &pMatrix) const = 0;
    virtual void getWorldToEyeTransform(AcGeMatrix3d &pMatrix) const = 0;
    virtual void getEyeToWorldTransform(AcGeMatrix3d &pMatrix) const = 0;
    virtual Adesk::Boolean isPerspective() const = 0;
    virtual Adesk::Boolean doPerspective(AcGePoint3d& pt) const = 0;
    virtual Adesk::Boolean doInversePerspective(AcGePoint3d& pt) const = 0;
    virtual void getNumPixelsInUnitSquare(const AcGePoint3d &givenWorldpt,
		                          AcGePoint2d &pixelArea) const = 0;
    virtual void getCameraLocation(AcGePoint3d &loc_vec) const = 0;
    virtual void getCameraTarget(AcGePoint3d &targ_vec) const = 0;
    virtual void getCameraUpVector(AcGeVector3d &up_vec) const = 0;
    virtual Adesk::UInt32 viewportId() const = 0;
    virtual Adesk::Int16  acadWindowId() const = 0;
    virtual void getViewportDcCorners(AcGePoint2d &lower_left,
                                      AcGePoint2d &upper_right) const = 0;
    virtual Adesk::Boolean getFrontAndBackClipValues(
                               Adesk::Boolean& clip_front,
                               Adesk::Boolean& clip_back,
                               double& front,
                               double& back) const = 0;
};


class AcGiSubEntityTraits: public AcRxObject 
//
// This class applies attributes to subsequently drawn AcGi geometry
// primitives.
//
// setColor                             set the color index
// setSelectionMarker                   set a selection marker
// setFillType                          set the fill mode
// color                                return the color index
// layer                                return the layer index
// lineTypeIndex                        return the line type index
// fillType                             return the fill type
//
{ 
public:
    ACRX_DECLARE_MEMBERS(AcGiSubEntityTraits);
    virtual void setColor(const Adesk::UInt16 color) = 0;
    virtual void setLayer(const AcDbObjectId layerId) = 0;
    virtual void setLineType(const AcDbObjectId linetypeId) = 0;
    virtual void setSelectionMarker(const Adesk::Int32 markerId) = 0;
    virtual void setFillType(const AcGiFillType) = 0;

    virtual Adesk::UInt16       color() const = 0;
    virtual AcDbObjectId        layerId() const = 0;
    virtual AcDbObjectId        lineTypeId() const = 0;
    virtual AcGiFillType        fillType() const = 0;

};


class AcGiWorldGeometry: public AcRxObject 
//
// This class contains the view independent, model coordinate,
// 3D geometry methods.
//
// getModelToWorldTransform  -- model to world coordinate transform     
// getWorldToModelTransform  -- world to model coordinate transform
// setExtents             -- set extents in model coordinates
// circle                 -- draw a circle 
// circle                 -- draw a circle given 3 points
// circular arc           -- draw a circular arc (or sector or chord)
// circular arc           -- draw a circular arc (or sector or chord)
//                               given 3 points
// polyline               -- draw a polyline
// polygon                -- draw a polygonal
// mesh                   -- draw an MxN quad mesh
//                              (edge, face, and optional vertex data)
// shell                  -- draw a set of polygons
//                              (edge, face, and optional vertex data)
// text                   -- draw some text using the current style
// text                   -- draw some text specifying the style
// xline                  -- draw a line with end points at infinity
// ray                    -- draw a line with only one end point at
//                               infinity.
//
{ 
public:
    ACRX_DECLARE_MEMBERS(AcGiWorldGeometry);
    virtual void getModelToWorldTransform(AcGeMatrix3d &pMatrix) const = 0;
    virtual void getWorldToModelTransform(AcGeMatrix3d &pMatrix) const = 0;
    virtual void setExtents(AcGePoint3d *pNewExtents) const = 0;

    virtual Adesk::Boolean  circle(const AcGePoint3d& center,
				const double radius,
				const AcGeVector3d& normal
				) const = 0;

    virtual Adesk::Boolean  circle(const AcGePoint3d& pt1,
				const AcGePoint3d& pt2,
				const AcGePoint3d& pt3
				) const = 0;

    virtual Adesk::Boolean  circularArc(const AcGePoint3d& center,
				const double radius,
				const AcGeVector3d& normal,
				const AcGeVector3d& startVector,
				const double sweepAngle,
				const AcGiArcType arcType = kAcGiArcSimple
				) const = 0;

    virtual Adesk::Boolean  circularArc(const AcGePoint3d& start,
				const AcGePoint3d& point,
				const AcGePoint3d& end,
				const AcGiArcType arcType = kAcGiArcSimple
				) const = 0;
      
    virtual Adesk::Boolean  polyline(const Adesk::UInt32 nbPoints,
				const AcGePoint3d* pVertexList
				) const = 0;
      
    virtual Adesk::Boolean  polygon(const Adesk::UInt32 nbPoints,
				const AcGePoint3d* pVertexList
				) const = 0;

    virtual Adesk::Boolean  mesh(const Adesk::UInt32 rows,
				const Adesk::UInt32 columns,
				const AcGePoint3d* pVertexList,
				const AcGiEdgeData* pEdgeData = NULL,
				const AcGiFaceData* pFaceData = NULL,
				const AcGiVertexData* pVertexData = NULL
				) const = 0;

    virtual Adesk::Boolean  shell(const Adesk::UInt32 nbVertex,
				const AcGePoint3d* pVertexList,
				const Adesk::UInt32 faceListSize,
				const Adesk::Int32* pFaceList,
				const AcGiEdgeData* pEdgeData = NULL,
				const AcGiFaceData* pFaceData = NULL,
				const AcGiVertexData* pVertexData = NULL
				) const = 0;
      
    virtual Adesk::Boolean  text(const AcGePoint3d& position,
				const AcGeVector3d& normal,
				const AcGeVector3d& direction,
				const double height,
				const double width,
				const double oblique,
				const char* pMsg
				) const = 0;

    virtual Adesk::Boolean  text(const AcGePoint3d& position,
				const AcGeVector3d& normal,
				const AcGeVector3d& direction,
				const char* pMsg,
				const Adesk::Int32 length,
				const Adesk::Boolean raw,
				const AcGiTextStyle &pTextStyle
				) const = 0;

    virtual Adesk::Boolean  xline(const AcGePoint3d& one_xline_point,
				const AcGePoint3d& a_different_xline_point
				) const = 0;

    virtual Adesk::Boolean  ray(const AcGePoint3d& rays_starting_point,
				const AcGePoint3d& a_different_ray_point
				) const = 0;
};


class AcGiViewportGeometry: public AcRxObject 
//
// This class contains the view dependent, model and drawing coordinate,
// 3D geometry methods.
//
// getModelToWorldTransform -- model to world coordinate transform
// getWorldToModelTransform -- world to model coordinate transform
//
// circle                -- draw a circle 
// circle                -- draw a circle given 3 points
// circularArc           -- draw a circular arc (or sector or chord)
// circularArc           -- draw a circular arc (or sector or chord)
//                              given 3 points
// polyline              -- draw a polyline
// polygon               -- draw a polygonal
// mesh                  -- draw an MxN quad mesh
//                              (edge, face, and optional vertex data)
// shell                 -- draw a set of polygons
//                              (edge, face, and optional vertex data)
// text                  -- draw some text using the current style
// text                  -- draw some text specifying the style
// xline                 -- draw a line with end points at infinity
// ray                   -- draw a line with only one end point at
//                               infinity.
// polylineEye           -- draw an eye coordinate 3D polyline
// polygonEye            -- draw an eye coordinate 3D polyline
// polylineDc            -- draw a display coordinate 3D polyline
// polygonDc             -- draw a display coordinate 3D polyline
//
{ 
public:
    ACRX_DECLARE_MEMBERS(AcGiViewportGeometry);

    virtual void getModelToWorldTransform(AcGeMatrix3d &pMatrix) const = 0;
    virtual void getWorldToModelTransform(AcGeMatrix3d &pMatrix) const = 0;

    virtual Adesk::Boolean  circle(const AcGePoint3d& center,
				const double radius,
				const AcGeVector3d& normal
				) const = 0;

    virtual Adesk::Boolean  circle(const AcGePoint3d& pt1,
				const AcGePoint3d& pt2,
				const AcGePoint3d& pt3
				) const = 0;

    virtual Adesk::Boolean  circularArc(const AcGePoint3d& center,
				const double radius,
				const AcGeVector3d& normal,
				const AcGeVector3d& startVector,
				const double sweepAngle,
				const AcGiArcType arcType = kAcGiArcSimple
				) const = 0;

    virtual Adesk::Boolean  circularArc(const AcGePoint3d& start,
				const AcGePoint3d& point,
				const AcGePoint3d& end,
				const AcGiArcType arcType = kAcGiArcSimple
				) const = 0;
      
    virtual Adesk::Boolean  polyline(const Adesk::UInt32 nbPoints,
				const AcGePoint3d* pVertexList
				) const = 0;
      
    virtual Adesk::Boolean  polygon(const Adesk::UInt32 nbPoints,
				const AcGePoint3d* pVertexList
				) const = 0;

    virtual Adesk::Boolean  mesh(const Adesk::UInt32 rows,
				const Adesk::UInt32 columns,
				const AcGePoint3d* pVertexList,
				const AcGiEdgeData* pEdgeData = NULL,
				const AcGiFaceData* pFaceData = NULL,
				const AcGiVertexData* pVertexData = NULL
				) const = 0;

    virtual Adesk::Boolean  shell(const Adesk::UInt32 nbVertex,
				const AcGePoint3d* pVertexList,
				const Adesk::UInt32 faceListSize,
				const Adesk::Int32* pFaceList,
				const AcGiEdgeData* pEdgeData = NULL,
				const AcGiFaceData* pFaceData = NULL,
				const AcGiVertexData* pVertexData = NULL
				) const = 0;
      
    virtual Adesk::Boolean  text(const AcGePoint3d& position,
				const AcGeVector3d& normal,
				const AcGeVector3d& direction,
				const double height,
				const double width,
				const double oblique,
				const char* pMsg
				) const = 0;

    virtual Adesk::Boolean  text(const AcGePoint3d& position,
				const AcGeVector3d& normal,
				const AcGeVector3d& direction,
				const char* pMsg,
				const Adesk::Int32 length,
				const Adesk::Boolean raw,
				const AcGiTextStyle &pTextStyle
				) const = 0;

    virtual Adesk::Boolean  xline(const AcGePoint3d& one_xline_point,
				const AcGePoint3d& a_different_xline_point
				) const = 0;

    virtual Adesk::Boolean  ray(const AcGePoint3d& rays_starting_point,
				const AcGePoint3d& a_different_ray_point
				) const = 0;

    virtual Adesk::Boolean  polylineEye(const Adesk::UInt32 nbPoints,
				const AcGePoint3d* pPoints
				) const = 0;
      
    virtual Adesk::Boolean  polygonEye(const Adesk::UInt32 nbPoints,
				const AcGePoint3d* pPoints
				) const = 0;

    virtual Adesk::Boolean  polylineDc(const Adesk::UInt32 nbPoints,
				const AcGePoint3d* pPoints
				) const = 0;
      
    virtual Adesk::Boolean  polygonDc(const Adesk::UInt32 nbPoints,
				const AcGePoint3d* pPoints
				) const = 0;
};


class AcGiEdgeData: public AcRxObject 
//
// This class contains arrays of edge attributes for meshes and shells.
//
// NOTE:
//   Setting an 'improper' size array of color, layers, etc.
//   will cause unpredictable or fatal results.
//
{ 
public:
    ACRX_DECLARE_MEMBERS(AcGiEdgeData);

    ~AcGiEdgeData();
    AcGiEdgeData();

    virtual void            setColors(const short *colors);
    virtual void            setLayers(const AcDbObjectId *layers);
    virtual void            setLineTypes(const AcDbObjectId *linetypes);
    virtual void            setSelectionMarkers(const Adesk::Int32*
						pSelectionMarkers);
    virtual void            setVisibility(const Adesk::UInt8* pVisibility);

    virtual short*          colors() const;
    virtual AcDbObjectId*   layerIds() const;
    virtual AcDbObjectId*   linetypeIds() const;
    virtual Adesk::Int32*   selectionMarkers() const;
    virtual Adesk::UInt8*   visibility() const;

private:
    AcGiImpEdgeData *mpAcGiImpEdgeData;
};


class AcGiFaceData: public AcRxObject 
//
// This class contains arrays of face attributes for meshes and shells.
//
// NOTE:
//   Setting an 'improper' size array of color, layers, etc.
//   will cause unpredictable or fatal results.
//
{ 
public:
    ACRX_DECLARE_MEMBERS(AcGiFaceData);

    ~AcGiFaceData();
    AcGiFaceData();
    virtual void            setColors(const short *colors);
    virtual void            setLayers(const AcDbObjectId *layers);
    virtual void            setSelectionMarkers(const Adesk::Int32*
						pSelectionMarkers);
    virtual void            setNormals(const AcGeVector3d* pNormals);
    virtual void            setVisibility(const Adesk::UInt8* vis);

    virtual short*          colors() const;
    virtual AcDbObjectId*   layerIds() const;
    virtual Adesk::Int32*   selectionMarkers() const;
    virtual AcGeVector3d*   normals() const;
    virtual Adesk::UInt8*  visibility() const;
private:
    AcGiImpFaceData *mpAcGiImpFaceData;
};


class AcGiVertexData: public AcRxObject 
//
// This class contains arrays of vertex attributes for meshes and shells.
//
// NOTE:
//   Setting an 'improper' size array of color, layers, etc.
//   will cause unpredictable or fatal results.
//
{ 
public:
    ACRX_DECLARE_MEMBERS(AcGiVertexData);
    ~AcGiVertexData();
    AcGiVertexData();
    virtual void            setNormals(const AcGeVector3d* pNormals);
    virtual AcGeVector3d*   normals() const;
    virtual void          setOrientationFlag(const AcGiOrientationType oflag); 
    virtual AcGiOrientationType orientationFlag() const;

private:
    AcGiImpVertexData *mpAcGiImpVertexData;
};


class AcGiTextStyle: public AcRxObject 
// 
// This class contains methods to create and modify a text
// style object
//
{ 
public:
    ACRX_DECLARE_MEMBERS(AcGiTextStyle);
    ~AcGiTextStyle();
    AcGiTextStyle();

    AcGiTextStyle(
	const char* fontName, 
	const char* bigFontName, 
	const double textSize, 
	const double xScale, 
	const double obliqueAngle, 
	const double trPercent,

	const Adesk::Boolean isBackward,
	const Adesk::Boolean isUpsideDown,
	const Adesk::Boolean isVertical, 

	const Adesk::Boolean isOverlined,
	const Adesk::Boolean isUnderlined);

    virtual char loadStyleRec() const;
    virtual void setTextSize(const double size);
    virtual void setXScale(const double xScale);
    virtual void setObliquingAngle(const double obliquingAngle);

    virtual void setTrackingPercent(const double trPercent);

    virtual void setBackward(const Adesk::Boolean isBackward);
    virtual void setUpsideDown(const Adesk::Boolean isUpsideDown);
    virtual void setVertical(const Adesk::Boolean isVertical);

    virtual void setUnderlined(const Adesk::Boolean isUnderlined);
    virtual void setOverlined(const Adesk::Boolean isOverlined);

    virtual void setFileName(const char* fontName);
    virtual void setBigFontFileName(const char* bigFontFileName);

    virtual double textSize() const;
    virtual double xScale() const;
    virtual double obliquingAngle() const;

    virtual double trackingPercent() const;

    virtual Adesk::Boolean isBackward() const;
    virtual Adesk::Boolean isUpsideDown() const;
    virtual Adesk::Boolean isVertical() const;

    virtual Adesk::Boolean isUnderlined() const;
    virtual Adesk::Boolean isOverlined() const;

    virtual const char* fileName() const;
    virtual const char* bigFontFileName() const;

    virtual AcGePoint2d extents(const char* pStr, 
				const Adesk::Boolean penups,
				const int len,
				const Adesk::Boolean raw) const;
private:
    AcGiImpTextStyle *mpAcGiImpTextStyle;
};

#endif
