#ifndef DB_DBAUDITA_H
#define DB_DBAUDITA_H 1

/* Next available MSG number is   1 */
/* $Id: dbaudita.h,v 1.9 1995/09/30 15:26:17 jamesr Exp $ */
/*
      (C) Copyright 1994-1995 by Autodesk, Inc.

******************************************************************************
*                                                                           *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                       *
*                                                                           *
*****************************************************************************
      Header file for auditing routines
*/

#include "dbmain.h"

typedef struct A_Flags *pA_Flags;

class AcDbAuditInfo;
class AcDbImpAuditInfo;
class AcDbAuditImp;

class AcDbAuditInfo
{
public:

	friend AcDbImpAuditInfo;
	friend AcDbAuditImp;

        enum AuditPass {
            PASS1 = 1,
            PASS2 = 2
        };

        AcDbAuditInfo();
	~AcDbAuditInfo();

        void setCoreInfo(struct A_Flags * coreInfo);

	Adesk::Boolean fixErrors(void) const;		// True if errors are to
						     	 // be fixed.

    int numErrors() const;

    int numFixes(void) const;

	void errorsFound(int count);			// Specify the number 	
							// of errors found.

	void errorsFixed(int count);    		// Specify the number of 
						      	// errors fixed.

	AuditPass auditPass(void) const;		// The audit pass
							// number:
						      	// 1 = pass 1,
							// 2 = pass 2.
	void printError(char * name,
	 		char * value, 
	 		char * validation,
	 		char * defaultValue);	// Print an error 
						// message string to the
						// Audit report file.
						// Obeys AUDITCTL.

	operator pA_Flags() const;
							
    void resetNumEntities();
	
    void incNumEntities();

    int numEntities();

    void printNumEntities(char* msg);

     private:

	AcDbImpAuditInfo * getImpAudit() const;


	AcDbImpAuditInfo  * mpImpAudit;
};

//	Class to hold the call back function.

class AcDbRecover{

    public:
    virtual  int callBack(AcDbDatabase*) = 0;
};

class  AcDbImpRecoverCallBack;

class AcDbRecoverCallBack {

    public:
	AcDbRecoverCallBack();
	virtual ~AcDbRecoverCallBack();

    virtual  Acad::ErrorStatus registerCallBack(AcDbRecover*);
    virtual  Acad::ErrorStatus removeCallBack();

    private:

        AcDbImpRecoverCallBack *mpImpRecoverCallBack;

};

#endif
