#ifndef AD_DBCFILERS_H
#define AD_DBCFILERS_H 1
//
// $Id: dbcfilrs.h,v 1.5 1995/10/26 16:13:54 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary
// to Autodesk,  Inc.,	  and considered a trade secret as defined
// in section 499C of the penal code of the State of California.
// Use of  this information  by  anyone  other	  than	authorized
// employees of Autodesk, Inc.	  is granted  only under a written
// non-disclosure agreement,  expressly  prescribing  the  scope
// and manner of such use.
//
// DESCRIPTION:
//
// This file contains "filer" class definitions of AcDbDeepCloneFiler
// and AcDbWblockCloneFiler.  These filers maintain a queue of typed
// object id's. All read/write requests are forwarded to a contained 
// "real" filer, except for object id's which are removed/added from/to 
// the queue.  The classes provide protocol for writing either "owned"
// or "hard" objects in the queue.

#include "dbfiler.h"

class AcDbWblockCloneFiler;
class AcDbImpDeepCloneFiler;

class AcDbDeepCloneFiler: public AcDbDwgFiler
{
    friend class AcDbSystemInternals;
public:
    ACRX_DECLARE_MEMBERS(AcDbDeepCloneFiler);

				AcDbDeepCloneFiler();
    virtual                    ~AcDbDeepCloneFiler();

    // overridden from AcDbDwgFiler 
    //
    virtual Acad::ErrorStatus   filerStatus() const;
    virtual AcDb::FilerType     filerType() const;
    virtual void		setFilerStatus(Acad::ErrorStatus);
    virtual void		resetFilerStatus();

    virtual Acad::ErrorStatus   readHardOwnershipId(AcDbHardOwnershipId*);
    virtual Acad::ErrorStatus   writeHardOwnershipId(
				  const AcDbHardOwnershipId&);

    virtual Acad::ErrorStatus   readSoftOwnershipId(AcDbSoftOwnershipId*);
    virtual Acad::ErrorStatus   writeSoftOwnershipId(
				  const AcDbSoftOwnershipId&);

    virtual Acad::ErrorStatus   readHardPointerId(AcDbHardPointerId*);
    virtual Acad::ErrorStatus   writeHardPointerId(
				  const AcDbHardPointerId&);

    virtual Acad::ErrorStatus   readSoftPointerId(AcDbSoftPointerId*);
    virtual Acad::ErrorStatus   writeSoftPointerId(
				  const AcDbSoftPointerId&);

    virtual Acad::ErrorStatus   readChar(char*);
    virtual Acad::ErrorStatus   writeChar(char);

    virtual Acad::ErrorStatus   readString(char**);
    virtual Acad::ErrorStatus   writeString(const char*);

    virtual Acad::ErrorStatus   readBChunk(ads_binary*);
    virtual Acad::ErrorStatus   writeBChunk(const ads_binary&);

    virtual Acad::ErrorStatus   readAcDbHandle(AcDbHandle*);
    virtual Acad::ErrorStatus   writeAcDbHandle(const AcDbHandle&);

    virtual Acad::ErrorStatus   readInt32(Adesk::Int32*);
    virtual Acad::ErrorStatus   writeInt32(Adesk::Int32);

    virtual Acad::ErrorStatus   readInt16(Adesk::Int16*);
    virtual Acad::ErrorStatus   writeInt16(Adesk::Int16);

    virtual Acad::ErrorStatus   readUInt32(Adesk::UInt32*);
    virtual Acad::ErrorStatus   writeUInt32(Adesk::UInt32);

    virtual Acad::ErrorStatus   readUInt16(Adesk::UInt16*);
    virtual Acad::ErrorStatus   writeUInt16(Adesk::UInt16);

    virtual Acad::ErrorStatus   readUInt8(Adesk::UInt8*);
    virtual Acad::ErrorStatus   writeUInt8(Adesk::UInt8);

    virtual Acad::ErrorStatus   readBoolean(Adesk::Boolean*);
    virtual Acad::ErrorStatus   writeBoolean(Adesk::Boolean);

    virtual Acad::ErrorStatus   readDouble(double*);
    virtual Acad::ErrorStatus   writeDouble(double);

    virtual Acad::ErrorStatus   readPoint2d(AcGePoint2d*);
    virtual Acad::ErrorStatus   writePoint2d(const AcGePoint2d&);

    virtual Acad::ErrorStatus   readPoint3d(AcGePoint3d*);
    virtual Acad::ErrorStatus   writePoint3d(const AcGePoint3d&);

    virtual Acad::ErrorStatus   readVector2d(AcGeVector2d*);
    virtual Acad::ErrorStatus   writeVector2d(const AcGeVector2d&);

    virtual Acad::ErrorStatus   readVector3d(AcGeVector3d*);
    virtual Acad::ErrorStatus   writeVector3d(const AcGeVector3d&);

    virtual Acad::ErrorStatus   readScale3d(AcGeScale3d*);
    virtual Acad::ErrorStatus   writeScale3d(const AcGeScale3d&);

    virtual Acad::ErrorStatus   readBytes(void*, Adesk::UInt32);
    virtual Acad::ErrorStatus   writeBytes(const void*, Adesk::UInt32);

    virtual Acad::ErrorStatus   readAddress(void **);
    virtual Acad::ErrorStatus   writeAddress(const void *);

    virtual Acad::ErrorStatus   seek(long offset, int method);
    virtual long		tell() const;

    virtual Acad::ErrorStatus   addReferences(AcDbIdRefQueue&); // JPR - Remove from API?  AcDbIdRefQueue not defined in API.

    virtual AcDbFilerController& controller() const;

    // protocol specific to AcDbDeepCloneFiler
    //
    virtual Adesk::Boolean	getNextOwnedObject(AcDbObjectId& id);
    virtual Adesk::Boolean	moreOwnedObjects() const;

protected:
    AcDbDeepCloneFiler(AcDbSystemInternals*);

private:
    AcDbImpDeepCloneFiler*      mpImpFiler;
};

class AcDbWblockCloneFiler: public  AcDbDeepCloneFiler
{
    friend class AcDbSystemInternals;
public:
    ACRX_DECLARE_MEMBERS(AcDbWblockCloneFiler);

    AcDbWblockCloneFiler();

    virtual Acad::ErrorStatus   addReferences(AcDbIdRefQueue&); // JPR - AcDbIdRefQueue not defined in API.

    virtual Adesk::Boolean      getNextHardObject(AcDbObjectId& id);
    virtual Adesk::Boolean      moreHardObjects() const;

    virtual AcDb::FilerType     filerType() const;
};

#endif
