#ifndef AD_DBDICT_H
#define AD_DBDICT_H
//
// $Id: dbdict.h,v 1.31 1995/10/26 16:14:18 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary
// to Autodesk,  Inc.,	and considered a trade secret as defined
// in section 499C of the penal code of the State of California.
// Use of  this information  by  anyone  other	than  authorized
// employees of Autodesk, Inc.	is granted  only under a written
// non-disclosure agreement,  expressly  prescribing  the  scope
// and manner of such use.
//
// DESCRIPTION:  Class header for AcDbDictionary, a database-resident
//		 dictionary.  A concerete API class whose implementation
//		 can be inherited at runtime.
//
//		 An instance of this class represents a single
//               "Drawing Symbol Table"-like object, to which
//		 AcDbObject instances can be added, accessed and
//		 removed.
//
//		 Object pointers retrieved from the database represent
//		 indivudual object are currently opened in the requested
//               mode.
//		 The return status indicates the status for opening
//		 the entry object.  If the status is not AcDb::eOpenOK,
//		 the returned pointer is NULL.
//
// CONTRACTS:
//		 Objects added to an AcDbDictionary instance must not
//		 have a presence in the database, i.e. they must have
//		 null handles.
//
//		 Objects removed or replaced in the dictionary are 
//		 erased via AcDbObject::erase().
//

#include "dbmain.h"

class AcDbImpDictionary;

// AcDbDictionaryIterator
// This class allows one to sequentially retrieve the objects in
// the dictionary, and keeps track of what the last object retrieved was.
//
// It is similar to AcRxDictionary Iterator, except the casting
// of returned elements is stronger, and it has a member which
// returns the status of the entry member when the attempt to
// open it was made.  If the status is Acad::ErrorStatus eOk, then
// AcDbDictionaryIterator::getValue() will return a non-null
// pointer, otherwise not.
//
class AcDbDictionaryIterator: public AcRxObject
{
public:
    ACRX_DECLARE_MEMBERS(AcDbDictionaryIterator);
    virtual ~AcDbDictionaryIterator() {}

    virtual const char*	      name	 () const = 0;

    virtual Acad::ErrorStatus getObject  (AcDbObject*&, AcDb::OpenMode) = 0;
    virtual AcDbObjectId      objectId	 () const = 0;

    virtual Adesk::Boolean    done	 () const = 0;
    virtual Adesk::Boolean    next       () = 0;
    virtual Adesk::Boolean    setPosition(AcDbObjectId) = 0;

protected:
    AcDbDictionaryIterator() {}
};

class AcDbDictionary: public AcDbObject
{
public:
    ACDB_DECLARE_MEMBERS(AcDbDictionary);
    AcDbDictionary();
    virtual ~AcDbDictionary();

    Acad::ErrorStatus	getAt  (const char* entryName, AcDbObject*& entryObj,
				AcDb::OpenMode openMode) const;
    Acad::ErrorStatus	getAt  (const char* entryName,
				AcDbObjectId& entryObj) const;

    Acad::ErrorStatus 	nameAt (AcDbObjectId objId, char*& name) const;
 
    Adesk::Boolean   has       (const char * entryName) const;
 
    Adesk::UInt32 numEntries() const;

    AcDbDictionaryIterator *
		     newIterator (AcRx::DictIterType theType =
				  AcRx::kDictSorted);

    Acad::ErrorStatus	remove	(const char * key);
    Acad::ErrorStatus	remove	(const char * key, AcDbObjectId& returnId);
    Acad::ErrorStatus   remove  (AcDbObjectId objId);

    Adesk::Boolean      setName(const char* oldName,
			        const char* newName);

    Acad::ErrorStatus	setAt (const char * srchKey,
			      AcDbObject* newValue,
			      AcDbObjectId& retObjId);

    Adesk::Boolean      isTreatElementsAsHard () const;
    void                setTreatElementsAsHard(Adesk::Boolean doIt);

    // --- AcDbObject Protocol

    virtual Acad::ErrorStatus subErase(Adesk::Boolean pErasing = Adesk::kTrue);

    virtual Acad::ErrorStatus dwgInFields(AcDbDwgFiler* filer);
    virtual Acad::ErrorStatus dwgOutFields(AcDbDwgFiler* filer) const;

    virtual Acad::ErrorStatus dxfInFields(AcDbDxfFiler* filer);
    virtual Acad::ErrorStatus dxfOutFields(AcDbDxfFiler* filer) const;

    // Support for persistant reactor to annotation.
    //
    virtual void              goodbye(const AcDbObject*);
    virtual void              erased (const AcDbObject*,
                                      Adesk::Boolean = Adesk::kTrue);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbDictionary);

#endif
