#ifndef AD_DBENTS_H
#define AD_DBENTS_H
//
// $Id: dbents.h,v 1.55 1995/10/26 16:14:26 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary
// to Autodesk,  Inc.,    and considered a trade secret as defined
// in section 499C of the penal code of the State of California.
// Use of  this information  by  anyone  other    than  authorized
// employees of Autodesk, Inc.    is granted  only under a written
// non-disclosure agreement,  expressly  prescribing  the  scope
// and manner of such use.
//
// DBENTS.H
//
// CREATED: Jon Curtis     10/15/93
// UPDATED:         11/4/93
//             11/21/93 - final pass before first integration
//             4/10/95 - Begin FCS API completion
//
// DESCRIPTION: Exported pre-R13 Entity classes
//
//  AcRxObject
//      AcDbObject
//          AcDbEntity
//
//              AcDbText
//                  AcDbAttribute
//                  AcDbAttributeDefinition
//
//              AcDbBlockBegin
//              AcDbBlockEnd
//              AcDbSequenceEnd
//              AcDbBlockReference
//                  AcDbMInsertBlock
//
//              AcDbVertex
//                  AcDb2dVertex
//                  AcDb3dPolylineVertex
//                  AcDbPolygonMeshVertex
//                  AcDbPolyFaceMeshVertex
//                  AcDbFaceRecord
//
//              AcDbCurve
//                  AcDb2dPolyline
//                  AcDb3dPolyline
//                  AcDbArc
//                  AcDbCircle
//                  AcDbLine
//   
//              AcDbDimension
//                  AcDbOrdinateDimension
//                  AcDbAlignedDimension
//                  AcDbRotatedDimension
//                  AcDb3PointAngularDimension
//                  AcDb2LineAngularDimension
//                  AcDbRadialDimension
//                  AcDbDiametricDimension
//   
//              AcDbPoint
//   
//              AcDbFace
//              AcDbPolyFaceMesh
//              AcDbPolygonMesh
//              AcDbTrace
//              AcDbSolid
//   
//              AcDbShape
//   
//              AcDbViewport

#include "dbmain.h"
#include "dbcurve.h"
#include "gescl3d.h"
#include "gepnt2d.h"
#include "dbmtext.h"

class AcDbAttribute;
class AcDbSequenceEnd;
class AcDb3dPolylineVertex;
class AcDb2dVertex;
class AcGePoint2dArray;
class AcGeDoubleArray;

class AcDbText: public AcDbEntity
{
public:
    AcDbText();
    AcDbText(const AcGePoint3d&  position,
             const char*         text,
                   AcDbObjectId  style    = AcDbObjectId::kNull,
                   double        height   = 0,
                   double        rotation = 0);
    ~AcDbText();
    ACDB_DECLARE_MEMBERS(AcDbText);

    AcGePoint3d         position() const;
    void                setPosition(const AcGePoint3d& pt);

    AcGePoint3d         alignmentPoint() const;
    void                setAlignmentPoint(const AcGePoint3d& pt);
    Adesk::Boolean      isDefaultAlignment() const;

    AcGeVector3d normal() const;
    void setNormal(const AcGeVector3d& newVal);

    double              thickness() const;
    void                setThickness(double newVal);

    double              oblique() const;
    void                setOblique(double newVal);

    double              rotation() const; 
    void                setRotation(double newVal);

    double              height() const; 
    void                setHeight(double newVal);

    double              widthFactor() const; 
    void                setWidthFactor(double newVal);

    char*               textString() const;
    void                setTextString(const char * newText);
     
    AcDbObjectId        textStyle() const;
    void                setTextStyle(AcDbObjectId newVal);
    
    Adesk::Boolean      isMirroredInX() const;
    void                mirrorInX(Adesk::Boolean);
     
    Adesk::Boolean      isMirroredInY() const;
    void                mirrorInY(Adesk::Boolean);
     
    AcDb::TextHorzMode  horizontalMode() const;
    void                setHorizontalMode(AcDb::TextHorzMode newVal);
     
    AcDb::TextVertMode  verticalMode() const;
    void                setVerticalMode(AcDb::TextVertMode newVal);

    //  Correct spelling
    int                 correctSpelling();
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbText);


class AcDbAttribute: public AcDbText
{
public:
    AcDbAttribute();
    AcDbAttribute(const AcGePoint3d&  position,
                  const char*         text,
                  const char*         tag,
                        AcDbObjectId style = AcDbObjectId::kNull);
    ~AcDbAttribute();
    ACDB_DECLARE_MEMBERS(AcDbAttribute);

    char*            tag() const;
    void             setTag(const char* newTag);

    Adesk::Boolean   isInvisible() const;
    void             setInvisible(Adesk::Boolean);

    Adesk::Boolean   isConstant() const;
    Adesk::Boolean   isVerifiable() const;
    Adesk::Boolean   isPreset() const;

    Adesk::UInt16    fieldLength() const;
    void             setFieldLength(Adesk::UInt16 newVal);
 
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbAttribute);


class AcDbAttributeDefinition: public AcDbText
{
public:
    AcDbAttributeDefinition();
    AcDbAttributeDefinition(const AcGePoint3d&  position,
                            const char*         text,
                            const char*         tag,
                            const char*         prompt,
                                  AcDbObjectId style = AcDbObjectId::kNull);
    ~AcDbAttributeDefinition();
    ACDB_DECLARE_MEMBERS(AcDbAttributeDefinition);

    char*            prompt() const;
    void             setPrompt(const char* newPrompt);
     
    char*            tag() const;
    void             setTag(const char* newTag);
 
    Adesk::Boolean   isInvisible() const;
    void             setInvisible(Adesk::Boolean);
 
    Adesk::Boolean   isConstant() const;
    void             setConstant(Adesk::Boolean);
 
    Adesk::Boolean   isVerifiable() const;
    void             setVerifiable(Adesk::Boolean);
 
    Adesk::Boolean   isPreset() const;
    void             setPreset(Adesk::Boolean);
     
    Adesk::UInt16    fieldLength() const;
    void             setFieldLength(Adesk::UInt16 newVal);

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbAttributeDefinition);


class AcDbBlockReference: public AcDbEntity
{
public:
    AcDbBlockReference();
    AcDbBlockReference(const AcGePoint3d&  position,
                             AcDbObjectId  blockTableRec);
    ~AcDbBlockReference();
    ACDB_DECLARE_MEMBERS(AcDbBlockReference);

    AcDbObjectId        blockTableRecord() const;
    Acad::ErrorStatus   setBlockTableRecord(AcDbObjectId);

    AcGePoint3d         position() const;
    void                setPosition(const AcGePoint3d& pt);

    AcGeScale3d         scaleFactors() const;
    void                setScaleFactors(const AcGeScale3d& scale);
 
    double              rotation() const;
    void                setRotation(double newVal);
 
    AcGeVector3d normal() const;
    void setNormal(const AcGeVector3d& newVal);

    AcGeMatrix3d        blockTransform() const;
    Acad::ErrorStatus   setBlockTransform(const AcGeMatrix3d&);

    // Attributes access

    Acad::ErrorStatus appendAttribute(AcDbAttribute* newAttrib);
    Acad::ErrorStatus appendAttribute(AcDbObjectId&  objId,
                                      AcDbAttribute* newAttrib);

    Acad::ErrorStatus openAttribute(AcDbAttribute*& att,
                                    AcDbObjectId    attId,
                                    AcDb::OpenMode  openMode,
                                    Adesk::Boolean  openErasedOne =
                                                    Adesk::kFalse);

    Acad::ErrorStatus openSequenceEnd(AcDbSequenceEnd*& seqend,
                                      AcDb::OpenMode    openMode);

    AcDbObjectIterator*  attributeIterator() const;

    // Subentity acquisition.
    //
    virtual  Acad::ErrorStatus getSubentPathsAtGsMarker(AcDb::SubentType type,
	int gsMark, const AcGePoint3d& pickPoint,
	const AcGeMatrix3d& viewXform, int& numPaths,
	AcDbFullSubentPath* & subentPaths, int numInserts = 0,
	AcDbObjectId* entAndInsertStack = NULL) const;

    virtual  Acad::ErrorStatus getGsMarkersAtSubentPath(
	const AcDbFullSubentPath& subPath, AcDbIntArray& gsMarkers) const;

    virtual Acad::ErrorStatus highlight( const AcDbFullSubentPath& subId = 
					  kNullSubent) const;

    virtual Acad::ErrorStatus unhighlight( const AcDbFullSubentPath& subId = 
					  kNullSubent) const;

    virtual AcDbEntity*       subentPtr(const AcDbFullSubentPath& id) const;
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbBlockReference);

class AcDbMInsertBlock: public AcDbBlockReference
{
public:
    AcDbMInsertBlock();
    AcDbMInsertBlock(const AcGePoint3d&  position,
                           AcDbObjectId  blockTableRec,
                           Adesk::UInt16 columns,
                           Adesk::UInt16 rows,
                           Adesk::UInt16 colSpacing,
                           Adesk::UInt16 rowSpacing);
    ~AcDbMInsertBlock();
    ACDB_DECLARE_MEMBERS(AcDbMInsertBlock);

    Adesk::UInt16    columns() const;
    void             setColumns(Adesk::UInt16 newVal);
 
    Adesk::UInt16    rows() const;
    void             setRows(Adesk::UInt16 newVal);
 
    double           columnSpacing() const;
    void             setColumnSpacing(double newVal);
 
    double           rowSpacing() const;
    void             setRowSpacing(double newVal);

};


class AcDbBlockBegin: public AcDbEntity
{
public:
    AcDbBlockBegin();
    ~AcDbBlockBegin();
    ACDB_DECLARE_MEMBERS(AcDbBlockBegin);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbBlockBegin);

class AcDbBlockEnd: public AcDbEntity
{
public:
    AcDbBlockEnd();
    ~AcDbBlockEnd();
    ACDB_DECLARE_MEMBERS(AcDbBlockEnd);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbBlockEnd);

class AcDbSequenceEnd: public AcDbEntity
{
public:
    AcDbSequenceEnd();
    ~AcDbSequenceEnd();
    ACDB_DECLARE_MEMBERS(AcDbSequenceEnd);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbSequenceEnd);

class AcDbVertex: public AcDbEntity
{
public:
    AcDbVertex();
    ~AcDbVertex();
    ACDB_DECLARE_MEMBERS(AcDbVertex);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbVertex);

class AcDb2dVertex: public AcDbVertex
{
public:
    AcDb2dVertex();
    ~AcDb2dVertex();
    ACDB_DECLARE_MEMBERS(AcDb2dVertex);

	    AcDb2dVertex(
			 const AcGePoint3d&  position,
			 double   bulge      = 0,
			 double   startWidth = 0,
			 double   endWidth   = 0,
			 double   tangent    = 0);

    AcDb::Vertex2dType vertexType() const;

    // This method accesses the position in ECS.  See AcDb2dPolyline for WCS.
    AcGePoint3d      position() const;
    void             setPosition(const AcGePoint3d& pt); 

    double           startWidth() const; 
    void             setStartWidth(double newVal);

    double           endWidth() const; 
    void             setEndWidth(double newVal);

    double           bulge() const; 
    void             setBulge(double newVal);

    Adesk::Boolean   isTangentUsed() const;
    void             useTangent();
    void ignoreTangent();

    double           tangent() const; 
    void             setTangent(double newVal);

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDb2dVertex);


class AcDb3dPolylineVertex: public AcDbVertex
{
public:
    AcDb3dPolylineVertex();
    ~AcDb3dPolylineVertex();
    ACDB_DECLARE_MEMBERS(AcDb3dPolylineVertex);

                     AcDb3dPolylineVertex(const AcGePoint3d&  position);

    AcDb::Vertex3dType vertexType() const;

    AcGePoint3d      position() const;
    void             setPosition(const AcGePoint3d& pt); 

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDb3dPolylineVertex);


class AcDbPolygonMeshVertex: public AcDbVertex
{
public:
    AcDbPolygonMeshVertex();
    AcDbPolygonMeshVertex(const AcGePoint3d&  position);
    ~AcDbPolygonMeshVertex();
    ACDB_DECLARE_MEMBERS(AcDbPolygonMeshVertex);

    AcDb::Vertex3dType vertexType() const;

    AcGePoint3d      position() const;
    void             setPosition(const AcGePoint3d& pt); 

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbPolygonMeshVertex);


class AcDbPolyFaceMeshVertex: public AcDbVertex
{
public:
    AcDbPolyFaceMeshVertex();
    AcDbPolyFaceMeshVertex(const AcGePoint3d&  position);
    ~AcDbPolyFaceMeshVertex();
    ACDB_DECLARE_MEMBERS(AcDbPolyFaceMeshVertex);

    AcDb::Vertex3dType vertexType() const;

    AcGePoint3d      position() const;
    void             setPosition(const AcGePoint3d& pt); 

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbPolyFaceMeshVertex);


class AcDbFaceRecord: public AcDbVertex
{
public:
    AcDbFaceRecord();
    // Use negative values to indicate invisible faces
    AcDbFaceRecord(Adesk::Int16   vtx0,
		   Adesk::Int16   vtx1,
		   Adesk::Int16   vtx2,
		   Adesk::Int16   vtx3);
    ~AcDbFaceRecord();
    ACDB_DECLARE_MEMBERS(AcDbFaceRecord);

    // input index must be 0-3.
    Acad::ErrorStatus getVertexAt(Adesk::UInt16 faceIndex, Adesk::Int16& vtxIndex) const;
    Acad::ErrorStatus setVertexAt(Adesk::UInt16 faceIndex, Adesk::Int16  vtxIndex);

    Acad::ErrorStatus    isEdgeVisibleAt(Adesk::UInt16   faceIndex,
                                         Adesk::Boolean& visible) const;

    Acad::ErrorStatus    makeEdgeVisibleAt(Adesk::UInt16   faceIndex);
    Acad::ErrorStatus    makeEdgeInvisibleAt(Adesk::UInt16 faceIndex);

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbFaceRecord);


class AcDb2dPolyline: public AcDbCurve
{
public:
    AcDb2dPolyline();
    ~AcDb2dPolyline();
    ACDB_DECLARE_MEMBERS(AcDb2dPolyline);

    AcDb2dPolyline (AcDb::Poly2dType type,
                    AcGePoint3dArray&   vertices,
		    double           elevation     = 0,
		    Adesk::Boolean   closed        = Adesk::kFalse,
		    double           defStartWidth = 0,
		    double           defEndWidth   = 0,
		    AcGeDoubleArray* bulges        = NULL);

    DBCURVE_METHODS

    AcDb::Poly2dType  polyType() const;
    void              setPolyType(AcDb::Poly2dType newVal);

    void              makeClosed();
    void              makeOpen();
 
    double            defaultStartWidth() const;
    void              setDefaultStartWidth(double newVal);

    double            defaultEndWidth() const;
    void              setDefaultEndWidth(double newVal);

    double            thickness() const;
    void              setThickness(double newVal);

    AcGeVector3d normal() const;
    void setNormal(const AcGeVector3d& newVal);

    double            elevation() const;
    void              setElevation(double newVal);

    Adesk::Boolean    isLinetypeGenerationOn() const;
    void              setLinetypeGenerationOn();
    void              setLinetypeGenerationOff();

    // Vertex access
    Acad::ErrorStatus appendVertex(AcDb2dVertex* newVert);
    Acad::ErrorStatus appendVertex(AcDbObjectId& objId,
                                   AcDb2dVertex* newVert);

    Acad::ErrorStatus openVertex(AcDb2dVertex*&  vert,
                                 AcDbObjectId    vertId,
                                 AcDb::OpenMode  openMode,
                                 Adesk::Boolean  openErasedOne =
                                                 Adesk::kFalse);

    Acad::ErrorStatus openSequenceEnd(AcDbSequenceEnd*& seqend,
                                      AcDb::OpenMode    openMode);

    AcDbObjectIterator*  vertexIterator() const;

    // vertex position in WCS
    AcGePoint3d       vertexPosition(const AcDb2dVertex& vert) const;
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDb2dPolyline);


class AcDb3dPolyline: public AcDbCurve
{
public:
    AcDb3dPolyline();
    ~AcDb3dPolyline();
    ACDB_DECLARE_MEMBERS(AcDb3dPolyline);

      AcDb3dPolyline(AcDb::Poly3dType  type,
		     AcGePoint3dArray& vertices,
		     Adesk::Boolean    closed = Adesk::kFalse);

    DBCURVE_METHODS

    void              makeClosed();
    void              makeOpen();
 
    AcDb::Poly3dType  polyType() const;
    void              setPolyType(AcDb::Poly3dType newVal);

    // Vertex access
    Acad::ErrorStatus appendVertex(AcDb3dPolylineVertex* newVert);
    Acad::ErrorStatus appendVertex(AcDbObjectId&         objId,
                                   AcDb3dPolylineVertex* newVert);

    Acad::ErrorStatus openVertex(AcDb3dPolylineVertex*&  vert,
                                 AcDbObjectId    vertId,
                                 AcDb::OpenMode  openMode,
                                 Adesk::Boolean  openErasedOne =
                                                 Adesk::kFalse);

    Acad::ErrorStatus openSequenceEnd(AcDbSequenceEnd*& seqend,
                                      AcDb::OpenMode    openMode);

    AcDbObjectIterator* vertexIterator() const;

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDb3dPolyline);


class AcDbArc: public AcDbCurve
{
public:
    AcDbArc();
    AcDbArc(const AcGePoint3d&  center,
                  double        radius,
                  double        startAngle,
                  double        endAngle);
    ~AcDbArc();
    ACDB_DECLARE_MEMBERS(AcDbArc);


    DBCURVE_METHODS

    AcGePoint3d          center() const;
    void                 setCenter(const AcGePoint3d& pt); 

    double               radius() const;
    void                 setRadius(double newVal);

    double               startAngle() const;
    void                 setStartAngle(double newVal);

    double               endAngle() const;
    void                 setEndAngle(double newVal);

    double               thickness() const;
    void                 setThickness(double newVal);

    AcGeVector3d normal() const;
    void setNormal(const AcGeVector3d& newVal);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbArc);


class AcDbCircle: public AcDbCurve
{
public:
    AcDbCircle();
    AcDbCircle(const AcGePoint3d&  center,
	       const AcGeVector3d& normal,
                     double        radius);
    ~AcDbCircle();
    ACDB_DECLARE_MEMBERS(AcDbCircle);

    DBCURVE_METHODS

    AcGePoint3d          center() const;
    void                 setCenter(const AcGePoint3d& pt); 

    double               radius() const;
    void                 setRadius(double newVal);

    double               thickness() const;
    void                 setThickness(double newVal);

    AcGeVector3d normal() const;
    void setNormal(const AcGeVector3d& newVal);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbCircle);


class AcDbLine: public AcDbCurve
{
public:
    AcDbLine();
    AcDbLine(const AcGePoint3d&  start,
             const AcGePoint3d&  end);
    ~AcDbLine();
    ACDB_DECLARE_MEMBERS(AcDbLine);

    DBCURVE_METHODS

    AcGePoint3d          startPoint() const;
    void                 setStartPoint(const AcGePoint3d&); 

    AcGePoint3d          endPoint() const;
    void                 setEndPoint(const AcGePoint3d&); 

    double               thickness() const;
    void                 setThickness(double);

    AcGeVector3d normal() const;
    void setNormal(const AcGeVector3d&);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbLine);


class AcDbDimension: public AcDbEntity
{
public:
    AcDbDimension();
    ~AcDbDimension();
    ACDB_DECLARE_MEMBERS(AcDbDimension);

    AcGePoint3d      textPosition() const; 
    void             setTextPosition(const AcGePoint3d& pt); 

    Adesk::Boolean   isUsingDefaultTextPosition() const;
    void             useSetTextPosition();
    void             useDefaultTextPosition();

    AcGeVector3d normal() const;
    void setNormal(const AcGeVector3d& newVal);

    char*            dimensionText() const; 
    void             setDimensionText(const char* newStr);
     
    double           textRotation() const;
    void             setTextRotation(double newVal);
 
    AcDbObjectId     dimensionStyle() const;
    void             setDimensionStyle(AcDbObjectId newStyle);
 
    double           horizontalRotation() const;
    void             setHorizontalRotation(double newVal);
 
    // Functions for accessing the Dimension's defining block
    //
    AcDbObjectId     dimBlockId() const;
    void             setDimBlockId(const AcDbObjectId&);
    AcGePoint3d      dimBlockPosition() const;
    void             setDimBlockPosition(const AcGePoint3d&);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbDimension);

class AcDbOrdinateDimension: public AcDbDimension
{
public:
    AcDbOrdinateDimension();
    AcDbOrdinateDimension(Adesk::Boolean useXAxis,
			  const AcGePoint3d&   definingPoint,
			  const AcGePoint3d&   leaderEndPoint,
			  char*          dimText        = NULL,
			  AcDbObjectId   dimStyle       = AcDbObjectId::kNull);
    ~AcDbOrdinateDimension();
    ACDB_DECLARE_MEMBERS(AcDbOrdinateDimension);

    Adesk::Boolean   isUsingXAxis() const;
    Adesk::Boolean   isUsingYAxis() const;
    void             useXAxis();
    void             useYAxis();

    AcGePoint3d      origin() const;
    void             setOrigin(const AcGePoint3d& pt);

    AcGePoint3d      definingPoint() const; 
    void             setDefiningPoint(const AcGePoint3d& pt); 

    AcGePoint3d      leaderEndPoint() const; 
    void             setLeaderEndPoint(const AcGePoint3d& pt); 

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbOrdinateDimension);


class AcDbRotatedDimension: public AcDbDimension
{
public:
    AcDbRotatedDimension();
    AcDbRotatedDimension(double         rotation,
			 const AcGePoint3d&   xLine1Point,
			 const AcGePoint3d&   xLine2Point,
			 const AcGePoint3d&   dimLinePoint,
			 char*          dimText      = NULL,
			 AcDbObjectId  dimStyle      = AcDbObjectId::kNull);
    ~AcDbRotatedDimension();
    ACDB_DECLARE_MEMBERS(AcDbRotatedDimension);

    AcGePoint3d      xLine1Point() const; 
    void             setXLine1Point(const AcGePoint3d& pt); 

    AcGePoint3d      xLine2Point() const; 
    void             setXLine2Point(const AcGePoint3d& pt); 

    AcGePoint3d      dimLinePoint() const; 
    void             setDimLinePoint(const AcGePoint3d& pt); 

    double           oblique() const;
    void             setOblique(double newVal);

    double           rotation() const;
    void             setRotation(double newVal);

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbRotatedDimension);
    

class AcDbAlignedDimension: public AcDbDimension
{
public:
    AcDbAlignedDimension();
    AcDbAlignedDimension(
                    const AcGePoint3d&   xLine1Point,
                    const AcGePoint3d&   xLine2Point,
                    const AcGePoint3d&   dimLinePoint,
                    char*          dimText      = NULL,
                    AcDbObjectId  dimStyle      = AcDbObjectId::kNull);
    ~AcDbAlignedDimension();
    ACDB_DECLARE_MEMBERS(AcDbAlignedDimension);

    AcGePoint3d      xLine1Point() const; 
    void             setXLine1Point(const AcGePoint3d& pt); 

    AcGePoint3d      xLine2Point() const; 
    void             setXLine2Point(const AcGePoint3d& pt); 

    AcGePoint3d      dimLinePoint() const; 
    void             setDimLinePoint(const AcGePoint3d& pt); 

    double           oblique() const;
    void             setOblique(double newVal);

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbAlignedDimension);

class AcDb3PointAngularDimension: public AcDbDimension
{
public:
    AcDb3PointAngularDimension();
    AcDb3PointAngularDimension(
                     const AcGePoint3d&   centerPoint,
                     const AcGePoint3d&   xLine1Point,
                     const AcGePoint3d&   xLine2Point,
                     const AcGePoint3d&   arcPoint,
                     char*          dimText     = NULL,
                     AcDbObjectId  dimStyle     = AcDbObjectId::kNull);
    ~AcDb3PointAngularDimension();
    ACDB_DECLARE_MEMBERS(AcDb3PointAngularDimension);

    AcGePoint3d      arcPoint() const; 
    void             setArcPoint(const AcGePoint3d& pt); 

    AcGePoint3d      xLine1Point() const; 
    void             setXLine1Point(const AcGePoint3d& pt); 

    AcGePoint3d      xLine2Point() const; 
    void             setXLine2Point(const AcGePoint3d& pt); 

    AcGePoint3d      centerPoint() const; 
    void             setCenterPoint(const AcGePoint3d& pt); 

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDb3PointAngularDimension);


class AcDb2LineAngularDimension: public AcDbDimension
{
public:
    AcDb2LineAngularDimension();
    AcDb2LineAngularDimension(
                     const AcGePoint3d&   xLine1Start,
                     const AcGePoint3d&   xLine1End,
                     const AcGePoint3d&   xLine2Start,
                     const AcGePoint3d&   xLine2End,
                     const AcGePoint3d&   arcPoint,
                     char*          dimText     = NULL,
                     AcDbObjectId   dimStyle    = AcDbObjectId::kNull);
    ~AcDb2LineAngularDimension();
    ACDB_DECLARE_MEMBERS(AcDb2LineAngularDimension);

    AcGePoint3d      arcPoint() const; 
    void             setArcPoint(const AcGePoint3d& pt); 

    AcGePoint3d      xLine1Start() const; 
    void             setXLine1Start(const AcGePoint3d& pt); 

    AcGePoint3d      xLine1End() const; 
    void             setXLine1End(const AcGePoint3d& pt); 

    AcGePoint3d      xLine2Start() const; 
    void             setXLine2Start(const AcGePoint3d& pt); 

    AcGePoint3d      xLine2End() const; 
    void             setXLine2End(const AcGePoint3d& pt); 

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDb2LineAngularDimension);


class AcDbRadialDimension: public AcDbDimension
{
public:
    AcDbRadialDimension();
    AcDbRadialDimension(
                     const AcGePoint3d&   center,
                     const AcGePoint3d&   chordPoint,
                     double         leaderLength,
                     char*          dimText       = NULL,
                     AcDbObjectId   dimStyle      = AcDbObjectId::kNull);
    ~AcDbRadialDimension();
    ACDB_DECLARE_MEMBERS(AcDbRadialDimension);

    double           leaderLength() const;
    void             setLeaderLength(double newVal);

    AcGePoint3d      center() const;
    void             setCenter(const AcGePoint3d& pt);

    AcGePoint3d      chordPoint() const;
    void             setChordPoint(const AcGePoint3d& pt);

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbRadialDimension);


class AcDbDiametricDimension: public AcDbDimension
{
public:
    AcDbDiametricDimension();
    AcDbDiametricDimension(
                     const AcGePoint3d&   chordPoint,
                     const AcGePoint3d&   farChordPoint,
                     double         leaderLength,
                     char*          dimText        = NULL,
                     AcDbObjectId   dimStyle       = AcDbObjectId::kNull);
    ~AcDbDiametricDimension();
    ACDB_DECLARE_MEMBERS(AcDbDiametricDimension);

    double           leaderLength() const;
    void             setLeaderLength(double newVal);

    AcGePoint3d      chordPoint() const;
    void             setChordPoint(const AcGePoint3d& pt);

    AcGePoint3d      farChordPoint() const;
    void             setFarChordPoint(const AcGePoint3d& pt);

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbDiametricDimension);


class AcDbPoint: public AcDbEntity
{
public:
    AcDbPoint();
    AcDbPoint(const AcGePoint3d&   position);
    ~AcDbPoint();
    ACDB_DECLARE_MEMBERS(AcDbPoint);

    AcGePoint3d       position() const;
    void              setPosition(const AcGePoint3d& pt);

    double            thickness() const;
    void              setThickness(double newVal);

    AcGeVector3d normal() const;
    void setNormal(const AcGeVector3d& newVal);

    double            ecsRotation() const;
    void              setEcsRotation(double newVal);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbPoint);


class AcDbFace: public AcDbEntity
{
public:                
    AcDbFace();
    AcDbFace(const AcGePoint3d&   pt0,
             const AcGePoint3d&   pt1,
             const AcGePoint3d&   pt2,
             const AcGePoint3d&   pt3,
                   Adesk::Boolean e0vis = Adesk::kTrue,
                   Adesk::Boolean e1vis = Adesk::kTrue,
                   Adesk::Boolean e2vis = Adesk::kTrue,
                   Adesk::Boolean e3vis = Adesk::kTrue);
    AcDbFace(const AcGePoint3d&   pt0,
             const AcGePoint3d&   pt1,
             const AcGePoint3d&   pt2,
                   Adesk::Boolean e0vis = Adesk::kTrue,
                   Adesk::Boolean e1vis = Adesk::kTrue,
                   Adesk::Boolean e2vis = Adesk::kTrue,
                   Adesk::Boolean e3vis = Adesk::kTrue);
    ~AcDbFace();
    ACDB_DECLARE_MEMBERS(AcDbFace);

    Acad::ErrorStatus getVertexAt(Adesk::UInt16, AcGePoint3d&) const;
    Acad::ErrorStatus setVertexAt(Adesk::UInt16, const AcGePoint3d&);

    Acad::ErrorStatus isEdgeVisibleAt(Adesk::UInt16, Adesk::Boolean&) const;
    Acad::ErrorStatus makeEdgeVisibleAt(Adesk::UInt16);
    Acad::ErrorStatus makeEdgeInvisibleAt(Adesk::UInt16);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbFace);


class AcDbPolyFaceMesh: public AcDbEntity
{
public:
    AcDbPolyFaceMesh();
    ~AcDbPolyFaceMesh();
    ACDB_DECLARE_MEMBERS(AcDbPolyFaceMesh);

    Adesk::Int16 numVertices() const;
    Adesk::Int16 numFaces() const;

    Acad::ErrorStatus appendVertex(AcDbPolyFaceMeshVertex* newVert);
    Acad::ErrorStatus appendVertex(AcDbObjectId&           objId,
                                   AcDbPolyFaceMeshVertex* newVert);

    Acad::ErrorStatus appendFaceRecord(AcDbFaceRecord* newFace);
    Acad::ErrorStatus appendFaceRecord(AcDbObjectId&   objId,
                                       AcDbFaceRecord* newFace);

    Acad::ErrorStatus openVertex(AcDbVertex*&      subObj,
                                 AcDbObjectId      subObjId,
                                 AcDb::OpenMode    openMode,
                                 Adesk::Boolean    openErasedOne =
                                                       Adesk::kFalse);

    Acad::ErrorStatus openSequenceEnd(AcDbSequenceEnd*& seqend,
                                      AcDb::OpenMode    openMode);

    AcDbObjectIterator*  vertexIterator() const;
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbPolyFaceMesh);


class AcDbPolygonMesh: public AcDbEntity
{
public:
    AcDbPolygonMesh();
    AcDbPolygonMesh(AcDb::PolyMeshType pType,
                    Adesk::Int16       mSize,
                    Adesk::Int16       nSize,
                    const AcGePoint3dArray& vertices,
                    Adesk::Boolean     mClosed = Adesk::kTrue,
                    Adesk::Boolean     nClosed = Adesk::kTrue);
    ~AcDbPolygonMesh();
    ACDB_DECLARE_MEMBERS(AcDbPolygonMesh);

    AcDb::PolyMeshType polyMeshType() const;
    void             setPolyMeshType(AcDb::PolyMeshType newVal);

    Adesk::Int16     mSize() const;
    void             setMSize(Adesk::Int16 newVal);

    Adesk::Int16     nSize() const;
    void             setNSize(Adesk::Int16 newVal);
 
    Adesk::Boolean   isMClosed() const;
    void             makeMClosed();
    void             makeMOpen();
 
    Adesk::Boolean   isNClosed() const;
    void             makeNClosed();
    void             makeNOpen();
 
    Adesk::Int16     mSurfaceDensity() const;
    void             setMSurfaceDensity(Adesk::Int16);

    Adesk::Int16     nSurfaceDensity() const;
    void             setNSurfaceDensity(Adesk::Int16);

    // Vertex access

    Acad::ErrorStatus appendVertex(AcDbPolygonMeshVertex* newVert);
    Acad::ErrorStatus appendVertex(AcDbObjectId&          objId,
                                   AcDbPolygonMeshVertex* newVert);

    Acad::ErrorStatus openVertex(AcDbPolygonMeshVertex*&  vert,
                                 AcDbObjectId             vertId,
                                 AcDb::OpenMode           openMode,
                                 Adesk::Boolean           openErasedOne =
                                                          Adesk::kFalse);

    Acad::ErrorStatus openSequenceEnd(AcDbSequenceEnd*& seqend,
                                      AcDb::OpenMode    openMode);

    AcDbObjectIterator*  vertexIterator() const;

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbPolygonMesh);


class AcDbSolid: public AcDbEntity
{
public:
    AcDbSolid();
    AcDbSolid(const AcGePoint3d&    pt0,
              const AcGePoint3d&    pt1,
              const AcGePoint3d&    pt2,
              const AcGePoint3d&    pt3);
    AcDbSolid(const AcGePoint3d&    pt0,
              const AcGePoint3d&    pt1,
              const AcGePoint3d&    pt2);
    ~AcDbSolid();
    ACDB_DECLARE_MEMBERS(AcDbSolid);

    Acad::ErrorStatus getPointAt(      Adesk::UInt16 index, 
                                       AcGePoint3d&  PntRes) const;
    Acad::ErrorStatus setPointAt(      Adesk::UInt16 index, 
                                 const AcGePoint3d&  pt);

    double           thickness() const;
    void             setThickness(double newVal);

    AcGeVector3d normal() const;
    void setNormal(const AcGeVector3d& newVal);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbSolid);


class AcDbTrace: public AcDbEntity
{
public:
    AcDbTrace();
    AcDbTrace(const AcGePoint3d&    pt0,
              const AcGePoint3d&    pt1,
              const AcGePoint3d&    pt2,
              const AcGePoint3d&    pt3);
    ~AcDbTrace();
    ACDB_DECLARE_MEMBERS(AcDbTrace);

    // returns eInvalidIndex if index is out of range
    Acad::ErrorStatus getPointAt(      Adesk::UInt16 index, 
                                       AcGePoint3d&  PntRes) const;
    Acad::ErrorStatus setPointAt(      Adesk::UInt16 index, 
                                 const AcGePoint3d&  pt);

    double           thickness() const;
    void             setThickness(double newVal);

    AcGeVector3d normal() const;
    void setNormal(const AcGeVector3d& newVal);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbTrace);


class AcDbShape: public AcDbEntity
{
public:
    AcDbShape();
    AcDbShape(const AcGePoint3d&  position,
                    double        size,
              const char*         name,
                    double        rotation    = 0,
                    double        widthFactor = 0);
    ~AcDbShape();
    ACDB_DECLARE_MEMBERS(AcDbShape);

    AcGePoint3d      position() const;
    void             setPosition(const AcGePoint3d& pt); 
    
    double           size() const;
    void             setSize(double newVal);
    
    char*            name() const;
    Acad::ErrorStatus setName(const char* newVal); 
    
    double           rotation() const;
    void             setRotation(double newVal);
    
    double           widthFactor() const;
    void             setWidthFactor(double newVal);
    
    double           oblique() const;
    void             setOblique(double newVal);

    double           thickness() const;
    void             setThickness(double newVal);

    AcGeVector3d normal() const;
    void setNormal(const AcGeVector3d& newVal);
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbShape);


class AcDbViewport: public AcDbEntity
{
public:
    AcDbViewport();
    ~AcDbViewport();
    ACDB_DECLARE_MEMBERS(AcDbViewport);

    // Set methods will return eCannotChangeActiveViewport if called
    // when the Viewport is currently active.  

    double            height() const;
    Acad::ErrorStatus setHeight(double newVal);

    double            width() const;
    Acad::ErrorStatus setWidth(double newVal);

    AcGePoint3d       centerPoint() const;
    Acad::ErrorStatus setCenterPoint(const AcGePoint3d& pt);

    // Returns -1 if Viewport is inactive.
    Adesk::Int16      number() const;

    Acad::ErrorStatus setOn();
    Acad::ErrorStatus setOff();

    AcGePoint3d       viewTarget() const;
    Acad::ErrorStatus setViewTarget(const AcGePoint3d& pt); 

    AcGeVector3d      viewDirection() const;
    Acad::ErrorStatus setViewDirection(const AcGeVector3d& vec);

    // Model Space height, and center in Display coordinates
    double            viewHeight() const;
    AcGePoint2d       viewCenter() const;

    double            twistAngle() const;
    Acad::ErrorStatus setTwistAngle(double newVal);

    double            lensLength() const;
    Acad::ErrorStatus setLensLength(double newVal);

    Adesk::Boolean    isFrontClipOn() const;
    Acad::ErrorStatus setFrontClipOn();
    Acad::ErrorStatus setFrontClipOff();

    Adesk::Boolean    isBackClipOn() const;
    Acad::ErrorStatus setBackClipOn();
    Acad::ErrorStatus setBackClipOff();

    Adesk::Boolean    isFrontClipAtEyeOn() const;
    Acad::ErrorStatus setFrontClipAtEyeOn();
    Acad::ErrorStatus setFrontClipAtEyeOff();

    // FrontZ 
    double            frontClipDistance() const;
    Acad::ErrorStatus setFrontClipDistance(double newVal);

    // BackZ 
    double            backClipDistance() const;
    Acad::ErrorStatus setBackClipDistance(double newVal);

    Adesk::Boolean    isPerspectiveOn() const;
    Acad::ErrorStatus setPerspectiveOn();
    Acad::ErrorStatus setPerspectiveOff();

    Adesk::Boolean    isUcsFollowModeOn() const;
    Acad::ErrorStatus setUcsFollowModeOn();
    Acad::ErrorStatus setUcsFollowModeOff();

    Adesk::Boolean    isUcsIconVisible() const;
    Acad::ErrorStatus setUcsIconVisible();
    Acad::ErrorStatus setUcsIconInvisible();

    Adesk::Boolean    isUcsIconAtOrigin() const;
    Acad::ErrorStatus setUcsIconAtOrigin();
    Acad::ErrorStatus setUcsIconAtCorner();

    Adesk::Boolean    isFastZoomOn() const;
    Acad::ErrorStatus setFastZoomOn();
    Acad::ErrorStatus setFastZoomOff();

    Adesk::UInt16     circleSides() const;
    Acad::ErrorStatus setCircleSides(Adesk::UInt16 newVal);

    Adesk::Boolean    isSnapOn() const;
    Acad::ErrorStatus setSnapOn();
    Acad::ErrorStatus setSnapOff();

    Adesk::Boolean    isSnapIsometric() const;
    Acad::ErrorStatus setSnapIsometric();
    Acad::ErrorStatus setSnapStandard();

    double            snapAngle() const;
    Acad::ErrorStatus setSnapAngle(double newVal);

    AcGePoint2d       snapBasePoint() const;
    Acad::ErrorStatus setSnapBasePoint(const AcGePoint2d& pt); 

    AcGeVector2d      snapIncrement() const;
    Acad::ErrorStatus setSnapIncrement(const AcGeVector2d& vec);

    Adesk::UInt16     snapIsoPair() const;
    Acad::ErrorStatus setSnapIsoPair(Adesk::UInt16 newVal);

    Adesk::Boolean    isGridOn() const;
    Acad::ErrorStatus setGridOn();
    Acad::ErrorStatus setGridOff();

    AcGeVector2d      gridIncrement() const;
    Acad::ErrorStatus setGridIncrement(const AcGeVector2d& vec);

    Adesk::Boolean    hiddenLinesRemoved() const;
    Acad::ErrorStatus showHiddenLines();
    Acad::ErrorStatus removeHiddenLines();
    
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbViewport);

#endif
