#ifndef AD_DBMLINESTYLE_H
#define AD_DBMLINESTYLE_H

//
// $Id: dbmstyle.h,v 1.14 1995/09/21 11:16:20 jamesr Exp $ 
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary
// to Autodesk,  Inc.,  and considered a trade secret as defined
// in section 499C of the penal code of the State of California.
// Use of  this information  by  anyone  other  than  authorized
// employees of Autodesk, Inc.  is granted  only under a written
// non-disclosure agreement,  expressly  prescribing  the  scope
// and manner of such use.
//
// DBMSTYLE.H
//
#ifndef AD_DBMAIN_H
#include "dbmain.h"
#endif

#define MSTYLE_NAME_LENGTH 32
#define MSTYLE_DESCRIPTION_LENGTH 256
#define LTYPE_LENGTH 32
#define MLINE_MAX_ELEMENTS 16
#define MIN_ANGLE (10.0 * PI / 180.0)   // 10 degreees
#define MAX_ANGLE (170.0 * PI / 180.0)  // 170 degrees

#define MSTYLE_DXF_FILL_ON           (0x1)
#define MSTYLE_DXF_SHOW_MITERS       (0x2)
#define MSTYLE_DXF_START_SQUARE_CAP  (0x10)
#define MSTYLE_DXF_START_INNER_ARCS  (0x20)
#define MSTYLE_DXF_START_ROUND_CAP   (0x40)
#define MSTYLE_DXF_END_SQUARE_CAP    (0x100)
#define MSTYLE_DXF_END_INNER_ARCS    (0x200)
#define MSTYLE_DXF_END_ROUND_CAP     (0x400)
#define MSTYLE_DXF_JUST_TOP          (0x1000)
#define MSTYLE_DXF_JUST_ZERO         (0x2000)
#define MSTYLE_DXF_JUST_BOT          (0x4000)

class AcDbMlineStyle: public  AcDbObject
{

  public:
    ACDB_DECLARE_MEMBERS(AcDbMlineStyle);

    AcDbMlineStyle();
    virtual ~AcDbMlineStyle();

    void initMlineStyle();
    void set(const AcDbMlineStyle & src);

    Acad::ErrorStatus setDescription(const char * description);
    const char * description() const;

    Acad::ErrorStatus setName(const char * name);
    const char * name() const;

    void setShowMiters(Adesk::Boolean showThem);
    Adesk::Boolean showMiters() const;

    void setStartSquareCap(Adesk::Boolean showIt);
    Adesk::Boolean startSquareCap() const;

    void setStartRoundCap(Adesk::Boolean showIt);
    Adesk::Boolean startRoundCap() const;

    void setStartInnerArcs(Adesk::Boolean showThem);
    Adesk::Boolean startInnerArcs() const;

    void setEndSquareCap(Adesk::Boolean showIt);
    Adesk::Boolean endSquareCap() const;

    void setEndRoundCap(Adesk::Boolean showIt);
    Adesk::Boolean endRoundCap() const;

    void setEndInnerArcs(Adesk::Boolean showThem);
    Adesk::Boolean endInnerArcs() const;

    Acad::ErrorStatus setFillColor(int colorIndex);
    int  fillColor() const;

    void setFilled(Adesk::Boolean onOff);
    Adesk::Boolean  filled() const;

    Acad::ErrorStatus setStartAngle(double ang);
    double  startAngle() const;

    Acad::ErrorStatus setEndAngle(double ang);
    double  endAngle() const;

    Acad::ErrorStatus addElement(int& index, double offset, int colorIndex, int lineTypeIndex);
    Acad::ErrorStatus removeElementAt(int elem);

    int numElements() const;

    Acad::ErrorStatus setElement(int & elem, double offset, int colorIndex,
                              int lineTypeIndex);

    Acad::ErrorStatus getElementAt(int elem, double& offset, int& colorIndex, int& lineTypeIndex) const;

    // --- AcDbObject Protocol
    virtual Acad::ErrorStatus subErase     (Adesk::Boolean pErasing
                                                     = Adesk::kTrue);

    virtual Acad::ErrorStatus dwgInFields  (AcDbDwgFiler* filer);
    virtual Acad::ErrorStatus dwgOutFields (AcDbDwgFiler* filer) const;

    virtual Acad::ErrorStatus dxfInFields  (AcDbDxfFiler* filer);
    virtual Acad::ErrorStatus dxfOutFields (AcDbDxfFiler* filer) const;

    // --- AcRxObject Protocol

    virtual void              copyFrom     (const AcRxObject* other);

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbMlineStyle);

#endif /* AD_DBMLINESTYLE_H */
