#ifndef GEOMENT_DBSOL3D_H
#define GEOMENT_DBSOL3D_H
//
// $Id: dbsol3d.h,v 1.27 1995/09/30 15:29:13 jamesr Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary
// to Autodesk,  Inc.,  and considered a trade secret as defined
// in section 499C of the penal code of the State of California.
// Use of  this information  by  anyone  other  than  authorized
// employees of Autodesk, Inc.  is granted  only under a written
// non-disclosure agreement,  expressly  prescribing  the  scope
// and manner of such use.
//
// DESCRIPTION:
//
// The AcDb3dSolid class is the interface class for representing
// solids inside AutoCAD.  All the functionality supported by
// this class is implemented in the class AcDbImp3dSolid and its
// base classes.

#include "dbmain.h"
#include "dbsubeid.h"
#include "gepnt3d.h"
#include "gevec3d.h"

class ENTITY;
class BODY;

class AcDb3dSolid: public  AcDbEntity
{
public:
    ACDB_DECLARE_MEMBERS(AcDb3dSolid);
	     AcDb3dSolid();
    virtual ~AcDb3dSolid();

    virtual Acad::ErrorStatus createBox        (double xLen,
						double yLen,
						double zLen);

    virtual Acad::ErrorStatus createFrustum    (double height,
						double xRadius,
						double yRadius,
						double topXRadius);

    virtual Acad::ErrorStatus createSphere     (double radius);

    virtual Acad::ErrorStatus createTorus      (double majorRadius,
						double minorRadius);

    virtual Acad::ErrorStatus createWedge      (double xLen,
						double yLen,
						double zLen);

    virtual Acad::ErrorStatus extrude          (const AcDbRegion* region,
						double height, double taper);

    virtual Acad::ErrorStatus extrudeAlongPath (const AcDbRegion* region,
						const AcDbCurve* path);

    virtual Acad::ErrorStatus revolve          (const AcDbRegion* region,
						const AcGePoint3d& axisPoint, 
						const AcGeVector3d& axisDir,
						double angleOfRevolution);

    virtual  Acad::ErrorStatus getArea(double& area) const;

    virtual void*             body             () const;

    virtual Acad::ErrorStatus checkInterference(const AcDb3dSolid* otherSolid,
						Adesk::Boolean createNewSolid, 
						Adesk::Boolean& solidsInterfere,
						AcDb3dSolid*& commonVolumeSolid)
					       const;

    virtual Acad::ErrorStatus getMassProp      (double& volume,
						AcGePoint3d& centroid,
						double momInertia[3],
						double prodInertia[3],
						double prinMoments[3],
						AcGeVector3d prinAxes[3], 
						double radiiGyration[3],
						AcDbExtents& extents) const;

    virtual Adesk::Boolean    isNull           () const;

    virtual void              list             () const;

    virtual Acad::ErrorStatus getSection          (const AcGePlane& plane,
						AcDbRegion*& sectionRegion)
					       const;

    virtual Acad::ErrorStatus stlOut           (const char* fileName,
						Adesk::Boolean asciiFormat)
					       const;

    virtual AcDbSubentId        internalSubentId     (void*           ent) const;
    virtual void*           internalSubentPtr    (const AcDbSubentId& id) const;

    // Subentity acquisition.
    //
    virtual  Acad::ErrorStatus getSubentPathsAtGsMarker(AcDb::SubentType type,
	int gsMark, const AcGePoint3d& pickPoint,
	const AcGeMatrix3d& viewXform, int& numPaths,
	AcDbFullSubentPath* & subentPaths, int numInserts = 0,
	AcDbObjectId* entAndInsertStack = NULL) const;

    virtual  Acad::ErrorStatus getGsMarkersAtSubentPath(
	const AcDbFullSubentPath& subPath, AcDbIntArray& gsMarkers) const;

    virtual AcDbEntity*       subentPtr(const AcDbFullSubentPath& id) const;

    virtual Acad::ErrorStatus booleanOper      (AcDb::BoolOperType operation,
						AcDb3dSolid* solid);

    virtual Acad::ErrorStatus setBody          (const void* modelerBody);

    virtual Acad::ErrorStatus getSlice            (const AcGePlane& plane,
						Adesk::Boolean getNegHalfToo, 
						AcDb3dSolid*& negHalfSolid);


    virtual Adesk::UInt32 numChanges() const;

};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDb3dSolid);

#endif
