#ifndef ACDB_DBSPLINE_H
#define ACDB_DBSPLINE_H
//
// $Id: dbspline.h,v 1.12 1995/09/30 15:29:21 jamesr Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary
// to Autodesk,  Inc.,	and considered a trade secret as defined
// in section 499C of the penal code of the State of California.
// Use of  this information  by  anyone  other	than  authorized
// employees of Autodesk, Inc.	is granted  only under a written
// non-disclosure agreement,  expressly  prescribing  the  scope
// and manner of such use.
//
// DBSPLINE.H
//
// CREATED: Keshav Sahoo                    March-3-1994
//
// DESCRIPTION: Exported class for Spline - AcDbSpline.

#include "gegbl.h"
#include "dbmain.h"
#include "dbcurve.h"


class AcDbSpline: public  AcDbCurve
{
public:
    ACDB_DECLARE_MEMBERS(AcDbSpline);

    AcDbSpline();
    virtual ~AcDbSpline();

    AcDbSpline(const AcGePoint3dArray&   points,
	       int                    order = 4, 
	       double                 fitTolerance = 0.0);

    AcDbSpline(const AcGePoint3dArray&   points,
	       const AcGeVector3d&    startTangent,
	       const AcGeVector3d&    endTangent,
	       int                    order = 4, 
	       double                 fitTolerance = 0.0);

    AcDbSpline(int                    degree, 
	       Adesk::Boolean         rational, 
	       Adesk::Boolean         closed, 
	       Adesk::Boolean         periodic,
	       const AcGePoint3dArray&   controlPoints, 
	       const AcGeDoubleArray& knots, 
	       const AcGeDoubleArray& weights,
	       double                 controlPtTol = 0.0, 
	       double                 knotTol = 0.0);

    AcDbSpline(const AcGePoint3d&  center,
	       const AcGeVector3d& unitNormal,
	       const AcGeVector3d& majorAxis,
	       double              radiusRatio,
	       double              startAngle 
			           = 0.0,
	       double              endAngle 
			           = 6.28318530717958647692);

    Adesk::Boolean          isNull() const;

    Adesk::Boolean          isRational() const;

    int                     degree() const;
    Acad::ErrorStatus       elevateDegree(int newDegree);

    int                     numControlPoints() const;
    Acad::ErrorStatus getControlPointAt(int index, AcGePoint3d& point) const;
    Acad::ErrorStatus setControlPointAt(int index, const AcGePoint3d& point);

    int                     numFitPoints() const;
    Acad::ErrorStatus getFitPointAt(int index, AcGePoint3d& point) const;
    Acad::ErrorStatus setFitPointAt(int index, const AcGePoint3d& point);

    Acad::ErrorStatus insertFitPointAt(int index, const AcGePoint3d& point);
    Acad::ErrorStatus removeFitPointAt(int index);

    double                  fitTolerance() const;
    Acad::ErrorStatus       setFitTol(double tol);

    Acad::ErrorStatus       getFitTangents(AcGeVector3d& startTangent, 
					   AcGeVector3d& endTangent) const;
    Acad::ErrorStatus       setFitTangents(const AcGeVector3d& startTangent, 
					   const AcGeVector3d& endTangent);

    Adesk::Boolean          hasFitData() const;

    Acad::ErrorStatus       getFitData(AcGePoint3dArray&     fitPoints, 
				       int&               degree, 
				       double&            fitTolerance, 
				       Adesk::Boolean&    tangentsExist,
                                       AcGeVector3d&      startTangent, 
				       AcGeVector3d&      endTangent) const;

    Acad::ErrorStatus       setFitData(const AcGePoint3dArray&  fitPoints, 
				       int                   degree, 
				       double                fitTolerance, 
                                       const AcGeVector3d&   startTangent, 
				       const AcGeVector3d&   endTangent);

    Acad::ErrorStatus       purgeFitData();
    Acad::ErrorStatus       updateFitData();

    Acad::ErrorStatus       getNurbsData(int&             degree, 
					 Adesk::Boolean&  rational, 
					 Adesk::Boolean&  closed, 
					 Adesk::Boolean&  periodic,
	                                 AcGePoint3dArray&   controlPoints, 
					 AcGeDoubleArray& knots, 
					 AcGeDoubleArray& weights,
					 double&          controlPtTol,
					 double&          knotTol) const;

    Acad::ErrorStatus       setNurbsData(int                  degree, 
					 Adesk::Boolean       rational, 
					 Adesk::Boolean       closed, 
					 Adesk::Boolean       periodic,
	                                 const AcGePoint3dArray& controlPoints, 
				         const AcGeDoubleArray& knots, 
				         const AcGeDoubleArray& weights,
					 double               controlPtTol,
					 double               knotTol);

    double weightAt(int index) const;
    Acad::ErrorStatus setWeightAt(int index, double weight);

    Acad::ErrorStatus       insertKnot(double param);

    Acad::ErrorStatus       reverseCurve();

    DBCURVE_METHODS
};
MAKE_ACDBOPENOBJECT_FUNCTION(AcDbSpline);

#endif
