#ifndef AC_GECYLNDR_H
#define AC_GECYLNDR_H
//
// $Id: gecylndr.h,v 1.5 1995/09/21 11:21:09 jamesr Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//               
// Description:
//
// This file implements the class AcGeImpCylinder, a representation for
// a right circular cylinder.  It is defined by its radius,
// axis of symmetry, and the origin (a point on the axis). It
// is generated by revolving a line parallel to the axis of symmetry,
// at a	distance of radius. The cylinder is paramterized as follows.
// u is the angle of revolution, measured from the refAxis (an axis
// perpendicular to the axis of symmetry). Right hand rule applied
// along the direction of the axis of symmetry defines the positive
// direction.  For a closed cylinder, u defaults to [-PI, PI).
// v parameter varies along the axis of symmetry.  It is positive in
// the direction of the axis of symmetry.  v = 0 correspond to the
// cross section containing the origin of the cylinder.  v is
// dimensionless.  Its speed is determined by the initial radius
// of the cylinder.
// The surface is perodic in u with a period of 2PI.
// [umin, umax] x [vmin, vmax] defines a four sided cylindrical
// patch bounded by two straight lines (at umin and umax), and
// two circular arcs (at vmin and vmax).  Following constraints
// apply to the definition of a cylindrical patch.
// umin < umax and |umin - umax| <= 2PI.
// 



#include "gegbl.h"
#include "gesurf.h"
#include "gevec3d.h"
#include "geintrvl.h"
#include "gearc3d.h"

class AcGeCircArc3d;

class AcGeCylinder : public AcGeSurface
{
public:
    AcGeCylinder ();
    AcGeCylinder (double radius, const AcGePoint3d& origin,
                  const AcGeVector3d& axisOfSymmetry);
    AcGeCylinder (double radius, const AcGePoint3d& origin,
                  const AcGeVector3d& axisOfSymmetry,
                  const AcGeVector3d& refAxis,
                  const AcGeInterval& height,
                  double startAngle, double endAngle);
    AcGeCylinder (const AcGeCylinder&);

    // Geometric properties.
    //
    double         radius            () const;
    AcGePoint3d    origin            () const;
    void           getAngles         (double& start, double& end) const;
    void           getHeight         (AcGeInterval&) const;
    double         heightAt          (double v) const;
    AcGeVector3d   axisOfSymmetry    () const;
    AcGeVector3d   refAxis           () const;
    Adesk::Boolean isClosedInU       (const AcGeTol& = 
    								   AcGeContext::gTol) const;
    Adesk::Boolean isOuterNormal     () const;
  
    AcGeCylinder&  setRadius         (double);
    AcGeCylinder&  setAngles         (double start, double end);
    AcGeCylinder&  setHeight         (const AcGeInterval&);
    AcGeCylinder&  set               (double radius, const AcGePoint3d& origin,
                                      const AcGeVector3d& axisOfSym);
    AcGeCylinder&  set               (double radius, const AcGePoint3d& origin,
                                      const AcGeVector3d& axisOfSym,
                                      const AcGeVector3d& refAxis,
                                      const AcGeInterval& height,
                                      double startAngle, double endAngle);
    // Assignment operator.
    //
    AcGeCylinder&  operator =     (const AcGeCylinder&);

    // Intersection with a linear entity
    //
    Adesk::Boolean intersectWith  (const AcGeLinearEnt3d&, int& intn, 
                                   AcGePoint3d& p1, AcGePoint3d& p2,
                                   const AcGeTol& = AcGeContext::gTol) const;

protected:

    friend class AcGeImpCylinder;

    // Construct object from its corresponding implementation object.
    AcGeCylinder (AcGeImpCylinder&, int);
    AcGeCylinder (AcGeImpCylinder*);
};

#endif
