#ifndef AC_GEELL3D_H
#define AC_GEELL3D_H
//
// $Id: geell3d.h,v 1.5 1995/12/20 15:17:07 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains the class AcGeEllipArc3d - A mathematical entity
// used to represent an ellipse arc in 3-space.

#include "gecurv3d.h"
#include "gevec3d.h"
#include "gepnt3d.h"
#include "geintrvl.h"
#include "geponc3d.h"
#include "geplane.h"

class AcGeEllipArc2d;
class AcGeCircArc3d;
class AcGeLineEnt3d;
class AcGePlanarEnt;


class AcGeEllipArc3d : public AcGeCurve3d
{
public:

    AcGeEllipArc3d();
    AcGeEllipArc3d(const AcGeEllipArc3d&);
    AcGeEllipArc3d(const AcGeCircArc3d&);
    AcGeEllipArc3d(const AcGePoint3d& cent, const AcGeVector3d& majorAxis,
                   const AcGeVector3d& minorAxis, double majorRadius,
                   double minorRadius);
    AcGeEllipArc3d(const AcGePoint3d& cent, const AcGeVector3d& majorAxis,
                   const AcGeVector3d& minorAxis, double majorRadius,
                   double minorRadius, double ang1, double ang2);
                                            
    // Return the point on this object that is closest to the other object.
    //
    AcGePoint3d    closestPointToPlane (const AcGePlanarEnt& plane,
                                        AcGePoint3d& pointOnPlane,
                                        const AcGeTol&
                                        = AcGeContext::gTol) const; 

    // Intersection with other geometric objects.
    //    
    Adesk::Boolean intersectWith (const AcGeLinearEnt3d&, int& intn,
                                  AcGePoint3d& p1, AcGePoint3d& p2,
                                  const AcGeTol& = AcGeContext::gTol) const;     
    Adesk::Boolean intersectWith (const AcGePlanarEnt&,int& numOfIntersect,
                                  AcGePoint3d& p1, AcGePoint3d& p2,
                                  const AcGeTol& = AcGeContext::gTol) const;
    
    // Projection-intersection with other geometric objects.
    //
    Adesk::Boolean projIntersectWith (const AcGeLinearEnt3d&,
                                      const AcGeVector3d& projDir, int &numInt,
                                      AcGePoint3d& pntOnArc1,
                                      AcGePoint3d& pntOnArc2,
                                      AcGePoint3d& pntOnLine1,
                                      AcGePoint3d& pntOnLine2,
                                      const AcGeTol& = AcGeContext::gTol) const;
    // Plane of the ellipse
    //
    void           getPlane      (AcGePlane&) const;

    // Inquiry Methods
    //
    Adesk::Boolean isCircular    (const AcGeTol& = AcGeContext::gTol) const;

    // Test if point is inside full ellipse
    //
    Adesk::Boolean isInside      (const AcGePoint3d&,
                                  const AcGeTol& = AcGeContext::gTol) const;

    // Definition of ellipse
    //
    AcGePoint3d    center        () const;
    double         minorRadius   () const;
    double         majorRadius   () const;
    AcGeVector3d   minorAxis     () const;
    AcGeVector3d   majorAxis     () const;
    AcGeVector3d   normal        () const;    
    double         startAng      () const;
    double         endAng        () const;
    AcGePoint3d    startPoint    () const;
    AcGePoint3d    endPoint      () const;

    AcGeEllipArc3d& setCenter     (const AcGePoint3d&);
    AcGeEllipArc3d& setMinorRadius(double);
    AcGeEllipArc3d& setMajorRadius(double);
    AcGeEllipArc3d& setAxes       (const AcGeVector3d&, const AcGeVector3d&);
    AcGeEllipArc3d& setAngles     (double startAngle, double endAngle);
    AcGeEllipArc3d& set           (const AcGePoint3d& cent,
                                   const AcGeVector3d& majorAxis,
                                   const AcGeVector3d& minorAxis,
                                   double majorRadius, double minorRadius);
    AcGeEllipArc3d& set           (const AcGePoint3d& cent,
                                   const AcGeVector3d& majorAxis,
                                   const AcGeVector3d& minorAxis,
                                   double majorRadius, double minorRadius,
                                   double ang1, double ang2);
    AcGeEllipArc3d& set           (const AcGeCircArc3d&);

    // Assignment operator.
    //
    AcGeEllipArc3d& operator =    (const AcGeEllipArc3d&);

protected:

    friend class AcGeImpEllipArc3d;

    // Construct object from its corresponding implementation object.
    AcGeEllipArc3d (AcGeImpEllipArc3d&, int);
    AcGeEllipArc3d (AcGeImpEllipArc3d*);
};

#endif
