#ifndef AC_GEENT3D_H
#define AC_GEENT3D_H
//
// $Id: geent3d.h,v 1.4 1995/10/26 16:33:57 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains the class GeEntity3d - An abstract base class
// for all Geometry Library 3d entities.


#include "gegbl.h"
#include "gepnt3d.h"
#include "geent2d.h"
#include "geintrvl.h"


class AcGeEntity3d
{
public:

    // Destructor.
    //
    ~AcGeEntity3d();

    // Run time type information.
    //
    Adesk::Boolean   isKindOf    (AcGe::EntityId) const;
    AcGe::EntityId   type        () const;

    // Make a copy of the entity.
    //
    AcGeEntity3d*    copy        () const;
    AcGeEntity3d&    operator =  (const AcGeEntity3d&);

    // Equivalence
    //
    Adesk::Boolean   operator == (const AcGeEntity3d&) const;
    Adesk::Boolean   operator != (const AcGeEntity3d&) const;
    Adesk::Boolean   isEqualTo   (const AcGeEntity3d&,
                                  const AcGeTol& = AcGeContext::gTol) const; 
                                              
    // Matrix multiplication
    //
    AcGeEntity3d&    transformBy (const AcGeMatrix3d&);
    AcGeEntity3d&    translateBy (const AcGeVector3d&);
    AcGeEntity3d&    rotateBy    (double, const AcGeVector3d&,
                                  const AcGePoint3d&
                                  = AcGePoint3d::kOrigin);       
    AcGeEntity3d&    mirror      (const AcGePlane&);
    AcGeEntity3d&    scaleBy     (double scaleFactor,
                                  const AcGePoint3d& wrtPoint
                                  = AcGePoint3d::kOrigin);
    // Point containment
    //
    Adesk::Boolean   isOn        (const AcGePoint3d&,
                                  const AcGeTol& = AcGeContext::gTol) const; 
                                         
protected:

    friend class AcGeImpEntity3d;

    AcGeImpEntity3d  *mpImpEnt;
    int              mDelEnt;

    // Private constructors so that no object of this class can be instantiated.
    AcGeEntity3d ();
    AcGeEntity3d (const AcGeEntity3d&);

    // Construct object from its corresponding implementation object.
    AcGeEntity3d (AcGeImpEntity3d&, int);
    AcGeEntity3d (AcGeImpEntity3d*);

    // Member functions that are used internally.
    AcGeEntity2d*    newEntity2d (AcGeImpEntity2d*) const;
	AcGeEntity2d*    newEntity2d (AcGeImpEntity2d&, int) const;
    AcGeEntity3d*    newEntity3d (AcGeImpEntity3d*) const;
	AcGeEntity3d*    newEntity3d (AcGeImpEntity3d&, int) const;
};


inline AcGeEntity2d*     
AcGeEntity3d::newEntity2d (AcGeImpEntity2d *impEnt ) const
{
    return new AcGeEntity2d ( impEnt );
}

inline AcGeEntity3d*     
AcGeEntity3d::newEntity3d (AcGeImpEntity3d *impEnt ) const
{
    return new AcGeEntity3d ( impEnt );
}

inline AcGeEntity3d*
AcGeEntity3d::newEntity3d(AcGeImpEntity3d& impEnt, int dummy) const
{
    return new AcGeEntity3d(impEnt, dummy);
}

inline AcGeEntity2d*
AcGeEntity3d::newEntity2d(AcGeImpEntity2d& impEnt, int dummy) const
{
    return new AcGeEntity2d(impEnt, dummy);
}


#endif
