#ifndef AC_GELENT2D_H
#define AC_GELENT2D_H
//
// $Id: gelent2d.h,v 1.3 1995/10/26 16:34:51 neb Exp $
//
// (C) Copyright 1993,1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains the class AcGeLinearEnt2d - A mathematical entity
// used to represent a line in 2-space.

#include "gecurv2d.h"
#include "gepnt2d.h"
#include "gevec2d.h"

class AcGeCircArc2d;

class AcGeLinearEnt2d : public AcGeCurve2d
{
public:

    // Intersection with other geometric objects.
    //
    Adesk::Boolean   intersectWith  (const AcGeLinearEnt2d&, AcGePoint2d&,
                                     const AcGeTol& = AcGeContext::gTol) const;
     
    // Find the overlap with other AcGeLinearEnt object
    //
    Adesk::Boolean   overlap        (const AcGeLinearEnt2d&, 
                                     AcGeLinearEnt2d*& overlap,
                                     const AcGeTol& = AcGeContext::gTol) const;

    // Direction tests.
    //
    Adesk::Boolean   isParallelTo   (const AcGeLinearEnt2d&,
                                     const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean   isPerpendicularTo(const AcGeLinearEnt2d&,
                                      const AcGeTol& = AcGeContext::gTol) const;

    // Test if two lines are colinear.
    //
    Adesk::Boolean   isColinearTo   (const AcGeLinearEnt2d&,
                                     const AcGeTol& = AcGeContext::gTol) const;

    // Perpendicular through a given point
    //
    void             getPerpLine    (const AcGePoint2d&, AcGeLine2d&) const;

    // Definition of line.
    //
    AcGePoint2d      pointOnLine    () const;
    AcGeVector2d     direction      () const;
    void             getLine        (AcGeLine2d&) const;

    // Assignment operator.
    //
    AcGeLinearEnt2d& operator =     (const AcGeLinearEnt2d&);

protected:

    friend class AcGeImpLinearEnt2d;

    // Private constructors so that no object of this class can be instantiated.
    AcGeLinearEnt2d ();
    AcGeLinearEnt2d (const AcGeLinearEnt2d&);

    // Construct object from its corresponding implementation object.
    AcGeLinearEnt2d (AcGeImpLinearEnt2d&, int);
    AcGeLinearEnt2d (AcGeImpLinearEnt2d*);

};

#endif
