#ifndef AC_GELENT3D_H
#define AC_GELENT3D_H
//
// $Id: gelent3d.h,v 1.3 1995/10/26 16:34:59 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains the class AcGeLinearEnt3d - A mathematical entity
// used to represent a line in 3-space.

#include "gecurv3d.h"
#include "gevec3d.h"
#include "gepnt3d.h"

class AcGeLine3d;
class AcGeCircArc3d;
class AcGePlanarEnt;

class AcGeLinearEnt3d : public AcGeCurve3d
{
public:

    // Intersection with other geometric objects.
    //
    Adesk::Boolean intersectWith (const AcGeLinearEnt3d&,
                                  AcGePoint3d& intPt,
                                  const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean intersectWith (const AcGePlanarEnt&, AcGePoint3d&,
                                  const AcGeTol& = AcGeContext::gTol) const;
    
    // Projection-intersection with other geometric objects.
    //
    Adesk::Boolean projIntersectWith (const AcGeLinearEnt3d&,
                                      const AcGeVector3d& projDir,
                                      AcGePoint3d& pntOnThisLine,
                                      AcGePoint3d& pntOnOtherLine,
                                      const AcGeTol& = AcGeContext::gTol) const;

    // Find the overlap with other AcGeLinearEnt object
    //
    Adesk::Boolean overlap       (const AcGeLinearEnt3d&, 
                                  AcGeLinearEnt3d*& overlap,
                                  const AcGeTol& = AcGeContext::gTol) const;
 
    // Containment tests.
    //
    Adesk::Boolean isOn          (const AcGePoint3d&,
                                  const AcGeTol& = AcGeContext::gTol) const; 
    Adesk::Boolean isOn          (const AcGePoint3d& pt, double& param,
                                  const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean isOn          (double param,
                                  const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean isOn          (const AcGePlane&,
                                  const AcGeTol& = AcGeContext::gTol) const;

    // Direction tests.
    //
    Adesk::Boolean isParallelTo  (const AcGeLinearEnt3d&,
                                  const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean isParallelTo  (const AcGePlanarEnt&,
                                  const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean isPerpendicularTo (const AcGeLinearEnt3d&,
                                      const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean isPerpendicularTo (const AcGePlanarEnt&,
                                      const AcGeTol& = AcGeContext::gTol) const;

    // Test if two lines are colinear.
    //
    Adesk::Boolean isColinearTo  (const AcGeLinearEnt3d&,
                                  const AcGeTol& = AcGeContext::gTol) const;

    // Perpendicular through a given point
    //
    void          getPerpPlane    (const AcGePoint3d&, AcGePlane&) const;

    // Definition of line.
    //
    AcGePoint3d    pointOnLine   () const;
    AcGeVector3d   direction     () const;
    void           getLine       (AcGeLine3d&) const;

    // Assignment operator.
    //
    AcGeLinearEnt3d& operator =  (const AcGeLinearEnt3d&);

protected:

    friend class AcGeImpLinearEnt3d;

    // Private constructors so that no object of this class can be instantiated.
    AcGeLinearEnt3d ();
    AcGeLinearEnt3d (const AcGeLinearEnt3d&);

    // Construct object from its corresponding implementation object.
    AcGeLinearEnt3d (AcGeImpLinearEnt3d&, int);
    AcGeLinearEnt3d (AcGeImpLinearEnt3d*);
};

#endif
