#ifndef AC_GEPLANAR_H
#define AC_GEPLANAR_H
//
// $Id: geplanar.h,v 1.6 1995/12/20 15:17:33 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains the class AcGePlanarEnt. It is an abstract class
// representing an oriented plane in 3-space.



#include "gesurf.h"
#include "gevec3d.h"

class AcGeLineSeg3d;
class AcGeLinearEnt3d;
class AcGeCircArc3d;

class AcGePlanarEnt : public AcGeSurface
{
public:

    // Intersection
    //
    Adesk::Boolean  intersectWith    (const AcGeLinearEnt3d&, AcGePoint3d&,
                                      const AcGeTol& = AcGeContext::gTol) const;

    // Closest point
    //
    AcGePoint3d     closestPointToLinearEnt (const AcGeLinearEnt3d& line,
                                             AcGePoint3d& pointOnLine,
                                             const AcGeTol&
                                               = AcGeContext::gTol) const; 
                                            
    AcGePoint3d     closestPointToPlanarEnt (const AcGePlanarEnt& otherPln, 
                                             AcGePoint3d& pointOnOtherPln,
                                             const AcGeTol&
                                               = AcGeContext::gTol) const; 

    // Direction tests.
    //
    Adesk::Boolean isParallelTo      (const AcGeLinearEnt3d&,
                                      const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean isParallelTo      (const AcGePlanarEnt&,
                                      const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean isPerpendicularTo (const AcGeLinearEnt3d&,
                                      const AcGeTol& = AcGeContext::gTol) const;
    Adesk::Boolean isPerpendicularTo (const AcGePlanarEnt&,
                                      const AcGeTol& = AcGeContext::gTol) const;

    // Point set equality.
    //
    Adesk::Boolean isCoplanarTo      (const AcGePlanarEnt&,
                                      const AcGeTol& = AcGeContext::gTol) const;

    // Get methods.
    //
    void              get            (AcGePoint3d&, AcGeVector3d& uVec,
                                      AcGeVector3d& vVec) const;
    void              get            (AcGePoint3d&, AcGePoint3d& origin, 
                                      AcGePoint3d&) const;

    // Geometric properties.
    //
    AcGePoint3d    pointOnPlane      () const;
    AcGeVector3d   normal            () const;
    void           getCoefficients(double&, double&, double&, double&) const;
    void           getCoordSystem(AcGePoint3d&, AcGeVector3d&, 
                                  AcGeVector3d&) const;
    
    // Assignment operator.
    //
    AcGePlanarEnt& operator =        (const AcGePlanarEnt&);

protected:
     
    friend class AcGeImpPlanarEnt;

    // Private constructors so that no object of this class can be instantiated.
    AcGePlanarEnt ();
    AcGePlanarEnt (const AcGePlanarEnt&);

    // Construct object from its corresponding implementation object.
    AcGePlanarEnt (AcGeImpPlanarEnt&, int);
    AcGePlanarEnt (AcGeImpPlanarEnt*);
};

#endif
