#ifndef AC_GEPONSRF_H
#define AC_GEPONSRF_H
//
// $Id: geponsrf.h,v 1.3 1995/09/15 11:11:51 janaka Exp $
//
// (C) Copyright 1993,1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
// class AcGePointOnSurface class to keep information of a point belonging
// to a surface.

#include "gepent3d.h"

class AcGeSurface;

class  AcGePointOnSurface : public AcGePointEnt3d
{
public:
    // Constructors.
    //
    AcGePointOnSurface  ();
    AcGePointOnSurface  (const AcGeSurface&);
    AcGePointOnSurface  (const AcGeSurface&, const AcGePoint2d& param);
    AcGePointOnSurface  (const AcGePointOnSurface&);

    // Assignment operator.
    //
    AcGePointOnSurface& operator =     (const AcGePointOnSurface&);

    // Query functions.
    //
    const AcGeSurface   *surface       () const;
    AcGePoint2d         parameter      () const;

    // Functions to evaluate a point.
    //
    AcGePoint3d         point          () const;
    AcGePoint3d         point          (const AcGePoint2d& param );
    AcGePoint3d         point          (const AcGeSurface&,
                                        const AcGePoint2d& param);

    // Functions to evaluate surface normal.
    //
    AcGeVector3d        normal         () const;
    AcGeVector3d        normal         (const AcGePoint2d& param );
    AcGeVector3d        normal         (const AcGeSurface&,
                                        const AcGePoint2d& param);
    // Functions to evaluate derivatives.
    //
    AcGeVector3d        uDeriv         (int order) const;
    AcGeVector3d        uDeriv         (int order, const AcGePoint2d& param);
    AcGeVector3d        uDeriv         (int order, const AcGeSurface&,
                                        const AcGePoint2d& param);

    AcGeVector3d        vDeriv         (int order) const;
    AcGeVector3d        vDeriv         (int order, const AcGePoint2d& param);
    AcGeVector3d        vDeriv         (int order, const AcGeSurface&,
                                        const AcGePoint2d& param);

    // Functions to evaluate the mixed partial.
    //
    AcGeVector3d        mixedPartial   () const;
    AcGeVector3d        mixedPartial   (const AcGePoint2d& param);
    AcGeVector3d        mixedPartial   (const AcGeSurface&,
                                        const AcGePoint2d& param);

    // Functions to compute the tangent vector in a given direction.
    //
    AcGeVector3d        tangentVector  (const AcGeVector2d&) const;
    AcGeVector3d        tangentVector  (const AcGeVector2d&,
                                        const AcGePoint2d& param);
    AcGeVector3d        tangentVector  (const AcGeVector2d&,
                                        const AcGeSurface&,
                                        const AcGePoint2d& param);

    // Functions to invert a tangent vector to parameter space.
    //
    AcGeVector2d        inverseTangentVector  (const AcGeVector3d&) const;
    AcGeVector2d        inverseTangentVector  (const AcGeVector3d&,
                                               const AcGePoint2d& param);
    AcGeVector2d        inverseTangentVector  (const AcGeVector3d&,
                                               const AcGeSurface&,
                                               const AcGePoint2d& param);

    // Set functions.  
    //
    AcGePointOnSurface& setSurface     (const AcGeSurface&);
    AcGePointOnSurface& setParameter   (const AcGePoint2d& param);

protected:

    friend class AcGeImpPointOnSurface;
};

#endif

