#ifndef AC_GESCL2D_H
#define AC_GESCL2D_H
//
// $Id: gescl2d.h,v 1.2 1995/09/21 11:23:24 jamesr Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains the class AcGeScale2d - a mathematical entity used
// to represent scaling transformations in 2-space.
// Contract:  The scale vector components must never be set to zero
// (or near zero within floating point tolerances).

#include "adesk.h"
#include "gegbl.h"

class AcGeMatrix2d;
class AcGeScale3d;

class AcGeScale2d
{
public:
    AcGeScale2d();
    AcGeScale2d(const AcGeScale2d&);
    AcGeScale2d(double factor);
    AcGeScale2d(double xFactor, double yFactor);

    // The identity scaling operation.
    //
    static const   AcGeScale2d kIdentity;

    // Multiplication.
    //
    AcGeScale2d    operator *  (const AcGeScale2d&) const;
    AcGeScale2d&   operator *= (const AcGeScale2d&);
    AcGeScale2d&   preMultBy   (const AcGeScale2d& leftSide);
    AcGeScale2d&   postMultBy  (const AcGeScale2d& rightSide);
    AcGeScale2d&   setToProduct(const AcGeScale2d&, const AcGeScale2d&);
    AcGeScale2d    operator *  (double) const;
    AcGeScale2d&   operator *= (double);
    AcGeScale2d&   setToProduct(const AcGeScale2d&, double);
    friend
    AcGeScale2d    operator *  (double, const AcGeScale2d&);

    // Multiplicative inverse.
    //
    AcGeScale2d    inverse        () const;
    AcGeScale2d&   invert         ();

    Adesk::Boolean isProportional(const AcGeTol& = AcGeContext::gTol) const;

    // Tests for equivalence using the infinity norm.
    //
    Adesk::Boolean operator == (const AcGeScale2d&) const;
    Adesk::Boolean operator != (const AcGeScale2d&) const;
    Adesk::Boolean isEqualTo   (const AcGeScale2d&,
				const AcGeTol& = AcGeContext::gTol) const;

     // For convenient access to the data.
    //
    double         operator [] (unsigned int) const;
    double&        operator [] (unsigned int);
    AcGeScale2d&   set         (double, double);

    // Conversion to/from matrix form.
    //
    operator       AcGeMatrix2d   () const;
    void           getMatrix      (AcGeMatrix2d&) const;
    AcGeScale2d&   extractScale   ( const AcGeMatrix2d& );
    AcGeScale2d&   removeScale    ( AcGeMatrix2d& );

    // Cast up to 3d scale.
    //
    operator       AcGeScale3d    () const;

    // The scale components in x and y.
    //
    double         sx, sy;
};

inline double
AcGeScale2d::operator [] (unsigned int i) const
{
    return *(&sx+i);
}

inline double&
AcGeScale2d::operator [] (unsigned int i)
{
    return *(&sx+i);
}

#endif
