#ifndef AC_GESCL3D_H
#define AC_GESCL3D_H
//
// $Id: gescl3d.h,v 1.2 1995/09/21 11:23:32 jamesr Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains the class AcGeScale3d - a mathematical entity used to
// represents scaling transformations in 3-space.
// Contract:  The scale vector components must never be set to zero
// (or near zero within floating point tolerances).

#include "adesk.h"
#include "gegbl.h"

class AcGeMatrix3d;

class AcGeScale3d
{
public:
    AcGeScale3d();
    AcGeScale3d(const AcGeScale3d&);
    AcGeScale3d(double factor);
    AcGeScale3d(double xFact, double yFact, double zFact);

    // The identity scaling operation.
    //
    static const   AcGeScale3d kIdentity;

    // Multiplication.
    //
    AcGeScale3d    operator *  (const AcGeScale3d&) const;
    AcGeScale3d&   operator *= (const AcGeScale3d&);
    AcGeScale3d&   preMultBy   (const AcGeScale3d& leftSide);
    AcGeScale3d&   postMultBy  (const AcGeScale3d& rightSide);
    AcGeScale3d&   setToProduct(const AcGeScale3d&, const AcGeScale3d&);
    AcGeScale3d    operator *  (double) const;
    AcGeScale3d&   operator *= (double);
    AcGeScale3d&   setToProduct(const AcGeScale3d&, double);
    friend
    AcGeScale3d    operator *  (double, const AcGeScale3d&);

    // Multiplicative inverse.
    //
    AcGeScale3d    inverse        () const;
    AcGeScale3d&   invert         ();

    Adesk::Boolean isProportional(const AcGeTol& = AcGeContext::gTol) const;

    // Tests for equivalence using the infinity norm.
    //
    Adesk::Boolean operator == (const AcGeScale3d&) const;
    Adesk::Boolean operator != (const AcGeScale3d&) const;
    Adesk::Boolean isEqualTo   (const AcGeScale3d&,
				const AcGeTol& = AcGeContext::gTol) const;

    // For convenient access to the data.
    //
    double         operator [] (unsigned int) const;
    double&        operator [] (unsigned int);
    AcGeScale3d&   set         (double, double, double);

    // Conversion to/from matrix form.
    //
    operator       AcGeMatrix3d   () const;
    void getMatrix(AcGeMatrix3d&) const;
    AcGeScale3d&   extractScale   ( const AcGeMatrix3d& );
    AcGeScale3d&   removeScale    ( AcGeMatrix3d& );

    // The scale components in x, y and z.
    //
    double         sx, sy, sz;
};

// Indexes the scale vector as if it were an array.  `sx' is index `0',
// `sy' is index `1' and `sz' is index `2'.
//
inline double
AcGeScale3d::operator [] (unsigned int i) const
{
    return *(&sx+i);
}

inline double&
AcGeScale3d::operator [] (unsigned int i)
{
    return *(&sx+i);
}

#endif
