#ifndef AC_GESPNT3D_H
#define AC_GESPNT3D_H
//
// $Id: gesent3d.h,v 1.4 1995/10/26 16:36:23 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains the class AcGeSplineEnt3d - A mathematical entity
// used to represent a different types of spline curves in 3-space.

#include "gecurv3d.h"
#include "gekvec.h"
#include "gept3dar.h"
#include "gevec3d.h"
#include "gepnt3d.h"

class AcGeKnotVector;
class AcGePoint3dArray;

class AcGeSplineEnt3d : public AcGeCurve3d
{
public:

    // Definition of spline
    //
    Adesk::Boolean    isRational            () const;
    int               degree                () const;
    int               order                 () const;
    int               numKnots              () const;
    const
    AcGeKnotVector&   knots                 () const;
    int               numControlPoints      () const;
    int               continuityAtKnot      (int, const AcGeTol& = 
                                             AcGeContext::gTol ) const;

    double            startParam            () const;
    double            endParam              () const;
    AcGePoint3d       startPoint            () const;
    AcGePoint3d       endPoint              () const;

    // Interpolation data
    //
    Adesk::Boolean    hasFitData            () const;

    // Editting
    //                                   
    double            knotAt                (int idx) const;
    AcGeSplineEnt3d&  setKnotAt             (int idx, double val);

    AcGePoint3d       controlPointAt        (int idx) const;
    AcGeSplineEnt3d&  setControlPointAt     (int idx, const AcGePoint3d& pnt);


    // Assignment operator.
    //
    AcGeSplineEnt3d&  operator =            (const AcGeSplineEnt3d&);

protected:

    friend class AcGeImpSplineEnt3d;

    // Private constructors so that no object of this class can be instantiated.
    AcGeSplineEnt3d ();
    AcGeSplineEnt3d (const AcGeSplineEnt3d&);

    // Construct object from its corresponding implementation object.
    AcGeSplineEnt3d (AcGeImpSplineEnt3d&, int);
    AcGeSplineEnt3d (AcGeImpSplineEnt3d*);
};

#endif

