#ifndef AC_GETOL_H
#define AC_GETOL_H

// $Id: getol.h,v 1.2 1995/10/26 16:36:46 neb Exp $

// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary  to
// Autodesk,  Inc., and considered a trade secret as defined in sec-
// tion 499C of the penal code of the State of California.   Use  of
// this information by anyone other than authorized employees of Au-
// todesk, Inc. is  granted  only  under  a  written  non-disclosure
// agreement,  expressly  prescribing  the  scope and manner of such
// use.
//
// DESCRIPTION:
//
// This file contains the functionality for managing tolerances within
// gelib.
//
// This is an instantiable class that is by default, initialized to
// the default tolerances.  Subsequently, the tolerances within it can
// be customized to suite a specific need.  For example, an instance of
// this class may be specialized to be utilized during surface intersection.

#include "gedblar.h" 

class AcGeTol {
public:
    // Constructors.
    //
    AcGeTol();

    // Inquiry functions.
    //
    double  equalPoint          () const;
    double  equalVector         () const;
   
    // Set functions.
    //
    void    setEqualPoint       ( double );
    void    setEqualVector      ( double );

private:
    AcGeDoubleArray     mTolArr;
    int                 mAbs;
};

/* Description -------------------------------------------------

1. Two points p1 and p2 are equal iff

          (p1 - p2).length() <= equalPoint

2. Two vectors v1 and v2 are equal iff  
       
          (v1 - v2).length() <= equalVector

3. Two vectors v1 and v2 are parallel iff 

         ( v1/v1.length() - v2/v2.length() ).length() < equalVector
     OR  ( v1/v1.length() + v2/v2.length() ).length() < equalVector

4. Two vectors v1 and v2 are perpendicular iff

      abs((v1.dotProduct(v2))/(v1.length()*v2.length())) <= equalVector

5. Two lines or rays are parallel ( perpendicular ) iff
   their directional vectors are parallel ( perpendicular )

6. Two lines are equal iff they have points within equlPoint
   and they are parallel.

IMPORTANT: These rules mean that two lines are close to each other as
           point sets in the part of the modeling space of diameter
           DIAM, if only the tolerance equalVector is set tighter than
           equalPoint/DIAM

---------------------------------------------------------------- */

// Inlines for AcGeTol
//
inline AcGeTol::AcGeTol() : mTolArr(2,2), mAbs(0)
{
    mTolArr.append ( 1.e-10 );
    mTolArr.append ( 1.e-10 );
}

inline void AcGeTol::setEqualVector( double val )
    { mTolArr[1] = val; }

inline double AcGeTol::equalVector() const
    { return mTolArr[1]; }

inline void AcGeTol::setEqualPoint( double val )
    { mTolArr[0] = val; }

inline double AcGeTol::equalPoint() const
    { return mTolArr[0]; }

#endif

