#ifndef _rxclass_h
#define _rxclass_h 1
//
// $Id: rxclass.h,v 1.17 1995/09/30 15:46:46 jamesr Exp $
//
// Copyright (C) 1992, 1993, 1994 by Autodesk, Inc.
//
// Permission to use, copy, modify, and distribute this software in 
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and 
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.  
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF 
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE 
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer 
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii) 
// (Rights in Technical Data and Computer Software), as applicable.

#include "rxobject.h"

class AcRxClass: public AcRxObject
{
  public:
    ACRX_DECLARE_MEMBERS(AcRxClass);

    // --- AcRxClass protocol

    virtual AcRxObject* addX(AcRxClass * protocolClass,
			       AcRxObject * protocolObject) = 0;

    virtual AcRxObject* getX(AcRxClass * protocolClass) = 0;

    virtual AcRxObject* delX(AcRxClass * protocolClass) = 0;

    virtual AcRxObject* queryX(AcRxClass * protocolClass) = 0;

    virtual AcRxObject* create() = 0;

    virtual const char* appName() const = 0;

    virtual const char* dxfName() const = 0;

    virtual const char* name() const = 0;

    virtual int         version() const = 0;

    virtual Adesk::Boolean isDerivedFrom(const AcRxClass*) const = 0;

    virtual AcRx::Ordering  comparedTo(const AcRxObject*) const = 0;
};

extern "C" AcRxClass* newAcRxClass(
    const char* className,
    const char* parentClassName,
    int         version = 0,
    AcRxObject* (*pseudoConstructor)() = NULL,
    const char* dxfName = NULL,
    const char* appName = NULL);


extern "C" void acrxBuildClassHierarchy();

#ifdef ACDB_COM_SUPPORT
extern "C" void acrxSetIidForClass ( const IID&, AcRxClass* pClass );
extern "C" AcRxClass * acrxClassForIid ( const IID& );  
extern "C" void acrxSetClsidForClass ( const CLSID&, AcRxClass* pClass );
extern "C" AcRxClass * acrxClassForClsid ( const CLSID& );
#endif

// Destruction:  DELETE CLASSES DURING APPLICATION UNLOAD ONLY !!!

extern "C" void deleteAcRxClass(AcRxClass* pClassObj);

inline Adesk::Boolean AcRxObject::isKindOf(const AcRxClass * aClass) const
{
    AcRxClass* c;
    return ((c = isA()) == NULL) ? Adesk::kTrue : c->isDerivedFrom(aClass);
}

#endif
