#ifndef _rxdict_h
#define _rxdict_h 1
//
// $Id: rxdict.h,v 1.12 1995/10/26 17:07:41 neb Exp $
//
// Copyright (C) 1992, 1993, 1994 by Autodesk, Inc.
//
// Permission to use, copy, modify, and distribute this software in 
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and 
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.  
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF 
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC.
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE 
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer 
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii) 
// (Rights in Technical Data and Computer Software), as applicable.

#include "rxobject.h"

class AcRxDictionaryIterator;

class AcRxDictionary: public AcRxObject
{
public:
    ACRX_DECLARE_MEMBERS(AcRxDictionary);

    virtual AcRxObject*    at      (const char* key) const = 0;
    virtual AcRxObject*    at      (Adesk::UInt32 id) const = 0;

    AcRxObject*            atPut   (const char* key, AcRxObject*);
    virtual AcRxObject*    atPut   (const char* key, AcRxObject*,
				    Adesk::UInt32& retId) = 0;
    virtual AcRxObject*    atPut   (Adesk::UInt32 id, AcRxObject*) = 0;

    virtual Adesk::Boolean resetKey(const char* oldKey, const char* newKey) = 0;
    virtual Adesk::Boolean resetKey(Adesk::UInt32 id, const char* newKey) = 0;

    virtual Adesk::Boolean atKeyAndIdPut(const char* newKey, Adesk::UInt32 id,
				    AcRxObject*) = 0;
									     
    virtual AcRxObject*    remove  (const char* key) = 0;
    virtual AcRxObject*    remove  (Adesk::UInt32 id) = 0;
	     
    virtual Adesk::Boolean has     (const char* entryName) const = 0;
    virtual Adesk::Boolean has     (Adesk::UInt32 id) const = 0;

    virtual Adesk::UInt32  idAt(const char* key) const = 0;

    virtual const char*    keyAt(Adesk::UInt32 id) const = 0;

    virtual Adesk::UInt32  numEntries() const = 0;

    virtual AcRxDictionaryIterator* newIterator(AcRx::DictIterType
                                    = AcRx::kDictSorted) = 0;

    virtual Adesk::Boolean deletesObjects() const = 0;
    virtual Adesk::Boolean isSorted() const = 0;
};

extern "C" AcRxDictionary* acrxSysRegistry();

inline AcRxObject* AcRxDictionary::atPut(const char* key, AcRxObject* pObj)
{ Adesk::UInt32 temp; return atPut(key, pObj, temp); }

#define acrxClassDictionary \
AcRxDictionary::cast(acrxSysRegistry()->at(ACRX_CLASS_DICTIONARY))

#define acrxServiceDictionary \
AcRxDictionary::cast(acrxSysRegistry()->at(ACRX_SERVICE_DICTIONARY))

#endif
