// $Id: miscfunk.cc,v 1.9 1995/09/30 15:41:46 jamesr Exp $
//*****************************************************************************
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary
// to Autodesk,  Inc.,  and considered a trade secret as defined
// in section 499C of the penal code of the State of California.
// Use of  this information  by  anyone  other  than  authorized
// employees of Autodesk, Inc.  is granted  only under a written
// non-disclosure agreement,  expressly  prescribing  the  scope
// and manner of such use.
//
// CREATED BY:  Anand M. Das                  April 24, 1994
//
// DESCRIPTION: Source file for miscellaneous functions that are
//              used by Rx applications.
//
//*****************************************************************************

#include "sample.h"
#include "adscodes.h"
#include "rxobject.h"
#include "dbsymtb.h"
#include "gemat3d.h"


//***********************************************************************
//***********************************************************************
//
//        The following function gets the matrix for transformations
//        needed from world coordinate system to user coordinate system.
//
//***********************************************************************
//***********************************************************************



Acad::ErrorStatus 
getWorldToUserTransform ( AcGeMatrix3d& ucsMat )
{
    struct  resbuf                 rb;
    AcGeVector3d                   xAxis, yAxis, zAxis;
    AcGePoint3d                    origin;

    // get the origin of the UCS

    if ( ads_getvar ( "UCSORG", &rb ) != RTNORM )
        goto Error;
    origin = asPnt3d ( rb.resval.rpoint );

    // get the xAxis of the UCS

    if ( ads_getvar ( "UCSXDIR", &rb ) != RTNORM )
        goto Error;
    xAxis = asVec3d ( rb.resval.rpoint );

    // get the yAxis of the UCS

    if ( ads_getvar ( "UCSYDIR", &rb ) != RTNORM )
        goto Error;
    yAxis = asVec3d ( rb.resval.rpoint );

    // get the zAxis by taking the cross product of the other axes

    zAxis = xAxis.crossProduct ( yAxis );

    // Now get the required matrix

    ucsMat.setCoordSystem ( origin, xAxis, yAxis, zAxis );

    return Acad::eOk;

Error:

    ads_printf ( "\n Severe Error in getWorldToUserTransform() \n" );

    return Acad::eUnrecoverableErrors;

}


//***********************************************************************
//***********************************************************************
//
//        The following function posts the object to the database.
//
//***********************************************************************
//***********************************************************************


Acad::ErrorStatus 
postObject ( AcDbEntity* entity, AcDbObjectId& objId )
{
    AcDbBlockTable*            blkTable;
    AcDbBlockTableRecord*      blkRecord;
    Acad::ErrorStatus          sts;

    // open the block table for read

    sts = acdbCurDwg()->getBlockTable ( blkTable, AcDb::kForRead );
    if ( sts != Acad::eOk )
        goto Error;

    // open the table record for write. At this time the object is being
    // attached to the model space

    sts = blkTable->getAt(ACDB_MODEL_SPACE, blkRecord, AcDb::kForWrite);
    if ( ( blkTable->close() != Acad::eOk ) || ( sts != Acad::eOk ) )
        goto Error;

    // append the entity to the database. Ignore the error status 
    // since the record has to be closed anyways.

    sts = blkRecord->appendAcDbEntity ( objId, entity );
    blkRecord->close();

Error:

    if ( entity->isReadEnabled() )
        entity->close();

    if ( sts != Acad::eOk )
        ads_printf ( "\n Severe Error in postObject() \n" );

    return sts ;

}





