/* Next available MSG number is   1 */
/* $Id: notfytst.cc,v 1.28 1995/10/26 17:18:21 neb Exp $ */
/*
      (C) Copyright 1982-1992,1994,1995 by Autodesk, Inc.

******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Autodesk, *
* Inc., and considered a trade secret as defined in section 499C of the      *
* penal code of the State of California.  Use of this information by anyone  *
* other than authorized employees of Autodesk, Inc. is granted only under a  *
* written non-disclosure agreement, expressly prescribing the scope and      *
* manner of such use.                                                        *
*                                                                            *
******************************************************************************/

/* ========= notfytst.cc:  Arx Notification Test ================ 

    Test App for AcDbObjectReactor,
    AcEditorReactor and AcDLinkerReactor usage, to
    obtain notification for:
    
    - Database changes.

    - editor events, specifically drawing load,
      system variable change and command invocation events.

    - temporary "program exit" notification, too.
    
    - dynamic linker events, specifically application loading and unloading.

    CREATED BY:  Bill Adkison, October 1993

*/

#include "rxobject.h"
#include "rxregsvc.h"
#include "rxdlinkr.h"
#include "rxditer.h"
#include "aced.h"
#include "dbmain.h"
#include "dbdict.h"
#include "dbidmap.h"

#include "adslib.h"

extern "C" {
// locally defined entry point invoked by Rx.
AcRx::AppRetCode acrxEntryPoint(AcRx::AppMsgCode msg, void* pkt);
}

static void printEntEvent(const AcDbObject* obj, char* eventStr,
                          const AcDbObject* obj2 = NULL);

// *** Utility routines to get yes or no answers and such.

Adesk::Boolean statOk(Acad::ErrorStatus stat)
{
    if (stat == Acad::eOk)
        return Adesk::kTrue;
    ads_printf(/*MSG0*/"\nStatus Check Failed:  %d\n", stat);
    return Adesk::kFalse;
}


static Adesk::Boolean getYorN(const char* pStr)
{
    char yorn_str[132];

    // specific prompt.
    ads_printf("\n%s", pStr);

    ads_initget(0, "No Yes");

    yorn_str[0] = 'Y';
    yorn_str[1] = '\0';

    switch (ads_getstring(Adesk::kFalse, " -- No/<Yes>:  ",
                          yorn_str)) {
    case RTKWORD:
        ads_getinput(yorn_str);
        /* Deliberate fallthrough */
    default:
	break;
    } 

    return (!((yorn_str[0] == 'N') || (yorn_str[0] == 'n')));
}

void printObj(const AcDbObject* obj)
{
    if (obj == NULL) {
        ads_printf("(NULL)");
        return;
    }

    AcDbHandle objHand;
    char  handbuf[17];

    // Getting the handle string should be easier than this...
    obj->getAcDbHandle(objHand);
    objHand.getIntoAsciiBuffer(handbuf);

    ads_printf
        ("\n   (class==%s, handle==%s, id==%lx, db==%lx)",
        obj->isA()->name(), handbuf,
	obj->objectId().asOldId(),
	obj->database());
}


static void printEntEvent(const AcDbObject* obj, char* eventStr, const AcDbObject* obj2)
{
    ads_printf("  Event: AcDbObjectReactor::%s", eventStr);
    printObj(obj);
    if (obj2 != NULL)
	printObj(obj2);
    ads_printf("\n");
}


/* =================== Commands defined in this Module ================ */

//********* AcDbObjectReactor --  Database event Notification Text


class SimpleObjReactor : public AcDbObjectReactor {

  public:
    virtual void erased(const AcDbObject* dbObj,
                              Adesk::Boolean pErasing = Adesk::kTrue);
 
    virtual void goodbye(const AcDbObject* dbObj);
 
    virtual void modified(const AcDbObject* dbObj);

  private:

    // Most Real reactors will have the address of their owner,
    // if they have one.
};

void SimpleObjReactor::modified(const AcDbObject* obj)
{
    printEntEvent(obj, "Modified");
}
 
void SimpleObjReactor::erased(const AcDbObject* obj,
                                     Adesk::Boolean pErased)
{
    if (pErased)
	printEntEvent(obj, "Erased");
    else
	printEntEvent(obj, "Unerased");
}
 
void SimpleObjReactor::goodbye(const AcDbObject* obj)
{
    printEntEvent(obj, "goodbye");
}
 
// Watch everything that happens.

class HairyEntReactor : public AcDbEntityReactor {

  public:

    virtual void cancelled(const AcDbObject* dbObj);
 
    virtual void copied(const AcDbObject* dbObj,
                        const AcDbObject* newObj);
 
    virtual void erased(const AcDbObject* dbObj,
                              Adesk::Boolean pErasing = Adesk::kTrue);
 
    virtual void goodbye(const AcDbObject* dbObj);
 
    virtual void openedForModify(const AcDbObject* dbObj);

    virtual void modified(const AcDbObject* dbObj);
 
    virtual void modifyUndone(const AcDbObject* dbObj);
 
    virtual void modifiedXData(const AcDbObject* dbObj);
 
    virtual void modifiedGraphics(const AcDbEntity* dbObj);
};


void HairyEntReactor::cancelled(const AcDbObject* obj)
{
    printEntEvent(obj, "cancelled");
}
 
void HairyEntReactor::copied(const AcDbObject* obj,
                               const AcDbObject* newObj)
{
    printEntEvent(obj, "copied from/to", newObj);
}
 
void HairyEntReactor::erased(const AcDbObject* obj,
                                     Adesk::Boolean pErased)
{
    if (pErased)
	printEntEvent(obj, "erased");
    else
	printEntEvent(obj, "unerased");
}
 
void HairyEntReactor::goodbye(const AcDbObject* obj)
{
    printEntEvent(obj, "goodbye");
}
 
void HairyEntReactor::openedForModify(const AcDbObject* obj)
{
    printEntEvent(obj, "openedForModify");
}
 
void HairyEntReactor::modified(const AcDbObject* obj)
{
    printEntEvent(obj, "modified");
}
 
void HairyEntReactor::modifyUndone(const AcDbObject* obj)
{
    printEntEvent(obj, "modifyUndone");
}
                                                          
                                                            
void HairyEntReactor::modifiedXData(const AcDbObject* obj)
{
    printEntEvent(obj, "modifiedXData");
}
 
void HairyEntReactor::modifiedGraphics(const AcDbEntity* obj)
{
    printEntEvent(obj, "modifiedGraphics");
}



SimpleObjReactor * or1 = NULL;
HairyEntReactor *  or2 = NULL;

/* "WATCH_ENT" -- Add or remove a reactor to a single selected
		   object.

*/

void watch_ent()
{
    ads_name en;
    ads_point pt, xform[4];
	struct resbuf* refstkres;
 
 
    switch (ads_nentsel("\nPick an Entity to Watch:  ", en, pt, xform, &refstkres)) {
 
    case RTNORM:   // Didn't really need this.
    			   ads_relrb(refstkres);
                   break;
    case RTNONE:   ads_alert("C'mon, I'm not gonna select it for ya!");
                   return;
    case RTCAN:    ads_alert("Coward!");
                   return;
    case RTERROR:  ads_alert("????!");
                   return;
    default:       ads_alert("Eh?");
                   return;
    }

    // Now, exchange the old-fangled ads_name for the new-fangled
    // object id...

    Acad::ErrorStatus retStat;
    AcDbObjectId eId;
    AcDbEntity* ent;

    retStat = acdbGetObjectId(eId, en);
    if ((retStat = acdbOpenAcDbEntity(ent, eId, AcDb::kForNotify))
        != Acad::eOk) {
        ads_printf("\nBad Clams for openAcDbObject: retStat==%d\n",
                   retStat);
        return;
    }

    AcDbObjectReactor* reactor;
    if (getYorN("Hairy"))
       reactor = or2;
    else
       reactor = or1;

    if (getYorN("Add Reactor")) {
        ent->addReactor(reactor);
        ads_printf("  Attached Reactor to entity.\n");
    } else {
        ent->removeReactor(reactor);
        ads_printf("  Removed Reactor from entity.\n");
    }

    ent->close();
}

/* "WATCH_OBJ" -- Add or remove a reactor to an object
		    specified by handle.  Useful for subentity 
		    and non-graphical object testing.
*/

 
void watch_obj()
{
 
 
    ads_name     objName;
    AcDbObjectId objId;
    char handleStr[17];
 
    switch (ads_getstring(Adesk::kFalse, "Enter Valid Object Handle: ",
                      handleStr)) {
        case RTNORM:
            if (ads_handent(handleStr, objName) == RTNORM) {
                if (acdbGetObjectId(objId, objName) == Acad::eOk) {
                    AcDbObject* obj;
                    statOk(acdbOpenObject(obj, objId, AcDb::kForNotify));
		    if (getYorN("Add Reactor")) {
			obj->addReactor(or2);
			ads_printf("  Attached Reactor to object.\n");
                        if (obj->isKindOf(AcDbEntity::desc()) != NULL)
                            ads_printf("  Owner Block==%l;",
                               AcDbEntity::cast(obj)->blockId());
		    } else {
			obj->removeReactor(or2);
			ads_printf("  Removed Reactor from object.\n");
		    }
		    obj->close();
                } else
		    ads_printf("  This object is already open.\n");
            }
	    return;
	case RTNONE:
	    ads_alert("Taking the easy way out, eh?");
	    return;
	case RTCAN:
	    ads_alert("Coward!");
	    return;
	case RTERROR:
	    ads_alert("????!");
	    return;
	default:
	    ads_alert("Eh?");
	    return;
    }
}

//********* AcRxDLinkerReactor --  Application Load Notification Test

class AppReactor : public AcRxDLinkerReactor {

  public:

    virtual void rxAppLoaded(const char* moduleName);

    virtual void rxAppUnloaded(const char* moduleName);

};

void AppReactor::rxAppLoaded(const char* moduleName)
{
    ads_printf("  Event:  AcRxDynamicLinkerReactor::rxAppLoaded '%s'\n",
	       moduleName);
}

void AppReactor::rxAppUnloaded(const char* moduleName)
{
    ads_printf("  Event:  AcRxDynamicLinkerReactor::rxAppUnloaded '%s'\n",
	       moduleName);
}

AppReactor * appr = NULL;

/* "WATCH_APPS" -- Add or remove a reactor to the acrxDynamicLinker system
		   object.

*/

void watch_apps()
{
    if (getYorN("Start Watching App Loads")) {
        acrxDynamicLinker->addReactor(appr);
        ads_printf("  Added Reactor to acrxDynamicLinker.\n");
    } else {
        acrxDynamicLinker->removeReactor(appr);
        ads_printf("  Removed Reactor from acrxDynamicLinker.\n");
    }
}

//********* AcEditorReactor --  System Variable Change Notification Test

void printAcEdEvent(const char * event)
{
    ads_printf("  Event:  AcEditorReactor::%s ", event);
}



class SysVarReactor : public AcEditorReactor {

  public:

    virtual void sysVarChanged(const char * varName,
                                     Adesk::Boolean success);

    virtual void sysVarWillChange(const char * varName);
};

void SysVarReactor::sysVarWillChange(const char* varName)
{
    printAcEdEvent("sysVarWillChange");
    ads_printf(" '%s'\n", varName);
}

void SysVarReactor::sysVarChanged(const char* varName,
                                        Adesk::Boolean success)
{
    printAcEdEvent("sysVarChanged");
    ads_printf(" '%s' (%ssuccessful).\n", varName, (success ? "" : "un"));
}

SysVarReactor * svr = NULL;


/* "WATCH_SYSVARS" -- Add or remove a reactor to the acedEditor system
		      object to monitor system variables.

*/

void watch_sysvars()
{
    if (getYorN("Start Watching System Variables")) {
        acedEditor->addReactor(svr);
        ads_printf("  Added Sysvar Reactor to acedEditor.\n");
    } else {
        acedEditor->removeReactor(svr);
        ads_printf("  Removed Sysvar Reactor from acedEditor.\n");
    }
}

//********* AcEditorReactor --  Command Event Notification Test


class CommandReactor : public AcEditorReactor {

  public:

    virtual void commandWillStart(const char * cmdStr);

    virtual void commandEnded(const char * cmdStr);

    virtual void commandCancelled(const char * cmdStr);

    virtual void commandFailed(const char * cmdStr);

};

void CommandReactor::commandWillStart(const char* cmdStr)
{
    printAcEdEvent("commandWillStart");
    ads_printf("  '%s'\n", cmdStr);
}

void CommandReactor::commandEnded(const char* cmdStr)
{
    printAcEdEvent("commandEnded");
    ads_printf("  '%s'\n", cmdStr);
}

void CommandReactor::commandCancelled(const char* cmdStr)
{
    printAcEdEvent("commandCancelled");
    ads_printf("  '%s'\n", cmdStr);
}

void CommandReactor::commandFailed(const char* cmdStr)
{
    printAcEdEvent("commandFailed");
    ads_printf("  '%s'\n", cmdStr);
}

CommandReactor * cmdr = NULL;


/* "WATCH_COMMANDS" -- Add or remove a reactor to the acedEditor system
		       object to monitor commands.

*/

void watch_commands()
{
    if (getYorN("Start Watching Commands")) {
        acedEditor->addReactor(cmdr);
        ads_printf("  Added Command Reactor to acedEditor.\n");
    } else {
        acedEditor->removeReactor(cmdr);
        ads_printf("  Removed Command Reactor from acedEditor.\n");
    }
}

//********* AcEditorReactor --  LISP Notification Test

class LispReactor : public AcEditorReactor {

  public:

    virtual void lispWillStart(const char * firstLine);
    virtual void lispEnded();
    virtual void lispCancelled();
};

void LispReactor::lispWillStart(const char* firstLine)
{
    printAcEdEvent("lispWillStart");
    ads_printf("  '%s'\n", firstLine);
}

void LispReactor::lispEnded()
{
    printAcEdEvent("lispEnded");
    ads_printf("\n");
}

void LispReactor::lispCancelled()
{
    printAcEdEvent("lispCancelled");
    ads_printf("\n");
}

LispReactor * lispr = NULL;


/* "WATCH_LISP" -- Add or remove a reactor to the acedEditor system
		   object to monitor LISP evaluations.

*/

void watch_lisp()
{
    if (getYorN("Start Watching LISP")) {
        acedEditor->addReactor(lispr);
        ads_printf("  Added LISP Reactor to acedEditor.\n");
    } else {
        acedEditor->removeReactor(lispr);
        ads_printf("  Removed LISP Reactor from acedEditor.\n");
    }
}

//********* AcEditorReactor --  DEEPCLONE Notification Test

class DeepCloneReactor : public AcEditorReactor {

  public:

    virtual void beginDeepClone(AcDbDatabase* pTo,
				AcDbIdMapping& idMap);
    virtual void beginDeepCloneXlation(AcDbIdMapping& idMap,
				       Acad::ErrorStatus* es);
    virtual void abortDeepClone(AcDbIdMapping& idMap);
    virtual void endDeepClone(AcDbIdMapping& idMap);


};

void DeepCloneReactor::beginDeepClone(AcDbDatabase* pTo,
				      AcDbIdMapping& idMap)
{
    printAcEdEvent("beginDeepClone");
    ads_printf(" ToDb==%lx idMap==%lx context==", (long) pTo, (long) &idMap);
    switch (idMap.deepCloneContext()) {
    case AcDb::kDcCopy:
	ads_printf("Copy Objects\n");
	break;
    case AcDb::kDcExplode:
	ads_printf("Explode Block Insert\n");
	break;
    case AcDb::kDcBlock:
	ads_printf("Create Block\n");
	break;
    case AcDb::kDcXrefBind:
	ads_printf("Xref Bind\n");
	break;
    case AcDb::kDcSymTableMerge:
	ads_printf("Symbol Table Merge\n");
	break;
    case AcDb::kDcSaveAs:
	ads_printf("SaveAs with VisRetain\n");
	break;
    case AcDb::kDcInsert:
	ads_printf("Drawing Insert\n");
	break;
    case AcDb::kDcWblock:
	ads_printf("Wblock\n");
	break;
    case AcDb::kDcObjects:
	ads_printf("Deep Clone Objects\n");
	break;
    }
}

void printIdMap(const AcDbIdMapping& idMap)
{
    AcDbIdMappingIter iter(idMap);
    AcDbIdPair idPair;

    ads_printf("\n   ((IdMap . %lx)", (long) &idMap);

    AcDbDatabase* pDb;
    if (idMap.origDb(pDb) == Acad::eOk) {
        if (pDb == acdbCurDwg())
	    ads_printf(" (OrigDb . CurDwg)");
	else
	    ads_printf(" (OrigDb . %lx)", (long) pDb);
    } else
	ads_printf(" (OrigDb . not-set)");

    idMap.destDb(pDb);   // destDb is always set during notification
    if (pDb == acdbCurDwg())
	ads_printf(" (DestDb . CurDwg))\n");
    else
	ads_printf(" (DestDb . %lx))\n", (long) pDb);

    for (iter.start(); !iter.done(); iter.next()) {
	if (!iter.getMap(idPair)) {
	    ads_printf(" (Map out-of-range)");
	    break;
	}
	if (idPair.isCloned()) 
	    ads_printf(" (%lx . %lx)", (long) idPair.key().asOldId(),
				       (long) idPair.value().asOldId());
	else
	    ads_printf(" (%lx . not-cloned)", (long) idPair.key().asOldId());
    }
    ads_printf(")\n");
}

void DeepCloneReactor::beginDeepCloneXlation(AcDbIdMapping& idMap,
					     Acad::ErrorStatus*)
{
    printAcEdEvent("beginDeepCloneXlation");
    printIdMap(idMap);
}

void DeepCloneReactor::abortDeepClone(AcDbIdMapping& idMap)
{
    printAcEdEvent("abortDeepClone");
    printIdMap(idMap);
}

void DeepCloneReactor::endDeepClone(AcDbIdMapping& idMap)
{
    printAcEdEvent("endDeepClone");
    printIdMap(idMap);
}


DeepCloneReactor * dcr = NULL;


/* "WATCH_DC" -- Add or remove a reactor to the acedEditor system
		   object to monitor DEEPCLONE.

*/

void watch_dc()
{
    if (getYorN("Start Watching DeepClone")) {
        acedEditor->addReactor(dcr);
        ads_printf("  Added DeepClone Reactor to acedEditor.\n");
    } else {
        acedEditor->removeReactor(dcr);
        ads_printf("  Removed DeepClone Reactor from acedEditor.\n");
    }
}

//********* AcDbDatabaseReactor --  Database Event Notification Test


class DbReactor : public AcDbDatabaseReactor {

  public:

    virtual void objectAppended(const AcDbDatabase* dwg,
                             const AcDbObject* dbObj);

    virtual void objectModified(const AcDbDatabase* dwg,
                                const AcDbObject* dbObj);

    virtual void objectErased(const AcDbDatabase* dwg,
                              const AcDbObject* dbObj,
                              Adesk::Boolean pErased);
};


void printDbEvent(const AcDbObject* obj, const char* eventStr)
{
    ads_printf("  Event: AcDbDatabaseReactor::%s ", eventStr);
    printObj(obj);
}

void DbReactor::objectAppended(const AcDbDatabase* db,
			       const AcDbObject*   obj)
{
    printDbEvent(obj, "objectAppended");
    ads_printf(" Db==%lx\n", (long) db);
}

void DbReactor::objectModified(const AcDbDatabase* db,
			       const AcDbObject*   obj)
{
    printDbEvent(obj, "objectModified");
    ads_printf(" Db==%lx\n", (long) db);
}

void DbReactor::objectErased(const AcDbDatabase* db,
			     const AcDbObject*   obj,
                             Adesk::Boolean      pErased)
{
    if (pErased)
	printDbEvent(obj, "objectErased");
    else
	printDbEvent(obj, "object(Un)erased");
    ads_printf(" Db==%lx\n", (long) db);
}

static Adesk::Boolean amWatchingDb;
DbReactor * dbr = NULL;


/* "WATCH_DB" -- Add or remove a reactor to the acedEditor system
		 object to monitor database events.

*/

void watch_db()
{
    if (getYorN("Start Watching the database")) {
        acdbCurDwg()->addReactor(dbr);
	amWatchingDb = Adesk::kTrue;
        ads_printf("  Added Database Reactor to acdbCurDwg().\n");
    } else {
        acdbCurDwg()->removeReactor(dbr);
	amWatchingDb = Adesk::kFalse;
        ads_printf("  Removed Database Reactor from acdbCurDwg().\n");
    }
}


//********* AcEditorReactor --  Dwg Load Notification Test


class DwgLoadReactor : public AcEditorReactor {

  public:

    virtual void beginClose(AcDbDatabase* pDwg);
 
    virtual void beginDxfIn(AcDbDatabase* newDb);
    virtual void abortDxfIn(AcDbDatabase* newDb);
    virtual void dxfInComplete(AcDbDatabase* newDb);
 
    virtual void beginDxfOut(AcDbDatabase* pDwg);
    virtual void abortDxfOut(AcDbDatabase* pDwg);
    virtual void dxfOutComplete(AcDbDatabase* pDwg);
 
    virtual void dwgFileOpened(AcDbDatabase* pDwg, char* fileName);
 
    virtual void beginSave(AcDbDatabase* pDwg, const char* pIntendedName);
    virtual void saveComplete(AcDbDatabase* pDwg, const char* pActualName);

    virtual void beginInsert(AcDbDatabase* pTo,
                             const char* pBlockName,
                             AcDbDatabase* pFrom);
    virtual void beginInsert(AcDbDatabase* pTo,
                             const AcGeMatrix3d& xform,
                             AcDbDatabase* pFrom);
    virtual void otherInsert(AcDbDatabase* pTo,
                             AcDbIdMapping& idMap,
                             AcDbDatabase* pFrom);
    virtual void abortInsert(AcDbDatabase* pTo);
    virtual void endInsert(AcDbDatabase* pTo);
 
    virtual void beginWblock(AcDbDatabase* pTo,
                        AcDbDatabase* pFrom,
                        const AcGePoint3d*& insertionPoint);
    virtual void beginWblock(AcDbDatabase* pTo,
                        AcDbDatabase* pFrom,
                        AcDbObjectId blockId);
    virtual void beginWblock(AcDbDatabase* pTo, AcDbDatabase* pFrom);
    virtual void otherWblock(AcDbDatabase* pTo,
                        AcDbIdMapping& pIdMap,
                        AcDbDatabase* pFrom);
    virtual void abortWblock(AcDbDatabase* pTo);
    virtual void endWblock(AcDbDatabase* pTo);

};


void printAcEdDbEvent(AcDbDatabase* db,
		  const char* event,
		  const char* name = NULL)
{
    printAcEdEvent(event);
    if (name == NULL)
	ads_printf(",  Db==%lx\n", (long) db);
    else
	ads_printf(",  Db==%lx,  Name==%s\n", (long) db, name);
}


void DwgLoadReactor::beginClose(AcDbDatabase* db)
{
    printAcEdDbEvent(db, "beginClose");
}

void DwgLoadReactor::dwgFileOpened(AcDbDatabase* db, char* fileName)
{
    printAcEdDbEvent(db, "dwgFileOpened");
    if (amWatchingDb) {
	db->addReactor(dbr);
	AcDbDictionary* namedObjDict;
	statOk(db->getNamedObjectsDictionary(namedObjDict, AcDb::kForNotify));
	namedObjDict->addReactor(or2);
	namedObjDict->close();
    }
}

void DwgLoadReactor::beginSave(AcDbDatabase* db, const char* intendedName)
{
    printAcEdDbEvent(db, "beginSave", intendedName);
}

void DwgLoadReactor::saveComplete(AcDbDatabase* db, const char* actualName)
{
    printAcEdDbEvent(db, "saveComplete", actualName);
}


void DwgLoadReactor::abortDxfIn(AcDbDatabase* newDb)
{
    printAcEdDbEvent(newDb, "abortDxfIn");
}


void DwgLoadReactor::beginDxfIn(AcDbDatabase* newDb)
{
    printAcEdDbEvent(newDb, "beginDxfIn");
}


void DwgLoadReactor::dxfInComplete(AcDbDatabase* newDb)
{
    printAcEdDbEvent(newDb, "dxfInComplete");
}
 

void DwgLoadReactor::abortDxfOut(AcDbDatabase* db)
{
    printAcEdDbEvent(db, "abortDxfOut");
}


void DwgLoadReactor::beginDxfOut(AcDbDatabase* db)
{
    printAcEdDbEvent(db, "beginDxfOut");
}


void DwgLoadReactor::dxfOutComplete(AcDbDatabase* db)
{
    printAcEdDbEvent(db, "dxfOutComplete");
}
 

void DwgLoadReactor::beginInsert(
  AcDbDatabase* pTo,
  const char* pBlockName,
  AcDbDatabase* pFrom)
{
    printAcEdEvent("beginInsert");
    ads_printf(",  from Db==%lx, to Db==%lx, block name\n",
	       (long) pFrom, (long) pTo, pBlockName);
}
 

void DwgLoadReactor::beginInsert(
  AcDbDatabase* pTo,
  const AcGeMatrix3d& xform,
  AcDbDatabase* pFrom)
{
    printAcEdEvent("beginInsert");
    ads_printf(", from Db==%lx, to Db==%lx, (xform)\n",
	       (long) pFrom, (long) pTo);
}
 

void DwgLoadReactor::otherInsert(
  AcDbDatabase* pTo,
  AcDbIdMapping& idMap,
  AcDbDatabase* pFrom)
{
    printAcEdEvent("otherInsert");
    ads_printf(",  from Db==%lx, to Db==%lx, (id map)\n",
	       (long) pFrom, (long) pTo);
}
 

void DwgLoadReactor::beginWblock(
  AcDbDatabase* pTo,
  AcDbDatabase* pFrom,
  const AcGePoint3d*&)
{
    printAcEdEvent("beginWblock");
    ads_printf(",  from Db==%lx, to Db==%lx, (set/insert pt)\n",
	       (long) pFrom, (long) pTo);
}
 

void DwgLoadReactor::beginWblock(
  AcDbDatabase* pTo,
  AcDbDatabase* pFrom,
  AcDbObjectId)
{
    printAcEdEvent("beginWblock");
    ads_printf(",  from Db==%lx, to Db==%lx, (block)\n",
	       (long) pFrom, (long) pTo);
}
 

void DwgLoadReactor::beginWblock(
  AcDbDatabase* pTo,
  AcDbDatabase* pFrom)
{
    printAcEdEvent("beginWblock");
    ads_printf(",  from Db==%lx, to Db==%lx\n",
	       (long) pFrom, (long) pTo);
}
 

void DwgLoadReactor::otherWblock(
  AcDbDatabase* pTo,
  AcDbIdMapping& pIdMap,
  AcDbDatabase* pFrom)
{
    printAcEdEvent("otherWblock");
    ads_printf(",  from Db==%lx, to Db==%lx (id map)\n",
	       (long) pFrom, (long) pTo);
}
 

void DwgLoadReactor::abortWblock(AcDbDatabase* pTo)
{
    printAcEdDbEvent(pTo, "abortWblock");
}
 

void DwgLoadReactor::endWblock(AcDbDatabase* pTo)
{
    printAcEdDbEvent(pTo, "endWblock");
}
 

void DwgLoadReactor::abortInsert(AcDbDatabase* pTo)
{
    printAcEdDbEvent(pTo, "abortInsert");
}
 

void DwgLoadReactor::endInsert(AcDbDatabase* pTo)
{
    printAcEdDbEvent(pTo, "endInsert");
}


DwgLoadReactor * dwgr = NULL;

/* "WATCH_DWGS" --     Add or remove a reactor to the acedEditor system
		       object to monitor drawing load/unload operations,
		       including:  OPENs,  CLOSEs (never explicit, done
				   implicitly with another OPEN, or
				   via QUIT or END), SAVEs, including
				   Auto-Save, INSERTs and WBLOCKs.
		                   XREFs watched separately, but
				   they occur as a result of any
				   of these operations.

*/

void watch_dwgs()
{
    if (getYorN("Start Watching Drawing Load/Saves")) {
        acedEditor->addReactor(dwgr);
        ads_printf("  Added Dwg Reactor to acedEditor.\n");
    } else {
        acedEditor->removeReactor(dwgr);
        ads_printf("  Removed Dwg Reactor from acedEditor.\n");
    }
}


//********* AcEditorReactor --  XREF-related events

class XrefReactor : public AcEditorReactor {

  public:

    virtual void beginAttach(AcDbDatabase* pTo,
			     const char * blockName,
    			     AcDbDatabase* pFrom);
    virtual void otherAttach(AcDbDatabase* pTo,
			     AcDbDatabase* pFrom);
    virtual void abortAttach(AcDbDatabase* pTo);
    virtual void endAttach(AcDbDatabase* pTo);
    virtual void redirected(AcDbObjectId newId, AcDbObjectId oldId);
    virtual void comandeered(AcDbDatabase* pTo, AcDbObjectId, 
			     AcDbDatabase* pFrom);

};

void XrefReactor::beginAttach(AcDbDatabase* pTo,
			      const char*   pName,
			      AcDbDatabase* pFrom)
{
    printAcEdEvent("beginAttach");
    ads_printf(",  from Db==%lx, to Db==%lx, name==%s\n",
	       (long) pFrom, (long) pTo, pName);
}

void XrefReactor::otherAttach(AcDbDatabase* pTo,
			      AcDbDatabase* pFrom)
{
    printAcEdEvent("otherAttach");
    ads_printf(",  from Db==%lx, to Db==%lx\n",
	       (long) pFrom, (long) pTo);
}

void XrefReactor::abortAttach(AcDbDatabase* pTo)
{
    printAcEdDbEvent(pTo, "abortAttach");
}

void XrefReactor::endAttach(AcDbDatabase* pTo)
{
    printAcEdDbEvent(pTo, "endAttach");
}

void XrefReactor::redirected(AcDbObjectId newId,
			     AcDbObjectId oldId)
{
    printAcEdEvent("redirected");
    ads_printf(", oldId==%lx, newId==%lx\n",
	       oldId.asOldId(), newId.asOldId());
}

void XrefReactor::comandeered(AcDbDatabase* pTo,
			      AcDbObjectId  id,
			      AcDbDatabase* pFrom)
{
    printAcEdEvent("comandeered");
    ads_printf(", from Db==%lx, to Db==%lx, id==%lx\n",
	       (long) pFrom, (long) pTo, id.asOldId());
}

XrefReactor* xrefr;


/* "WATCH_XREFS" --    Monitor all XREF actions.

*/

void watch_xrefs()
{
    if (getYorN("Start Watching XREF Resolution")) {
        acedEditor->addReactor(xrefr);
        ads_printf("  Added Xref Reactor to acedEditor.\n");
    } else {
        acedEditor->removeReactor(xrefr);
        ads_printf("  Removed Xref Reactor from acedEditor.\n");
    }
}

/* "WATCH_EVENTS" -- Add or remove all reactors defined in this
                     module to the acedEditor system
		     object to monitor every event that can
                     be monitored.  Prepare for a flood on
                     the text screen...

                     NOTE: if you have been adding reactors
			   one by one, entering this command and
			   entering "No" will clear them all.

*/

void clear_reactors()
{
        acrxDynamicLinker->removeReactor(appr);
        acedEditor->removeReactor(svr);
        acedEditor->removeReactor(cmdr);
        acedEditor->removeReactor(dwgr);
        acedEditor->removeReactor(xrefr);
        acedEditor->removeReactor(lispr);
        acedEditor->removeReactor(dcr);
	if (acdbCurDwg() != NULL)
	    acdbCurDwg()->removeReactor(dbr);
}



void watch_events()
{
    if (getYorN("Start Watching Everything")) {
        acrxDynamicLinker->addReactor(appr);
        acedEditor->addReactor(svr);
        acedEditor->addReactor(cmdr);
        acedEditor->addReactor(dwgr);
        acedEditor->addReactor(xrefr);
        acedEditor->addReactor(lispr);
        acedEditor->addReactor(dcr);
        acdbCurDwg()->addReactor(dbr);
        ads_printf("\nAdded all AcEditorReactors to acedEditor, \n");
        ads_printf("added AppReactor to acrxDynamicLinker and\n");
	ads_printf("added Db Reactor to acdbCurDwg().\n");
	amWatchingDb = Adesk::kTrue;
    } else {
	clear_reactors();
        ads_printf("\nRemoved all AcEditorReactors from acedEditor,\n");
        ads_printf("removed AppReactor from acrxDynamicLinker and\n");
	ads_printf("removed Db Reactor from acdbCurDwg().\n");
	amWatchingDb = Adesk::kFalse;
    }
}

/* =================== Arx Interface Functions ================ */

AcRx::AppRetCode acrxEntryPoint(AcRx::AppMsgCode msg, void*) {
    switch(msg) {
	case AcRx::kInitAppMsg:
	    or1    = new SimpleObjReactor();
	    or2    = new HairyEntReactor();
	    appr   = new AppReactor();
	    svr    = new SysVarReactor();
	    cmdr   = new CommandReactor();
	    lispr  = new LispReactor();
	    dcr    = new DeepCloneReactor();
	    dwgr   = new DwgLoadReactor();
	    xrefr  = new XrefReactor();
	    dbr    = new DbReactor();
	    amWatchingDb = Adesk::kFalse;

	    acedRegCmds->addCommand("NOTIFY_TEST",
				    "WATCH_ENT",
				    "WATCH_ENT",
				    ACRX_CMD_TRANSPARENT,
				    &watch_ent);

	    acedRegCmds->addCommand("NOTIFY_TEST",
				    "WATCH_OBJ",
				    "WATCH_OBJ",
				    ACRX_CMD_TRANSPARENT,
				    &watch_obj);

	    acedRegCmds->addCommand("NOTIFY_TEST",
				    "WATCH_APPS",
				    "WATCH_APPS",
				    ACRX_CMD_TRANSPARENT,
				    &watch_apps);

	    acedRegCmds->addCommand("NOTIFY_TEST",
				    "WATCH_SYSVARS",
				    "WATCH_SYSVARS",
				    ACRX_CMD_TRANSPARENT,
				    &watch_sysvars);

	    acedRegCmds->addCommand("NOTIFY_TEST",
				    "WATCH_CMDS",
				    "WATCH_CMDS",
				    ACRX_CMD_TRANSPARENT,
				    &watch_commands);

	    acedRegCmds->addCommand("NOTIFY_TEST",
				    "WATCH_DWGS",
				    "WATCH_DWGS",
				    ACRX_CMD_TRANSPARENT,
				    &watch_dwgs);

	    acedRegCmds->addCommand("NOTIFY_TEST",
				    "WATCH_LISP",
				    "WATCH_LISP",
				    ACRX_CMD_TRANSPARENT,
				    &watch_lisp);

	    acedRegCmds->addCommand("NOTIFY_TEST",
				    "WATCH_DC",
				    "WATCH_DC",
				    ACRX_CMD_TRANSPARENT,
				    &watch_dc);

	    acedRegCmds->addCommand("NOTIFY_TEST",
				    "WATCH_XREFS",
				    "WATCH_XREFS",
				    ACRX_CMD_TRANSPARENT,
				    &watch_xrefs);

	    acedRegCmds->addCommand("NOTIFY_TEST",
				    "WATCH_DB",
				    "WATCH_DB",
				    ACRX_CMD_TRANSPARENT,
				    &watch_db);

	    acedRegCmds->addCommand("NOTIFY_TEST",
				    "WATCH_EVENTS",
				    "WATCH_EVENTS",
				    ACRX_CMD_TRANSPARENT,
				    &watch_events);
	    break;
	case AcRx::kUnloadAppMsg:
	    //
	    // Pull off all the global object reactors, but
	    // beware! Either you removed or1 and or2 (of "watch_ent"
	    // and "watch_obj" commands) off of all object reactor
	    // chains, or else you'll die the next time an object
	    // they are attached to try to invoke them.
	    //
	    clear_reactors();
	    acedRegCmds->removeGroup("NOTIFY_TEST");
	    delete or1;
	    delete or2;
            delete appr;
            delete svr;
            delete cmdr;
            delete dwgr;
            delete xrefr;
            delete lispr;
            delete dcr;
            delete dbr;
        default:
	    break;
    }
    return AcRx::kRetOK;
}
