//
// $Id: main.cc,v 1.13 1995/12/20 17:04:58 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary
// to Autodesk,  Inc.,	and considered a trade secret as defined
// in section 499C of the penal code of the State of California.
// Use of  this information  by  anyone  other	than  authorized
// employees of Autodesk, Inc.	is granted  only under a written
// non-disclosure agreement,  expressly  prescribing  the  scope
// and manner of such use.
//
// MAIN.CC 
//
// CREATED BY: Keshav Sahoo                  May 1994 
//
// DESCRIPTION:
//
// This file contains the code for interfacing to AutoCAD.


#include <stdlib.h>
#include <iostream.h>
#include <string.h>

#include "assert.h"

#include "rxobject.h"
#include "rxregsvc.h"
#include "rxdlinkr.h"
#include "aced.h"

#include "adslib.h"

#include "poly.h"
#include "command.h"



extern "C" {

// locally defined entry point invoked by Rx.

AcRx::AppRetCode acrxEntryPoint(AcRx::AppMsgCode msg, void* pkt);

}

AcRxDictionary * polyCmds = NULL;

static AcDbPolyEditorReactor *pEdReactor = NULL;
AcDbPolyDBReactor            *pDBReactor = NULL;
static AcDbPolyObjReactor    *pObjReactor = NULL;



// Initialization Function

void initModule()
{
    AcDbPoly::rxInit();

    // Add some commands to test the POLY sample app.

    acedRegCmds->addCommand("POLYGON",
                            "POLY",
                            "POLY",
                            ACRX_CMD_MODAL,
                            &polyCommand);

    acedRegCmds->addCommand("POLYGON",
                            "POLYEDIT",
                            "POLYEDIT",
                            ACRX_CMD_MODAL,
                            &polyeditCommand);

    acedRegCmds->addCommand("POLYGON",
                            "TRANSTWO",
                            "TRANSTWO",
                            ACRX_CMD_MODAL,
                            &trantest2Command);

    acedRegCmds->addCommand("POLYGON",
                            "TRANSTHREE",
                            "TRANSTHREE",
                            ACRX_CMD_MODAL,
                            &trantest3Command);

    acedRegCmds->addCommand("POLYGON",
                            "TRANSACT",
                            "TRANSACT",
                            ACRX_CMD_MODAL,
                            &transactCommand);

    acedRegCmds->addCommand("POLYGON",
                            "SCANTST",
                            "SCANTST",
                            ACRX_CMD_MODAL,
                            &testOnScandr);

    acedRegCmds->addCommand("POLYGON",
                            "INSH",
                            "INSH",
                            ACRX_CMD_MODAL,
                            &insertHighlightCommand);

    acedRegCmds->addCommand("POLYGON",
                            "ADDXD",
                            "ADDXD",
                            ACRX_CMD_MODAL,
                            &addXDict);

    acedRegCmds->addCommand("POLYGON",
                            "XRECRD",
                            "XRECRD",
                            ACRX_CMD_MODAL,
                            &createXrecord);

    acedRegCmds->addCommand("POLYGON",
                            "SAVETST",
                            "SAVETST",
                            ACRX_CMD_MODAL,
                            &saveCurDwgToANewDwg);

    acedRegCmds->addCommand("POLYGON",
                            "BTRITR",
                            "BTRITR",
                            ACRX_CMD_MODAL,
                            &btrIterator);

    acedRegCmds->addCommand("POLYGON",
                            "WBTEST",
                            "WBTEST",
                            ACRX_CMD_MODAL,
                            &wblockTest);

    acedRegCmds->addCommand("POLYGON",
                            "IHIL",
                            "IHIL",
                            ACRX_CMD_MODAL,
                            &highlightBlockInsert);

    acedRegCmds->addCommand("POLYGON",
                            "VFXDICT",
                            "VFXDICT",
                            ACRX_CMD_MODAL, 
                            &verifyXDict);


    acrxBuildClassHierarchy();

    // Attach a reactor to the editor.

    pEdReactor = new AcDbPolyEditorReactor();
    pDBReactor = new AcDbPolyDBReactor();
    pObjReactor = new AcDbPolyObjReactor();

    acedEditor->addReactor(pEdReactor);

    if (acdbCurDwg() != NULL)
        acdbCurDwg()->addReactor(pDBReactor);
}



AcRx::AppRetCode acrxEntryPoint(AcRx::AppMsgCode msg, void* pkt) {
    AcDbDatabase *pDb;
    switch(msg) {
	case AcRx::kInitAppMsg:
	    initModule();
            acrxUnlockApplication(pkt);		// Try to allow unloading
	    break;
	case AcRx::kUnloadAppMsg:
            acedRegCmds->removeGroup("POLYGON");
	    delete polyCmds;

            acedEditor->removeReactor(pEdReactor);
	    delete pEdReactor;

            if (pDBReactor != NULL) {

                if ((pDb = acdbCurDwg()) != NULL)
                    pDb->removeReactor(pDBReactor);
	        delete pDBReactor;

                // There should always be a current database
                // when we get unloaded - Keshav
                //
                assert(pDb != NULL);
            }

	    // You are responsible for removing the object reactor you
	    // had planted on different objects. It's getting deleted.
	    // If not removed, some of those objects will end up with 
	    // dangling pointer for this reactor.
	    //
	    delete pObjReactor;

            deleteAcRxClass(AcDbPoly::desc());

            break;
        case AcRx::kLoadADSMsg:
            break;
        case AcRx::kUnloadADSMsg:
            break;
        case AcRx::kInvkSubrMsg:
            break;
        case AcRx::kCfgMsg:
            break;
        case AcRx::kEndMsg:
            break;
        case AcRx::kQuitMsg:
            break;
        case AcRx::kSaveMsg:
            break;
        default:
	    break;
    }
    return AcRx::kRetOK;
}

