//
// $Id: transact.cc,v 1.9 1995/12/20 17:05:20 neb Exp $
//
// (C) Copyright 1993-1995 by Autodesk, Inc.
//
// The information contained herein is confidential, proprietary
// to Autodesk,  Inc.,	and considered a trade secret as defined
// in section 499C of the penal code of the State of California.
// Use of  this information  by  anyone  other	than  authorized
// employees of Autodesk, Inc.	is granted  only under a written
// non-disclosure agreement,  expressly  prescribing  the  scope
// and manner of such use.
//
// TRANSACT.CC 
//
// CREATED BY: Keshav Sahoo		     March 1995 
//
// DESCRIPTION:
//
// This file contains code for testing the transaction model.


#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <string.h>
#include <assert.h>

#include "rxobject.h"
#include "rxregsvc.h"
#include "rxdlinkr.h"
#include "aced.h"
#include "actrans.h"

#include "adslib.h"
#include "dbents.h"
#include "dbregion.h"
#include "dbsol3d.h"
#include "dbdict.h"
#include "dbsymtb.h"

#include "gepnt3d.h"
#include "gevec3d.h"
#include "gemat3d.h"
#include "geplane.h"
#include "geassign.h"

#include "poly.h"
#include "util.h"

#include "dbsubeid.h"


// static functions used in this file.

static Acad::ErrorStatus    createAndPostPoly();
static Acad::ErrorStatus    extrudePoly(AcDbPoly* poly, double height);
static Acad::ErrorStatus    getASolid(char*                prompt, 
	                              AcTransaction*       transaction, 
	                              AcDb::OpenMode       mode,
	                              AcDb3dSolid*&        solid);

static Acad::ErrorStatus    getYOrN(char*           promptStr, 
				    Adesk::Boolean  defaultY, 
				    Adesk::Boolean& yes);


void trantest2Command()
{
    Acad::ErrorStatus es = Acad::eOk;
    ads_name ename;
    AcDbObjectId objId;
    ads_point pickpt;

    ads_entsel("\nSelect a polygon: ", ename, pickpt);
    AOK(acdbGetObjectId(objId, ename));

    // Start a transaction
    //
    AcTransaction* transOne = actrTransactionManager->startTransaction();

    ads_printf("\n\n######  Started transaction. ######\n");

    // Open the poly for write
    //

    AcDbObject *pObj = NULL;
    AcDbPoly *pPoly = NULL;
    AOK(acdbOpenObject(pObj, objId, AcDb::kForWrite));
    assert(pObj != NULL);
    pPoly = AcDbPoly::cast(pObj);
    assert(pPoly != NULL);

    AOK(transOne->getObject(pObj, objId, AcDb::kForRead));
    assert(pObj != NULL);
    pPoly = AcDbPoly::cast(pObj);
    assert(pPoly != NULL);

    pPoly = NULL;
    AOK(acdbOpenObject(pObj, objId, AcDb::kForRead));
    assert(pObj != NULL);
    pPoly = AcDbPoly::cast(pObj);
    assert(pPoly != NULL);
    pPoly->close();

    actrTransactionManager->endTransaction();
    actrTransactionManager->endTransaction();
    ads_printf("\n\n>>>>>>  Ended transaction. <<<<<<\n");

    // It should still be open for write.
    //
    pPoly->setNumSides(pPoly->numSides() + 2);
    pPoly->close();  // final close

}


void trantest3Command()
{
    Acad::ErrorStatus es = Acad::eOk;
    ads_name ename;
    AcDbObjectId objId;
    ads_point pickpt;

    ads_entsel("\nSelect a polygon: ", ename, pickpt);
    AOK(acdbGetObjectId(objId, ename));

    // Start a transaction
    //
    AcTransaction* transOne = actrTransactionManager->startTransaction();

    ads_printf("\n\n######  Started transaction. ######\n");

    // Open the poly for read twice
    //

    AcDbObject *pObj = NULL;
    AcDbPoly *pPoly = NULL;
    AOK(acdbOpenObject(pObj, objId, AcDb::kForRead));
    assert(pObj != NULL);
    pPoly = AcDbPoly::cast(pObj);
    assert(pPoly != NULL);

    pPoly = NULL;
    AOK(acdbOpenObject(pObj, objId, AcDb::kForRead));
    assert(pObj != NULL);
    pPoly = AcDbPoly::cast(pObj);
    assert(pPoly != NULL);

    AOK(transOne->getObject(pObj, objId, AcDb::kForWrite));
    assert(pObj != NULL);
    pPoly = AcDbPoly::cast(pObj);
    assert(pPoly != NULL);

    pPoly = NULL;
    AOK(acdbOpenObject(pObj, objId, AcDb::kForRead));
    assert(pObj != NULL);
    pPoly = AcDbPoly::cast(pObj);
    assert(pPoly != NULL);

    pPoly = NULL;
    AOK(acdbOpenObject(pObj, objId, AcDb::kForRead));
    assert(pObj != NULL);
    pPoly = AcDbPoly::cast(pObj);
    assert(pPoly != NULL);

    pPoly->close();
    pPoly->close();

    actrTransactionManager->endTransaction();
    actrTransactionManager->endTransaction();
    ads_printf("\n\n>>>>>>  Ended transaction. <<<<<<\n");

    // It should still be open for read twice.
    //
    ads_printf("\nNumber of sides = %d", pPoly->numSides());
    pPoly->close();  
    pPoly->close();  // final close
}


// Invoked by the command - TRANSACT
//
// Simple but (hopefully) comprehensive test of TransactionModeler.
//


void transactCommand()
{
    Acad::ErrorStatus es = Acad::eOk;

    // Create and post a poly.
    //
    ads_printf("\nCreating a poly...Please supply the required input.");
    AOK(createAndPostPoly());

    // Start a transaction
    //
    AcTransaction* transOne = actrTransactionManager->startTransaction();
    assert(transOne != NULL);
    ads_printf("\n\n######  Started transaction one. ######\n");

#if 0
    // Want to test creating inside transaction too. Should work, but
    // commented out for now.
    //
    AOK(createAndPostPoly());
#endif

    // Let's, optionally, disable graphics generation.
    // 
    Adesk::Boolean yes = Adesk::kFalse;
    if (getYOrN("\nDisable graphics generation? [N] : ", Adesk::kFalse, yes) 
				      == Acad::eOk 
				      && yes == Adesk::kTrue) {
        ads_printf("\n\n !!!! No graphics update inside transaction. !!!!\n");
	AOK(actrTransactionManager->enableGraphicsFlush(Adesk::kFalse));
    }

    // Select the poly and extrude it.

    AcDbObject	 *obj = NULL;
    AcDbPoly	 *poly = NULL;
    AcDb3dSolid  *solid = NULL;
    AcDbObjectId  objId;

    ads_name	  ename;
    ads_point	  pickpt;

    for (;;) {

        switch (ads_entsel("\nSelect a polygon: ", ename, pickpt)) {

	case RTNORM:

	    AOK(acdbGetObjectId(objId, ename));

	    AOK(transOne->getObject(obj, objId, AcDb::kForRead));
	    assert(obj != NULL);

	    poly = AcDbPoly::cast(obj);
	    if (poly == NULL) {
                ads_printf("\nNot a polygon. Try again");
		AOK(obj->close());
		continue;
	    }
	    break;

	case RTNONE:
	case RTCAN:
	    AOK(actrTransactionManager->endTransaction());
	    return;
	default:
	    continue;
	}
	break;
    }

    // The purpose of this piece of code is to just test if one
    // can use ads_command and not invoke the enforcing error
    // that gets triggerred when a command ends and non-zero
    // transactions are active.

    ads_command(RTSTR, "_globcheck", RTSHORT, 0, RTNONE);

    ads_command(RTSTR, "LINE", RTSTR, "4,4", RTSTR, "8,8", RTSTR, "", RTNONE);
    ads_name lineName;
    ads_entlast(lineName);
    ads_entdel(lineName);


    // Now that we have a poly, convert it to a region and extrude it.
    //
    ads_printf("\nWe will be extruding the poly.");
    AcGePoint3d center = poly->center();
    ads_point pt;
    pt[0] = center[0]; pt[1] = center[1]; pt[2] = center[2];
    double height;
    if (ads_getdist(pt, "\nEnter Extrusion height: ", &height) != RTNORM)
	return;

    AOK(extrudePoly(poly, height));

    // Create a  at the center of the polygon of the same height
    // as the extruded poly.
    //
    ads_printf("\nCreated a cylinder at the center of the poly.");
    double radius = (poly->startPoint() - poly->center()).length() * 0.5;

    solid = new AcDb3dSolid;
    assert(solid != NULL);
    AOK(solid->createFrustum(height, radius, radius, radius));

    AcGeMatrix3d mat;
    poly->getCoordSys(mat);
    AOK(solid->transformBy(mat));

    // Move it up again by half the height along the normal.
    //
    AcGeVector3d x(1, 0, 0), y(0, 1, 0), z(0, 0, 1);
    AcGePoint3d  moveBy(poly->normal()[0] * height * 0.5, 
                        poly->normal()[1] * height * 0.5, 
                        poly->normal()[2] * height * 0.5);
    mat.setCoordSystem(moveBy, x, y, z);
    AOK(solid->transformBy(mat));

    AOK(addToDb(solid));
    actrTransactionManager->addNewlyCreatedDBRObject(solid);
    solid->draw();

    // Start another transaction. Ask the user to select the extruded
    // solid followed by selecting the cylinder. Make a hole in the
    // extruded solid by subtracting the cylinder from it.
    //
    AcTransaction *transTwo = actrTransactionManager->startTransaction();
    assert(transTwo != NULL);
    ads_printf("\n\n######  Started transaction two. ######\n");

    AcDb3dSolid *extrusion, *cylinder;
    AOK(getASolid("\nSelect the extrsion, and I mean the extrusion: ", 
		  transTwo, AcDb::kForWrite, extrusion));
    assert(extrusion != NULL);
    AOK(getASolid("\nSelect the cylinder, and I mean the cylinder: ", 
		  transTwo, AcDb::kForWrite, cylinder));
    assert(cylinder != NULL);

    ads_printf("\nSubtracted the cylinder from the extrusion.");
    AOK(extrusion->booleanOper(AcDb::kBoolSubtract, cylinder));
    extrusion->draw();

    // At this point, cylinder is a NULL solid. We might as well erase it.
    //
    assert(cylinder->isNull());
    AOK(cylinder->erase());
    AOK(cylinder->close()); // This close is just for recording undo of erase.

    // Start another transaction and slice the resulting solid into two 
    // halves. 
    //
    AcTransaction *transThree = actrTransactionManager->startTransaction();
    assert(transThree != NULL);
    ads_printf("\n\n#####  Started transaction three. ######\n");

    AcGeVector3d vec, normal;
    vec = poly->startPoint() - poly->center();
    normal = poly->normal().crossProduct(vec);
    normal.normalize();
    AcGePlane sectionPlane(center, normal);

    int ufilerPos = cylinder->undoFiler()->tell(); // Debug

    AcDb3dSolid *otherHalf = NULL;
    ads_printf("\nSliced the resulting solid into half and moved one piece.");
    AOK(extrusion->getSlice(sectionPlane, Adesk::kTrue, otherHalf));
    assert(otherHalf != NULL);

    ufilerPos = cylinder->undoFiler()->tell(); // debug

    // Move the other half three times the vector length along the vector.
    //
    moveBy.set(vec[0] * 3.0, vec[1] * 3.0, vec[2] * 3.0);
    mat.setCoordSystem(moveBy, x, y, z);
    AOK(otherHalf->transformBy(mat));
    AcDbObjectId otherHalfId;
    ufilerPos = cylinder->undoFiler()->tell(); // Debug
    AOK(addToDb(otherHalf, otherHalfId));
    actrTransactionManager->addNewlyCreatedDBRObject(otherHalf);
    otherHalf->draw();
    ufilerPos = cylinder->undoFiler()->tell(); // Debug

    extrusion->draw();

    ufilerPos = cylinder->undoFiler()->tell(); // Debug

    // Well, the other half has been closed by now.
    //
    AOK(transThree->getObject(obj, otherHalfId, AcDb::kForRead));
    otherHalf = AcDb3dSolid::cast(obj);
    if (otherHalf == NULL) {
	ads_printf("\nNot a solid.");
	AOK(obj->close());
        INTERNAL_STATUS_ERROR(Acad::eNullObjectPointer);
    }
    otherHalf->draw();

    ufilerPos = cylinder->undoFiler()->tell(); // Debug

    // After all this work, let's abort transaction three, so that
    // we are back to the hole in the extrusion.
    //
    yes = Adesk::kTrue;
    if (getYOrN("\nLet's abort transaction three, yes? [Y] : ", 
                                      Adesk::kTrue, yes) 
				      == Acad::eOk 
				      && yes == Adesk::kTrue) {
        ads_printf("\n\n$$$$$$  Aborting transaction three. $$$$$$\n");
	AOK(actrTransactionManager->abortTransaction());
        ads_printf("\nBack to the un-sliced solid.");
    } else {
	AOK(actrTransactionManager->endTransaction());
        ads_printf("\n\n>>>>>>  Ending transaction three. <<<<<<\n");
    }

    // Start another transaction (three again). This time slice the
    // solid along a plane that is perpendicular to the plane we used
    // last time. That's the slice we really wanted.
    //
    transThree = actrTransactionManager->startTransaction();
    assert(transThree != NULL);
    ads_printf("\n\n#####  Started transaction three. ######\n");

    moveBy.set(normal[0] * 3.0, normal[1] * 3.0, normal[2] * 3.0);
    normal = vec;
    normal.normalize();
    sectionPlane.set(center, normal);

    otherHalf = NULL;
    ads_printf("\nSliced the resulting solid into half along a plane");
    ads_printf("\nperpendicular to the old one and moved one piece.");
    AOK(extrusion->getSlice(sectionPlane, Adesk::kTrue, otherHalf));
    assert(otherHalf != NULL);

    mat.setCoordSystem(moveBy, x, y, z);
    AOK(otherHalf->transformBy(mat));
    AOK(addToDb(otherHalf, otherHalfId));
    actrTransactionManager->addNewlyCreatedDBRObject(otherHalf);
    otherHalf->draw();

    extrusion->draw();

    // Well, the other half has been closed by now.
    //
    AOK(transThree->getObject(obj, otherHalfId, AcDb::kForRead));
    otherHalf = AcDb3dSolid::cast(obj);
    if (otherHalf == NULL) {
	ads_printf("\nNot a solid.");
	AOK(obj->close());
        INTERNAL_STATUS_ERROR(Acad::eNullObjectPointer);
    }
    otherHalf->draw();

    // Now, optionally, let's end all the transactions.
    //
    yes = Adesk::kFalse;
    if (getYOrN("\nAbort transaction three? [N] : ", Adesk::kFalse, yes) 
				      == Acad::eOk 
				      && yes == Adesk::kTrue) {
        ads_printf("\n\n$$$$$$  Aborting transaction three. $$$$$$\n");
	AOK(actrTransactionManager->abortTransaction());
        ads_printf("\nBack to the un-sliced solid.");
    } else {
	AOK(actrTransactionManager->endTransaction());
        ads_printf("\n\n>>>>>>  Ending transaction three. <<<<<<\n");
    }

    yes = Adesk::kFalse;
    if (getYOrN("\nAbort transaction two? [N] : ", Adesk::kFalse, yes) 
				      == Acad::eOk 
				      && yes == Adesk::kTrue) {
        ads_printf("\n\n$$$$$$  Aborting transaction two. $$$$$$\n");
	AOK(actrTransactionManager->abortTransaction());
        ads_printf("\nBack to separate extrusion and cylinder.");
    } else {
	AOK(actrTransactionManager->endTransaction());
        ads_printf("\n\n>>>>>>  Ending transaction two. <<<<<<\n");
    }

    yes = Adesk::kFalse;
    if (getYOrN("\nAbort transaction one? [N] : ", Adesk::kFalse, yes) 
				      == Acad::eOk 
				      && yes == Adesk::kTrue) {
        ads_printf("\n\n$$$$$$  Aborting transaction one. $$$$$$\n");
	AOK(actrTransactionManager->abortTransaction());
        ads_printf("\nBack to just the Poly.");
    } else {
	AOK(actrTransactionManager->endTransaction());
        ads_printf("\n\n>>>>>>  Ending transaction one. <<<<<<\n");
    }

    // This is the one that was started in the commandBegin notification
    // of the editor reactor.

    //AOK(actrTransactionManager->abortTransaction());
}


static Acad::ErrorStatus
createAndPostPoly()
{
    int nSides = 0;

    while (nSides < 3) {

        ads_initget(INP_NNEG, "");
        switch (ads_getint("\nEnter number of sides: ", &nSides)) {

	case RTNORM:
	    if (nSides < 3)
               ads_printf("\nNeed at least 3 sides.");
	    break;
	default:
	    return Acad::eInvalidInput;
	}
    }

    ads_point center, startPt, normal;

    if (ads_getpoint(NULL, "\nLocate center of polygon: ", center) 
						       != RTNORM)
	return Acad::eInvalidInput;

    startPt[0] = center[0]; startPt[1] = center[1]; startPt[2] = center[2];
    while (asPnt3d(startPt) == asPnt3d(center)) {
        switch (ads_getpoint(center, "\nLocate start point of polygon: ", 
							    startPt)) {
	    case RTNORM:
		if (asPnt3d(center) == asPnt3d(startPt))
                    ads_printf("\nPick a point different from the center.");
		break;
	    default:
		return Acad::eInvalidInput;
	}
    }


    // Set the normal to the plane of the polygon to be the same as the
    // z direction of the current ucs, i.e. (0, 0, 1) since we also got the
    // center and start point in the current UCS. (ads_getpoint() returns in
    // the current UCS.)

    normal[X] = 0.0; normal[Y] = 0.0; normal[Z] = 1.0;

    rx_uc2wc(center, center, Adesk::kFalse);
    rx_uc2wc(startPt, startPt, Adesk::kFalse);
    rx_uc2wc(normal, normal, Adesk::kTrue);

    AcGePoint3d cen = asPnt3d(center), start = asPnt3d(startPt);
    AcGeVector3d norm = asVec3d(normal);

    AcDbPoly* poly = new AcDbPoly(cen, start, nSides, norm, "transactPoly");

    poly->setDatabaseDefaults(acdbCurDwg());
    //AOK(addToDb(poly));
    AOK(postToDb(poly));
    //actrTransactionManager->addNewlyCreatedDBRObject(poly);
    //poly->draw();

    return Acad::eOk;
}



// Extrudes the poly to a given height.

static Acad::ErrorStatus extrudePoly(AcDbPoly* poly, double height)
{
    Acad::ErrorStatus es = Acad::eOk;

    // Explode to a set of lines
    //
    AcDbVoidPtrArray lines;
    AOK(poly->explode(lines));

    // Create a region from the set of lines.
    //
    AcDbVoidPtrArray regions;
    AOK(AcDbRegion::createFromCurves(lines, regions));
    assert(regions.length() == 1);
    AcDbRegion* region = AcDbRegion::cast((AcRxObject*)regions[0]);
    assert(region != NULL);

    // Extrude the region to create a solid.
    //
    AcDb3dSolid* solid = new AcDb3dSolid;
    assert(solid != NULL);
    AOK(solid->extrude(region, height, 0.0));

    for (int i = 0; i < lines.length(); i++) {
	delete (AcRxObject*)lines[i];
    }
    for (i = 0; i < regions.length(); i++) {
	delete (AcRxObject*)regions[i];
    }

    // What's with the memory associated with this region??? TODO.

    // Now we have a solid. Post it to database.

    AOK(solid->setPropertiesFrom(poly));
    AOK(addToDb(solid));
    actrTransactionManager->addNewlyCreatedDBRObject(solid);
    solid->draw();

    return Acad::eOk;
}


static Acad::ErrorStatus
getASolid(char*                prompt, 
	  AcTransaction*       transaction, 
	  AcDb::OpenMode       mode,
	  AcDb3dSolid*&        solid)
{
    AcDbObject	 *obj = NULL;
    AcDbObjectId  objId;

    ads_name	  ename;
    ads_point	  pickpt;

    for (;;) {

        switch (ads_entsel(prompt, ename, pickpt)) {

	case RTNORM:

	    AOK(acdbGetObjectId(objId, ename));

	    AOK(transaction->getObject(obj, objId, mode));
	    assert(obj != NULL);

	    solid = AcDb3dSolid::cast(obj);
	    if (solid == NULL) {
                ads_printf("\nNot a solid. Try again");
		AOK(obj->close());
		continue;
	    }
	    break;

	case RTNONE:
	case RTCAN:
	    return Acad::eInvalidInput;
	default:
	    continue;
	}
	break;
    }

    return Acad::eOk;
}


static Acad::ErrorStatus
getYOrN(char* promptStr, Adesk::Boolean defaultY, Adesk::Boolean& yes)
{
    char option[256];
    yes = Adesk::kTrue;

    ads_initget(0, "Yes No");
    int as = ads_getkword(promptStr, option);

    switch (as) {
    case RTNORM:
        if (strcmp(option, "Yes") == 0)
	    yes = Adesk::kTrue;
	else
	    yes = Adesk::kFalse;
	break;

    case RTNONE:
        if (defaultY)
	    yes = Adesk::kTrue;
	else
	    yes = Adesk::kFalse;
	break;

    case RTCAN:
	break;
    default:
	break;
    }

    return Acad::eOk;
}
